@@ -21,6 +21,7 @@ class Configure(config.package.Package):
     config.package.Package.setupDependencies(self, framework)
     self.f2cblaslapack = framework.require('config.packages.f2cblaslapack', self)
     self.fblaslapack   = framework.require('config.packages.fblaslapack', self)
+    self.blis          = framework.require('config.packages.blis', self)
     self.openblas      = framework.require('config.packages.openblas', self)
     self.flibs         = framework.require('config.packages.flibs',self)
     self.mathlib       = framework.require('config.packages.mathlib',self)
@@ -81,15 +82,9 @@ class Configure(config.package.Package):
   def checkBlas(self, blasLibrary, otherLibs, fortranMangle, routineIn = 'dot'):
     '''This checks the given library for the routine, dot by default'''
     oldLibs = self.compilers.LIBS
-    prototype = ''
-    call      = ''
     routine   = self.mangleBlas(routineIn)
-    if fortranMangle=='stdcall':
-      if routine=='ddot'+self.suffix:
-        prototype = 'double __stdcall DDOT(int*,double*,int*,double*,int*);'
-        call      = 'DDOT(0,0,0,0,0);'
     self.libraries.saveLog()
-    found   = self.libraries.check(blasLibrary, routine, otherLibs = otherLibs, fortranMangle = fortranMangle, prototype = prototype, call = call)
+    found   = self.libraries.check(blasLibrary, routine, otherLibs = otherLibs, fortranMangle = fortranMangle)
     self.logWrite(self.libraries.restoreLog())
     self.compilers.LIBS = oldLibs
     return found
@@ -99,19 +94,11 @@ class Configure(config.package.Package):
     if not isinstance(routinesIn, list): routinesIn = [routinesIn]
     routines = list(routinesIn)
     found   = 1
-    prototypes = ['']
-    calls      = ['']
     routines   = map(self.mangleBlas, routines)
 
-    if fortranMangle=='stdcall':
-      if routines == ['dgetrs','dgeev']:
-        prototypes = ['void __stdcall DGETRS(char*,int,int*,int*,double*,int*,int*,double*,int*,int*);',
-                      'void __stdcall DGEEV(char*,int,char*,int,int*,double*,int*,double*,double*,double*,int*,double*,int*,double*,int*,int*);']
-        calls      = ['DGETRS(0,0,0,0,0,0,0,0,0,0);',
-                      'DGEEV(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);']
-    for routine, prototype, call in zip(routines, prototypes, calls):
+    for routine in routines:
       self.libraries.saveLog()
-      found = found and self.libraries.check(lapackLibrary, routine, otherLibs = otherLibs, fortranMangle = fortranMangle, prototype = prototype, call = call)
+      found = found and self.libraries.check(lapackLibrary, routine, otherLibs = otherLibs, fortranMangle = fortranMangle)
       self.logWrite(self.libraries.restoreLog())
       if not found: break
     self.compilers.LIBS = oldLibs
@@ -120,8 +107,6 @@ class Configure(config.package.Package):
   def checkLib(self, lapackLibrary, blasLibrary = None):
     '''Checking for BLAS and LAPACK symbols'''
 
-    #check for BLASLAPACK_STDCALL calling convention!!!!
-
     if blasLibrary is None:
       self.separateBlas = 0
       blasLibrary       = lapackLibrary
@@ -175,6 +160,9 @@ class Configure(config.package.Package):
       # TODO: use self.f2cblaslapack.libDir directly
       libDir = os.path.join(self.f2cblaslapack.directory,'lib')
       f2cBlas = [os.path.join(libDir,'libf2cblas.a')]
+      if self.blis.found:
+        # The real BLAS is provided by libblis, but we still need libf2cblas for aux functions needed by libf2clapack
+        f2cBlas += self.blis.lib
       f2cLapack = [os.path.join(libDir,'libf2clapack.a')]
       yield ('f2cblaslapack', f2cBlas, f2cLapack, '32','no')
       yield ('f2cblaslapack', f2cBlas+['-lquadmath'], f2cLapack, '32','no')
@@ -185,6 +173,10 @@ class Configure(config.package.Package):
       libDir = os.path.join(self.fblaslapack.directory,'lib')
       yield ('fblaslapack', os.path.join(libDir,'libfblas.a'), os.path.join(libDir,'libflapack.a'), '32','no')
       raise RuntimeError('--download-fblaslapack libraries cannot be used')
+    if self.blis.found:
+      self.f2c = 0
+      # TODO: Where shall we find liblapack.a?
+      yield ('BLIS', self.blis.lib, 'liblapack.a', self.blis.known64, self.blis.usesopenmp)
     if self.openblas.found:
       self.f2c = 0
       if self.openblas.libDir:
@@ -220,14 +212,14 @@ class Configure(config.package.Package):
       mkl = os.getenv('MKLROOT')
       if mkl:
         # Since user did not select MKL specifically first try compiler defaults and only if they fail use the MKL
-        yield ('Default compiler libraries', '', '','unknown','unknow')
-        yield ('Default compiler locations', 'libblas.a', 'liblapack.a','unknown','unknow')
-        yield ('Default compiler locations /usr/local/lib', os.path.join('/usr','local','lib','libblas.a'), os.path.join('/usr','local','lib','liblapack.a'),'unknown','unknow')
-        yield ('OpenBLAS default compiler locations', None, 'libopenblas.a','unknown','unknow')
-        yield ('OpenBLAS default compiler locations /usr/local/lib', None, os.path.join('/usr','local','lib','libopenblas.a'),'unknown','unknow')
-        yield ('Default compiler locations with gfortran', None, ['liblapack.a', 'libblas.a','libgfortran.a'],'unknown','unknow')
-        yield ('Default compiler locations', 'libblis.a', 'liblapack.a','unknown','unknown')
-        yield ('Default compiler locations /usr/local/lib', os.path.join('/usr','local','lib','libblis.a'), os.path.join('/usr','local','lib','liblapack.a'),'unknown','unknown')
+        yield ('Default compiler libraries', '', '','unknown','unknown')
+        yield ('BLIS default compiler locations', 'libblis.a', 'liblapack.a','unknown','unknown')
+        yield ('BLIS default compiler locations /usr/local/lib', os.path.join('/usr','local','lib','libblis.a'), os.path.join('/usr','local','lib','liblapack.a'),'unknown','unknown')
+        yield ('OpenBLAS default compiler locations', None, 'libopenblas.a','unknown','unknown')
+        yield ('OpenBLAS default compiler locations /usr/local/lib', None, os.path.join('/usr','local','lib','libopenblas.a'),'unknown','unknown')
+        yield ('Default compiler locations', 'libblas.a', 'liblapack.a','unknown','unknown')
+        yield ('Default compiler locations /usr/local/lib', os.path.join('/usr','local','lib','libblas.a'), os.path.join('/usr','local','lib','liblapack.a'),'unknown','unknown')
+        yield ('Default compiler locations with gfortran', None, ['liblapack.a', 'libblas.a','libgfortran.a'],'unknown','unknown')
         self.logWrite('Did not detect default BLAS and LAPACK locations so using the value of MKLROOT to search as --with-blas-lapack-dir='+mkl)
         self.argDB['with-blaslapack-dir'] = mkl
 
@@ -366,6 +358,8 @@ class Configure(config.package.Package):
       yield ('User specified AMD ACML lib dir', None, [os.path.join(dir,'lib','libacml.a'), os.path.join(dir,'lib','libacml_mv.a')],'32','unknown')
       yield ('User specified AMD ACML lib dir', None, os.path.join(dir,'lib','libacml_mp.a'),'32','unknown')
       yield ('User specified AMD ACML lib dir', None, [os.path.join(dir,'lib','libacml_mp.a'), os.path.join(dir,'lib','libacml_mv.a')],'32','unknown')
+      # BLIS
+      yield ('User specified installation root BLIS/LAPACK', os.path.join(dir, 'libblis.a'), os.path.join(dir, 'liblapack.a'), 'unknown', 'unknown')
       # Search for OpenBLAS
       yield ('User specified OpenBLAS', None, os.path.join(dir, 'libopenblas.a'),'unknown','unknown')
       # Search for atlas
@@ -379,8 +373,7 @@ class Configure(config.package.Package):
         yield ('User specified installation root IBM ESSL', None, os.path.join(dir, lib, 'libessl.a'),'32','unknown')
       # Search for liblapack.a and libblas.a after the implementations with more specific name to avoid
       # finding these in /usr/lib despite using -L<blaslapack-dir> while attempting to get a different library.
-      yield ('User specified installation root BLAS/LAPACK', os.path.join(dir, 'libblas.a'),    os.path.join(dir, 'liblapack.a'),'unknown','unknow')
-      yield ('User specified installation root BLIS/LAPACK', os.path.join(dir, 'libblis.a'),    os.path.join(dir, 'liblapack.a'),'unknown','unknown')
+      yield ('User specified installation root BLAS/LAPACK', os.path.join(dir, 'libblas.a'),    os.path.join(dir, 'liblapack.a'),'unknown','unknown')
       raise RuntimeError('You set a value for --with-blaslapack-dir=<dir>, but '+self.argDB['with-blaslapack-dir']+' cannot be used\n')
     if self.defaultPrecision == '__float128':
       raise RuntimeError('__float128 precision requires f2c libraries; suggest --download-f2cblaslapack\n')
@@ -388,6 +381,7 @@ class Configure(config.package.Package):
 
     # Try compiler defaults
     yield ('Default compiler libraries', '', '','unknown','unknown')
+    yield ('Default BLIS', 'libblis.a', 'liblapack.a','unknown','unknown')
     yield ('Default compiler locations', 'libblas.a', 'liblapack.a','unknown','unknown')
     yield ('Default OpenBLAS', None, 'libopenblas.a','unknown','unknown')
     # Intel on Mac
@@ -495,8 +489,6 @@ class Configure(config.package.Package):
         self.addDefine('BLASLAPACK_UNDERSCORE', 1)
     elif self.mangling == 'caps':
         self.addDefine('BLASLAPACK_CAPS', 1)
-    elif self.mangling == 'stdcall':
-        self.addDefine('BLASLAPACK_STDCALL', 1)
 
     if self.suffix != '':
         self.addDefine('BLASLAPACK_SUFFIX', self.suffix)
@@ -598,10 +590,8 @@ class Configure(config.package.Package):
     '''Check for missing LAPACK routines'''
     if self.foundLapack:
       mangleFunc = hasattr(self.compilers, 'FC') and not self.f2c
-    routines = ['gels','gelss','geqrf','gerfs','gesv','gesvd','getrf','getri','gges',
-                'hgeqz','hseqr','ormqr','potrf','potri','potrs','pttrf','pttrs',
-                'stebz','stein','steqr','syev','syevx','sygvx','sytrf','sytri','sytrs',
-                'tgsen','trsen','trtrs','orgqr']  # skip these: 'hetrf','hetri','hetrs',
+    routines = ['gelss','gerfs','gges','hgeqz','hseqr','orgqr','ormqr','stebz',
+                'stegr','stein','steqr','sytri','tgsen','trsen','trtrs']
     self.libraries.saveLog()
     oldLibs = self.compilers.LIBS
     found, missing = self.libraries.checkClassify(self.lapackLibrary, map(self.mangleBlas,routines), otherLibs = self.getOtherLibs(), fortranMangle = mangleFunc)
