@@ -4,11 +4,11 @@ import os
 class Configure(config.package.GNUPackage):
   def __init__(self, framework):
     config.package.GNUPackage.__init__(self, framework)
-    self.download          = ['http://www.open-mpi.org/software/hwloc/v1.10/downloads/hwloc-1.10.0.tar.gz']
+    self.download          = ['http://www.open-mpi.org/software/hwloc/v1.11/downloads/hwloc-2.1.0.tar.gz',
+                              'http://ftp.mcs.anl.gov/pub/petsc/externalpackages/hwloc-2.1.0.tar.gz']
     self.functions         = ['hwloc_topology_init']
     self.includes          = ['hwloc.h']
     self.liblist           = [['libhwloc.a'],['libhwloc.a','libxml2.a']]
-    self.lookforbydefault  = 1
     self.downloadonWindows = 1
     return
 
