@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import print_function
 import os, sys
 
 extraLogs = []
@@ -8,19 +9,14 @@ petsc_arch = ''
 if 'LC_LOCAL' in os.environ and os.environ['LC_LOCAL'] != '' and os.environ['LC_LOCAL'] != 'en_US' and os.environ['LC_LOCAL']!= 'en_US.UTF-8': os.environ['LC_LOCAL'] = 'en_US.UTF-8'
 if 'LANG' in os.environ and os.environ['LANG'] != '' and os.environ['LANG'] != 'en_US' and os.environ['LANG'] != 'en_US.UTF-8': os.environ['LANG'] = 'en_US.UTF-8'
 
-if not hasattr(sys, 'version_info') or not sys.version_info[0] == 2 or not sys.version_info[1] >= 6:
-  print '*******************************************************************************'
-  print '*       Python2 version 2.6 or higher is required to run ./configure          *'
-  print '*          Try: "python2.7 ./configure" or "python2.6 ./configure"            *'
-  print '*******************************************************************************'
-  sys.exit(4)
-
 def check_for_option_mistakes(opts):
   for opt in opts[1:]:
     name = opt.split('=')[0]
+    if name.find(' ') >= 0:
+      raise ValueError('The option "'+name+'" has a space character in the name - this is likely incorrect usage.');
     if name.find('_') >= 0:
       exception = False
-      for exc in ['mkl_cpardiso', 'mkl_pardiso', 'superlu_dist', 'superlu_mt', 'PETSC_ARCH', 'PETSC_DIR', 'CXX_CXXFLAGS', 'LD_SHARED', 'CC_LINKER_FLAGS', 'CXX_LINKER_FLAGS', 'FC_LINKER_FLAGS', 'AR_FLAGS', 'C_VERSION', 'CXX_VERSION', 'FC_VERSION', 'size_t', 'MPI_Comm','MPI_Fint','int64_t']:
+      for exc in ['mkl_sparse', 'mkl_sparse_optimize', 'mkl_cpardiso', 'mkl_pardiso', 'superlu_dist', 'PETSC_ARCH', 'PETSC_DIR', 'CXX_CXXFLAGS', 'LD_SHARED', 'CC_LINKER_FLAGS', 'CXX_LINKER_FLAGS', 'FC_LINKER_FLAGS', 'AR_FLAGS', 'C_VERSION', 'CXX_VERSION', 'FC_VERSION', 'size_t', 'MPI_Comm','MPI_Fint','int64_t']:
         if name.find(exc) >= 0:
           exception = True
       if not exception:
@@ -29,6 +25,9 @@ def check_for_option_mistakes(opts):
       optval = opt.split('=')[1]
       if optval == 'ifneeded':
         raise ValueError('The option '+opt+' should probably be '+opt.replace('ifneeded', '1'));
+    for exc in ['mkl_sparse', 'mkl_sparse_optimize', 'mkl_cpardiso', 'mkl_pardiso', 'superlu_dist']:
+      if name.find(exc.replace('_','-')) > -1:
+        raise ValueError('The option '+opt+' should be '+opt.replace(exc.replace('_','-'),exc));
   return
 
 def check_for_unsupported_combinations(opts):
@@ -37,7 +36,19 @@ def check_for_unsupported_combinations(opts):
 
 def check_for_option_changed(opts):
 # Document changes in command line options here.
-  optMap = [('with-64bit-indices','with-64-bit-indices'),('c-blas-lapack','f2cblaslapack'),('cholmod','suitesparse'),('umfpack','suitesparse'),('f-blas-lapack','fblaslapack')]
+  optMap = [('with-64bit-indices','with-64-bit-indices'),
+            ('with-mpi-exec','with-mpiexec'),
+            ('c-blas-lapack','f2cblaslapack'),
+            ('cholmod','suitesparse'),
+            ('umfpack','suitesparse'),
+            ('f-blas-lapack','fblaslapack'),
+            ('with-cuda-arch',
+             'CUDAFLAGS=-arch'),
+            ('with-packages-dir','with-packages-download-dir'),
+            ('with-external-packages-dir','with-packages-build-dir'),
+            ('package-dirs','with-packages-search-path'),
+            ('download-petsc4py-python','with-python-exec'),
+            ('search-dirs','with-executables-search-path')]
   for opt in opts[1:]:
     optname = opt.split('=')[0].strip('-')
     for oldname,newname in optMap:
@@ -66,25 +77,15 @@ def check_petsc_arch(opts):
 def chkenable():
   #Replace all 'enable-'/'disable-' with 'with-'=0/1/tail
   #enable-fortran is a special case, the resulting --with-fortran is ambiguous.
-  #Would it mean --with-fc= or --with-fortran-interfaces=?
+  #Would it mean --with-fc=
+  en_dash = u'\N{EN DASH}'
+  if sys.version_info < (3, 0):
+    en_dash = en_dash.encode('utf-8')
   for l in range(0,len(sys.argv)):
     name = sys.argv[l]
-    if name.find('enable-fortran') >= 0:
-      if name.find('=') == -1:
-        sys.argv[l] = name.replace('enable-fortran','with-fortran-interfaces')+'=1'
-      else:
-        head, tail = name.split('=', 1)
-        sys.argv[l] = head.replace('enable-fortran','with-fortran-interfaces')+'='+tail
-      continue
-    if name.find('disable-fortran') >= 0:
-      if name.find('=') == -1:
-        sys.argv[l] = name.replace('disable-fortran','with-fortran-interfaces')+'=0'
-      else:
-        head, tail = name.split('=', 1)
-        if tail == '1': tail = '0'
-        sys.argv[l] = head.replace('disable-fortran','with-fortran-interfaces')+'='+tail
-      continue
 
+    if name.find(en_dash)  >= 0:
+      sys.argv[l] = name.replace(en_dash,'-')
     if name.find('enable-cxx') >= 0:
       if name.find('=') == -1:
         sys.argv[l] = name.replace('enable-cxx','with-clanguage=C++')
@@ -128,30 +129,14 @@ def chkenable():
         if tail == '1': tail = '0'
         sys.argv[l] = head.replace('without-','with-')+'='+tail
 
-def argsAddDownload(value,deps = [],options = []):
-  # Adds --download-value to args if the command line DOES NOT already has --with-value or --download-value in it
-  # this is to prevent introducing conflicting arguments to ones that already exist
-  for opt in sys.argv[1:]:
-    optname = opt.split('=')[0].strip('-')
-    if optname in ['download-'+value,'with-'+value,'with-'+value+'-dir','with-'+value+'-include','with-'+value+'-lib']: return
-  sys.argv.append('--download-'+value)
-  for i in deps:
-    argsAddDownload(i)
-  for i in options:
-    sys.argv.append(i)
-
 def chksynonyms():
   #replace common configure options with ones that PETSc BuildSystem recognizes
-  downloadxsdk = 0
-  downloadideas = 0
+  simplereplacements = {'F77' : 'FC', 'F90' : 'FC'}
   for l in range(0,len(sys.argv)):
     name = sys.argv[l]
 
-    if name.find('download-xsdk=') >= 0 or name.endswith('download-xsdk'):
-      downloadxsdk = 1
-
-    if name.find('download-ideas=') >= 0 or name.endswith('download-ideas'):
-      downloadideas = 1
+    if name.find('with-blas-lapack') >= 0:
+      sys.argv[l] = name.replace('with-blas-lapack','with-blaslapack')
 
     if name.find('with-debug=') >= 0 or name.endswith('with-debug'):
       if name.find('=') == -1:
@@ -181,52 +166,36 @@ def chksynonyms():
       if tail.find('quad')>=0:
         sys.argv[l]='--with-precision=__float128'
 
-  if downloadideas:
-    downloadxsdk = 1
-    argsAddDownload('pflotran')
-    argsAddDownload('alquimia')
-    # mstk currently cannot build a shared library
-    argsAddDownload('mstk',[],['--download-mstk-shared=0'])
-    argsAddDownload('ascem-io')
-    argsAddDownload('unittestcpp')
+    for i,j in simplereplacements.items():
+      if name.find(i+'=') >= 0:
+        sys.argv[l] = name.replace(i+'=',j+'=')
+      elif name.find('with-'+i.lower()+'=') >= 0:
+        sys.argv[l] = name.replace(i.lower()+'=',j.lower()+'=')
 
-  if downloadxsdk:
-    # Common external libraries
-    argsAddDownload('hdf5')
-    argsAddDownload('netcdf')
-    argsAddDownload('metis')
-
-    argsAddDownload('superlu_dist',['parmetis'])
-
-    argsAddDownload('hypre')
-
-    argsAddDownload('trilinos',['boost','xsdktrilinos'],['--with-cxx-dialect=C++11'])
-
-
-def chkwinf90():
+def chkwincompilerusinglink():
   for arg in sys.argv:
-    if (arg.find('win32fe') >= 0 and (arg.find('f90') >=0 or arg.find('ifort') >=0)):
+    if (arg.find('win32fe') >= 0 and (arg.find('f90') >=0 or arg.find('ifort') >=0 or arg.find('icl') >=0)):
       return 1
   return 0
 
 def chkdosfiles():
   # cygwin - but not a hg clone - so check one of files in bin dir
-  if "\r\n" in open(os.path.join('bin','petscmpiexec'),"rb").read():
-    print '==============================================================================='
-    print ' *** Scripts are in DOS mode. Was winzip used to extract petsc sources?    ****'
-    print ' *** Please restart with a fresh tarball and use "tar -xzf petsc.tar.gz"   ****'
-    print '==============================================================================='
+  if b"\r\n" in open(os.path.join('lib','petsc','bin','petscmpiexec'),"rb").read():
+    print('===============================================================================')
+    print(' *** Scripts are in DOS mode. Was winzip used to extract petsc sources?    ****')
+    print(' *** Please restart with a fresh tarball and use "tar -xzf petsc.tar.gz"   ****')
+    print('===============================================================================')
     sys.exit(3)
   return
 
 def chkcygwinlink():
-  if os.path.exists('/usr/bin/cygcheck.exe') and os.path.exists('/usr/bin/link.exe') and chkwinf90():
+  if os.path.exists('/usr/bin/cygcheck.exe') and os.path.exists('/usr/bin/link.exe') and chkwincompilerusinglink():
       if '--ignore-cygwin-link' in sys.argv: return 0
-      print '==============================================================================='
-      print ' *** Cygwin /usr/bin/link detected! Compiles with CVF/Intel f90 can break!  **'
-      print ' *** To workarround do: "mv /usr/bin/link.exe /usr/bin/link-cygwin.exe"     **'
-      print ' *** Or to ignore this check, use configure option: --ignore-cygwin-link    **'
-      print '==============================================================================='
+      print('===============================================================================')
+      print(' *** Cygwin /usr/bin/link detected! Compiles with Intel icl/ifort can break!  **')
+      print(' *** To workarround do: "mv /usr/bin/link.exe /usr/bin/link-cygwin.exe"     **')
+      print(' *** Or to ignore this check, use configure option: --ignore-cygwin-link. But compiles can fail. **')
+      print('===============================================================================')
       sys.exit(3)
   return 0
 
@@ -234,19 +203,19 @@ def chkbrokencygwin():
   if os.path.exists('/usr/bin/cygcheck.exe'):
     buf = os.popen('/usr/bin/cygcheck.exe -c cygwin').read()
     if buf.find('1.5.11-1') > -1:
-      print '==============================================================================='
-      print ' *** cygwin-1.5.11-1 detected. ./configure fails with this version ***'
-      print ' *** Please upgrade to cygwin-1.5.12-1 or newer version. This can  ***'
-      print ' *** be done by running cygwin-setup, selecting "next" all the way.***'
-      print '==============================================================================='
+      print('===============================================================================')
+      print(' *** cygwin-1.5.11-1 detected. ./configure fails with this version ***')
+      print(' *** Please upgrade to cygwin-1.5.12-1 or newer version. This can  ***')
+      print(' *** be done by running cygwin-setup, selecting "next" all the way.***')
+      print('===============================================================================')
       sys.exit(3)
   return 0
 
 def chkusingwindowspython():
   if sys.platform == 'win32':
-    print '==============================================================================='
-    print ' *** Windows python detected. Please rerun ./configure with cygwin-python. ***'
-    print '==============================================================================='
+    print('===============================================================================')
+    print(' *** Windows python detected. Please rerun ./configure with cygwin-python. ***')
+    print('===============================================================================')
     sys.exit(3)
   return 0
 
@@ -268,6 +237,17 @@ def chkcygwinpython():
 ===============================================================================''')
   return 0
 
+def chkcygwinwindowscompilers():
+  '''Adds win32fe for Microsoft/Intel compilers'''
+  if os.path.exists('/usr/bin/cygcheck.exe'):
+    for l in range(1,len(sys.argv)):
+      option = sys.argv[l]
+      for i in ['cl','icl','ifort']:
+        if option.startswith(i):
+          sys.argv[l] = 'win32fe '+option
+          break
+  return 0
+
 def chkrhl9():
   if os.path.exists('/etc/redhat-release'):
     try:
@@ -286,6 +266,44 @@ def chkrhl9():
 ===============================================================================''')
   return 0
 
+def chktmpnoexec():
+  if not hasattr(os,'ST_NOEXEC'): return
+  if 'TMPDIR' in os.environ: tmpDir = os.environ['TMPDIR']
+  else: tmpDir = '/tmp'
+  if os.statvfs(tmpDir).f_flag & os.ST_NOEXEC:
+    if os.statvfs(os.path.abspath('.')).f_flag & os.ST_NOEXEC:
+      print('************************************************************************')
+      print('* TMPDIR '+tmpDir+' has noexec attribute. Same with '+os.path.abspath('.')+' where petsc is built.')
+      print('* Suggest building PETSc in a location without this restriction!')
+      print('* Alternatively, set env variable TMPDIR to a location that is not restricted to run binaries.')
+      print('************************************************************************')
+      sys.exit(4)
+    else:
+      newTmp = os.path.abspath('tmp-petsc')
+      print('************************************************************************')
+      print('* TMPDIR '+tmpDir+' has noexec attribute. Using '+newTmp+' instead.')
+      print('************************************************************************')
+      if not os.path.isdir(newTmp): os.mkdir(os.path.abspath(newTmp))
+      os.environ['TMPDIR'] = newTmp
+  return
+
+def check_cray_modules():
+  import script
+  '''For Cray systems check if the cc, CC, ftn compiler suite modules have been set'''
+  cray = os.getenv('CRAY_SITE_LIST_DIR')
+  if not cray: return
+  cray = os.getenv('CRAYPE_DIR')
+  if not cray:
+   print('************************************************************************')
+   print('* You are on a Cray system but no programming environments have been loaded')
+   print('* Perhaps you need:')
+   print('*       module load intel ; module load PrgEnv-intel')
+   print('*   or  module load PrgEnv-cray')
+   print('*   or  module load PrgEnv-gnu')
+   print('* See https://www.mcs.anl.gov/petsc/documentation/installation.html#doemachines')
+   print('************************************************************************')
+   sys.exit(4)
+
 def check_broken_configure_log_links():
   '''Sometime symlinks can get broken if the original files are deleted. Delete such broken links'''
   import os
@@ -307,9 +325,11 @@ def move_configure_log(framework):
 
     # Just in case - confdir is not created
     lib_dir = os.path.join(petsc_arch,'lib')
+    petsc_dir = os.path.join(petsc_arch,'lib','petsc')
     conf_dir = os.path.join(petsc_arch,'lib','petsc','conf')
     if not os.path.isdir(petsc_arch): os.mkdir(petsc_arch)
     if not os.path.isdir(lib_dir): os.mkdir(lib_dir)
+    if not os.path.isdir(petsc_dir): os.mkdir(petsc_dir)
     if not os.path.isdir(conf_dir): os.mkdir(conf_dir)
 
     curr_bkp  = curr_file + '.bkp'
@@ -332,7 +352,7 @@ def move_configure_log(framework):
 def print_final_timestamp(framework):
   import time
   framework.log.write(('='*80)+'\n')
-  framework.log.write('Finishing Configure Run at '+time.ctime(time.time())+'\n')
+  framework.log.write('Finishing configure run at '+time.strftime('%a, %d %b %Y %H:%M:%S %z')+'\n')
   framework.log.write(('='*80)+'\n')
   return
 
@@ -342,23 +362,23 @@ def petsc_configure(configure_options):
     if petscdir.find(' ') > -1:
       raise RuntimeError('Your PETSC_DIR '+petscdir+' has spaces in it; this is not allowed.\n Change the directory with PETSc to not have spaces in it')
     try:
-      sys.path.append(os.path.join(petscdir,'bin'))
+      sys.path.append(os.path.join(petscdir,'lib','petsc','bin'))
       import petscnagupgrade
       file     = os.path.join(petscdir,'.nagged')
       if not petscnagupgrade.naggedtoday(file):
         petscnagupgrade.currentversion(petscdir)
     except:
       pass
-  print '==============================================================================='
-  print '             Configuring PETSc to compile on your system                       '
-  print '==============================================================================='
+  print('===============================================================================')
+  print('             Configuring PETSc to compile on your system                       ')
+  print('===============================================================================')
 
   try:
     # Command line arguments take precedence (but don't destroy argv[0])
     sys.argv = sys.argv[:1] + configure_options + sys.argv[1:]
     check_for_option_mistakes(sys.argv)
     check_for_option_changed(sys.argv)
-  except (TypeError, ValueError), e:
+  except (TypeError, ValueError) as e:
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
@@ -385,6 +405,16 @@ def petsc_configure(configure_options):
   chkcygwinpython()
   chkcygwinlink()
   chkdosfiles()
+  chkcygwinwindowscompilers()
+  chktmpnoexec()
+
+  for l in range(1,len(sys.argv)):
+    if sys.argv[l].startswith('--with-fc=') and sys.argv[l].endswith('nagfor'):
+      # need a way to save this value and later CC so that petscnagfor may use them
+      name = sys.argv[l].split('=')[1]
+      sys.argv[l] = '--with-fc='+os.path.join(os.path.abspath('.'),'lib','petsc','bin','petscnagfor')
+      break
+
 
   # Should be run from the toplevel
   configDir = os.path.abspath('config')
@@ -395,8 +425,13 @@ def petsc_configure(configure_options):
   sys.path.insert(0, configDir)
   import config.base
   import config.framework
-  import cPickle
+  import pickle
+  import traceback
 
+  # Check Cray without modules
+  check_cray_modules()
+
+  tbo = None
   framework = None
   try:
     framework = config.framework.Framework(['--configModules=PETSc.Configure','--optionsModule=config.compilerOptions']+sys.argv[1:], loadArgDB = 0)
@@ -404,7 +439,7 @@ def petsc_configure(configure_options):
     framework.logPrint('\n'.join(extraLogs))
     framework.configure(out = sys.stdout)
     framework.storeSubstitutions(framework.argDB)
-    framework.argDB['configureCache'] = cPickle.dumps(framework)
+    framework.argDB['configureCache'] = pickle.dumps(framework)
     framework.printSummary()
     framework.argDB.save(force = True)
     framework.logClear()
@@ -416,7 +451,8 @@ def petsc_configure(configure_options):
       # perhaps print an error about unable to shuffle logs?
       pass
     return 0
-  except (RuntimeError, config.base.ConfigureSetupError), e:
+  except (RuntimeError, config.base.ConfigureSetupError) as e:
+    tbo = sys.exc_info()[2]
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
@@ -424,46 +460,53 @@ def petsc_configure(configure_options):
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except (TypeError, ValueError), e:
+  except (TypeError, ValueError) as e:
+    # this exception is automatically deleted by Python so we need to save it to print below
+    tbo = sys.exc_info()[2]
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
-    +'                ERROR in COMMAND LINE ARGUMENT to ./configure \n' \
+    +'    TypeError or ValueError possibly related to ERROR in COMMAND LINE ARGUMENT while running ./configure \n' \
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except ImportError, e :
+  except ImportError as e :
+    # this exception is automatically deleted by Python so we need to save it to print below
+    tbo = sys.exc_info()[2]
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
-    +'                     UNABLE to FIND MODULE for ./configure \n' \
+    +'                     ImportError while runing ./configure \n' \
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except OSError, e :
+  except OSError as e :
+    tbo = sys.exc_info()[2]
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
-    +'                    UNABLE to EXECUTE BINARIES for ./configure \n' \
+    +'                    OSError while running ./configure \n' \
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except SystemExit, e:
+  except SystemExit as e:
+    tbo = sys.exc_info()[2]
     if e.code is None or e.code == 0:
       return
-    if e.code is 10:
+    if e.code == 10:
       sys.exit(10)
     msg ='*******************************************************************************\n'\
     +'         CONFIGURATION FAILURE  (Please send configure.log to petsc-maint@mcs.anl.gov)\n' \
     +'*******************************************************************************\n'
     se  = str(e)
-  except Exception, e:
+  except Exception as e:
+    tbo = sys.exc_info()[2]
     msg ='*******************************************************************************\n'\
     +'        CONFIGURATION CRASH  (Please send configure.log to petsc-maint@mcs.anl.gov)\n' \
     +'*******************************************************************************\n'
     se  = str(e)
 
-  print msg
+  print(msg)
   if not framework is None:
     framework.logClear()
     if hasattr(framework, 'log'):
@@ -474,22 +517,24 @@ def petsc_configure(configure_options):
         if hasattr(framework,'compilerFixes'):
           framework.log.write('**** C specific Configure header '+framework.compilerFixes+' ****\n')
           framework.outputCHeader(framework.log)
-      except Exception, e:
+      except Exception as e:
         framework.log.write('Problem writing headers to log: '+str(e))
-      import traceback
       try:
         framework.log.write(msg+se)
-        traceback.print_tb(sys.exc_info()[2], file = framework.log)
+        traceback.print_tb(tbo, file = framework.log)
         print_final_timestamp(framework)
         if hasattr(framework,'log'): framework.log.close()
         move_configure_log(framework)
-      except:
-        pass
+      except Exception as e:
+        print('Error printing error message from exception or printing the traceback:'+str(e))
+        traceback.print_tb(sys.exc_info()[2])
       sys.exit(1)
+    else:
+      print(se)
+      traceback.print_tb(tbo)
   else:
-    print se
-    import traceback
-    traceback.print_tb(sys.exc_info()[2])
+    print(se)
+    traceback.print_tb(tbo)
   if hasattr(framework,'log'): framework.log.close()
 
 if __name__ == '__main__':
