@@ -6,11 +6,14 @@
 #include <petscbt.h>
 #include <petscsf.h>
 #include <petsc/private/dmimpl.h>
-#include <petsc/private/isimpl.h>     /* for inline access to atlasOff */
-#include <petsc/private/hash.h>
 
 PETSC_EXTERN PetscLogEvent DMPLEX_Interpolate;
-PETSC_EXTERN PetscLogEvent PETSCPARTITIONER_Partition;
+PETSC_EXTERN PetscLogEvent DMPLEX_Partition;
+PETSC_EXTERN PetscLogEvent DMPLEX_PartSelf;
+PETSC_EXTERN PetscLogEvent DMPLEX_PartLabelInvert;
+PETSC_EXTERN PetscLogEvent DMPLEX_PartLabelCreateSF;
+PETSC_EXTERN PetscLogEvent DMPLEX_PartStratSF;
+PETSC_EXTERN PetscLogEvent DMPLEX_CreatePointSF;
 PETSC_EXTERN PetscLogEvent DMPLEX_Distribute;
 PETSC_EXTERN PetscLogEvent DMPLEX_DistributeCones;
 PETSC_EXTERN PetscLogEvent DMPLEX_DistributeLabels;
@@ -25,6 +28,7 @@ PETSC_EXTERN PetscLogEvent DMPLEX_GlobalToNaturalEnd;
 PETSC_EXTERN PetscLogEvent DMPLEX_NaturalToGlobalBegin;
 PETSC_EXTERN PetscLogEvent DMPLEX_NaturalToGlobalEnd;
 PETSC_EXTERN PetscLogEvent DMPLEX_Stratify;
+PETSC_EXTERN PetscLogEvent DMPLEX_Symmetrize;
 PETSC_EXTERN PetscLogEvent DMPLEX_Preallocate;
 PETSC_EXTERN PetscLogEvent DMPLEX_ResidualFEM;
 PETSC_EXTERN PetscLogEvent DMPLEX_JacobianFEM;
@@ -32,36 +36,34 @@ PETSC_EXTERN PetscLogEvent DMPLEX_InterpolatorFEM;
 PETSC_EXTERN PetscLogEvent DMPLEX_InjectorFEM;
 PETSC_EXTERN PetscLogEvent DMPLEX_IntegralFEM;
 PETSC_EXTERN PetscLogEvent DMPLEX_CreateGmsh;
+PETSC_EXTERN PetscLogEvent DMPLEX_RebalanceSharedPoints;
+PETSC_EXTERN PetscLogEvent DMPLEX_CreateFromFile;
+PETSC_EXTERN PetscLogEvent DMPLEX_CreateFromCellList;
+PETSC_EXTERN PetscLogEvent DMPLEX_CreateFromCellList_Coordinates;
 
 PETSC_EXTERN PetscBool      PetscPartitionerRegisterAllCalled;
 PETSC_EXTERN PetscErrorCode PetscPartitionerRegisterAll(void);
 
-typedef enum {REFINER_NOOP = 0,
-              REFINER_SIMPLEX_1D,
-              REFINER_SIMPLEX_2D,
-              REFINER_HYBRID_SIMPLEX_2D,
-              REFINER_SIMPLEX_TO_HEX_2D,
-              REFINER_HEX_2D,
-              REFINER_HYBRID_HEX_2D,
-              REFINER_SIMPLEX_3D,
-              REFINER_HYBRID_SIMPLEX_3D,
-              REFINER_SIMPLEX_TO_HEX_3D,
-              REFINER_HEX_3D,
-              REFINER_HYBRID_HEX_3D} CellRefiner;
-
 typedef struct _PetscPartitionerOps *PetscPartitionerOps;
 struct _PetscPartitionerOps {
   PetscErrorCode (*setfromoptions)(PetscOptionItems*,PetscPartitioner);
   PetscErrorCode (*setup)(PetscPartitioner);
   PetscErrorCode (*view)(PetscPartitioner,PetscViewer);
   PetscErrorCode (*destroy)(PetscPartitioner);
-  PetscErrorCode (*partition)(PetscPartitioner, DM, PetscInt, PetscInt, PetscInt[], PetscInt[], PetscSection, IS *);
+  PetscErrorCode (*partition)(PetscPartitioner, PetscInt, PetscInt, PetscInt[], PetscInt[], PetscSection, PetscSection, PetscSection, IS *);
 };
 
 struct _p_PetscPartitioner {
   PETSCHEADER(struct _PetscPartitionerOps);
-  void           *data;             /* Implementation object */
-  PetscInt        height;           /* Height of points to partition into non-overlapping subsets */
+  void        *data;            /* Implementation object */
+  PetscInt    height;           /* Height of points to partition into non-overlapping subsets */
+  PetscInt    edgeCut;          /* The number of edge cut by the partition */
+  PetscReal   balance;          /* The maximum partition size divided by the minimum size */
+  PetscViewer viewer;
+  PetscViewer viewerGraph;
+  PetscBool   viewGraph;
+  PetscBool   noGraph;          /* if true, the partitioner does not need the connectivity graph, only the number of local vertices */
+  PetscBool   usevwgt;          /* if true, the partitioner looks at the local section vertSection to weight the vertices of the graph */
 };
 
 typedef struct {
@@ -69,9 +71,31 @@ typedef struct {
 } PetscPartitioner_Chaco;
 
 typedef struct {
-  PetscInt ptype;
+  MPI_Comm  pcomm;
+  PetscInt  ptype;
+  PetscReal imbalanceRatio;
+  PetscInt  debugFlag;
+  PetscInt  randomSeed;
 } PetscPartitioner_ParMetis;
 
+typedef struct {
+  MPI_Comm  pcomm;
+  PetscInt  strategy;
+  PetscReal imbalance;
+} PetscPartitioner_PTScotch;
+
+static const char *const
+PTScotchStrategyList[] = {
+  "DEFAULT",
+  "QUALITY",
+  "SPEED",
+  "BALANCE",
+  "SAFETY",
+  "SCALABILITY",
+  "RECURSIVE",
+  "REMAP"
+};
+
 typedef struct {
   PetscSection section;   /* Sizes for each partition */
   IS           partition; /* Points in each partition */
@@ -86,15 +110,29 @@ typedef struct {
   PetscInt dummy;
 } PetscPartitioner_Gather;
 
-/* Utility struct to store the contents of a Gmsh file in memory */
-typedef struct {
-  PetscInt dim;      /* Entity dimension */
-  PetscInt id;       /* Element number */
-  PetscInt numNodes; /* Size of node array */
-  int nodes[8];      /* Node array */
-  PetscInt numTags;  /* Size of tag array */
-  int tags[4];       /* Tag array */
-} GmshElement;
+typedef struct _DMPlexCellRefinerOps *DMPlexCellRefinerOps;
+struct _DMPlexCellRefinerOps {
+  PetscErrorCode (*refine)(DMPlexCellRefiner, DMPolytopeType, PetscInt *, DMPolytopeType *[], PetscInt *[], PetscInt *[], PetscInt *[]);
+  PetscErrorCode (*mapsubcells)(DMPlexCellRefiner, DMPolytopeType, PetscInt, DMPolytopeType, PetscInt, PetscInt, PetscInt *, PetscInt *);
+  PetscErrorCode (*getaffinetransforms)(DMPlexCellRefiner, DMPolytopeType, PetscInt *, PetscReal *[], PetscReal *[], PetscReal *[]);
+  PetscErrorCode (*getaffinefacetransforms)(DMPlexCellRefiner, DMPolytopeType, PetscInt *, PetscReal *[], PetscReal *[], PetscReal *[], PetscReal *[]);
+  PetscErrorCode (*getcellvertices)(DMPlexCellRefiner, DMPolytopeType, PetscInt *, PetscReal *[]);
+  PetscErrorCode (*getsubcellvertices)(DMPlexCellRefiner, DMPolytopeType, DMPolytopeType, PetscInt, PetscInt *, PetscInt *[]);
+};
+
+struct _p_DMPlexCellRefiner {
+  PETSCHEADER(struct _DMPlexCellRefinerOps);
+  DM        dm;         /* The original DM */
+  PetscBool setupcalled;
+  DMPlexCellRefinerType type;
+  PetscInt *ctOrder;    /* [i] = ct: An array with cell types in depth order */
+  PetscInt *ctOrderInv; /* [ct] = i: An array with the ordinal numbers for each cell type */
+  PetscInt *ctStart;    /* The number for the first cell of each polytope type in the original mesh, indexed by cell type */
+  PetscInt *ctStartNew; /* The number for the first cell of each polytope type in the new mesh, indexed by cell type */
+  PetscInt *offset;     /* [ct][ctNew]: The offset in the new point numbering of a point of type ctNew produced from an old point of type ct */
+  PetscFE      *coordFE; /* Finite element for each cell type, used for localized coordinate interpolation */
+  PetscFEGeom **refGeom; /* Geometry of the reference cell for each cell type */
+};
 
 /* Utility struct to store the contents of a Fluent file in memory */
 typedef struct {
@@ -119,10 +157,21 @@ struct _PetscGridHash {
   DMLabel      cellsSparse; /* Sparse storage for cell map */
 };
 
+/* Point Numbering in Plex:
+
+   Points are numbered contiguously by stratum. Strate are organized as follows:
+
+   First Stratum:  Cells [height 0]
+   Second Stratum: Vertices [depth 0]
+   Third Stratum:  Faces [height 1]
+   Fourth Stratum: Edges [depth 1]
+
+   We do this so that the numbering of a cell-vertex mesh does not change after interpolation. Within a given stratum,
+   we allow additional segregation of by cell type.
+*/
 typedef struct {
   PetscInt             refct;
 
-  /* Sieve */
   PetscSection         coneSection;       /* Layout of cones (inedges for DAG) */
   PetscInt             maxConeSize;       /* Cached for fast lookup */
   PetscInt            *cones;             /* Cone for each point */
@@ -133,17 +182,21 @@ typedef struct {
   PetscBool            refinementUniform; /* Flag for uniform cell refinement */
   PetscReal            refinementLimit;   /* Maximum volume for refined cell */
   PetscErrorCode     (*refinementFunc)(const PetscReal [], PetscReal *); /* Function giving the maximum volume for refined cell */
-  PetscInt             hybridPointMax[8]; /* Allow segregation of some points, each dimension has a divider (used in VTK output and refinement) */
+  PetscInt             overlap;           /* Overlap of the partitions as passed to DMPlexDistribute() or DMPlexDistributeOverlap() */
+  DMPlexInterpolatedFlag interpolated;
+  DMPlexInterpolatedFlag interpolatedCollective;
 
   PetscInt            *facesTmp;          /* Work space for faces operation */
 
   /* Hierarchy */
-  PetscBool            regularRefinement; /* This flag signals that we are a regular refinement of coarseMesh */
+  DMPlexCellRefinerType cellRefiner;       /* Strategy for refining cells */
+  PetscBool             regularRefinement; /* This flag signals that we are a regular refinement of coarseMesh */
 
   /* Generation */
   char                *tetgenOpts;
   char                *triangleOpts;
   PetscPartitioner     partitioner;
+  PetscBool            partitionBalance;  /* Evenly divide partition overlap when distributing */
   PetscBool            remeshBd;
 
   /* Submesh */
@@ -151,6 +204,7 @@ typedef struct {
 
   /* Labels and numbering */
   PetscObjectState     depthState;        /* State of depth label, so that we can determine if a user changes it */
+  PetscObjectState     celltypeState;     /* State of celltype label, so that we can determine if a user changes it */
   IS                   globalVertexNumbers;
   IS                   globalCellNumbers;
 
@@ -192,6 +246,7 @@ typedef struct {
   /* Debugging */
   PetscBool            printSetValues;
   PetscInt             printFEM;
+  PetscInt             printL2;
   PetscReal            printTol;
 } DM_Plex;
 
@@ -215,6 +270,7 @@ PETSC_EXTERN PetscErrorCode DMPlexView_HDF5(DM, PetscViewer);
 PETSC_EXTERN PetscErrorCode DMPlexLoad_HDF5(DM, PetscViewer);
 #endif
 
+PETSC_INTERN PetscErrorCode DMPlexClosurePoints_Private(DM,PetscInt,const PetscInt[],IS*);
 PETSC_INTERN PetscErrorCode DMSetFromOptions_NonRefinement_Plex(PetscOptionItems *, DM);
 PETSC_INTERN PetscErrorCode DMCoarsen_Plex(DM, MPI_Comm, DM *);
 PETSC_INTERN PetscErrorCode DMCoarsenHierarchy_Plex(DM, PetscInt, DM []);
@@ -232,8 +288,14 @@ PETSC_INTERN PetscErrorCode DMComputeL2GradientDiff_Plex(DM,PetscReal,PetscError
 PETSC_INTERN PetscErrorCode DMComputeL2FieldDiff_Plex(DM,PetscReal,PetscErrorCode(**)(PetscInt,PetscReal,const PetscReal[],PetscInt,PetscScalar *,void *),void **,Vec,PetscReal *);
 PETSC_INTERN PetscErrorCode DMLocatePoints_Plex(DM, Vec, DMPointLocationType, PetscSF);
 
+PETSC_INTERN PetscErrorCode DMPlexBuildFromCellList_Internal(DM, PetscInt, PetscInt, PetscInt, PetscInt, const int[], PetscBool);
+PETSC_INTERN PetscErrorCode DMPlexBuildFromCellList_Parallel_Internal(DM, PetscInt, PetscInt, PetscInt, PetscInt, const int[], PetscBool, PetscSF *);
+PETSC_INTERN PetscErrorCode DMPlexBuildCoordinates_Internal(DM, PetscInt, PetscInt, PetscInt, const double[]);
+PETSC_INTERN PetscErrorCode DMPlexBuildCoordinates_Parallel_Internal(DM, PetscInt, PetscInt, PetscInt, PetscSF, const PetscReal[]);
+PETSC_INTERN PetscErrorCode DMPlexLoadLabels_HDF5_Internal(DM, PetscViewer);
 PETSC_INTERN PetscErrorCode DMPlexView_HDF5_Internal(DM, PetscViewer);
 PETSC_INTERN PetscErrorCode DMPlexLoad_HDF5_Internal(DM, PetscViewer);
+PETSC_INTERN PetscErrorCode DMPlexLoad_HDF5_Xdmf_Internal(DM, PetscViewer);
 PETSC_INTERN PetscErrorCode VecView_Plex_HDF5_Internal(Vec, PetscViewer);
 PETSC_INTERN PetscErrorCode VecView_Plex_HDF5_Native_Internal(Vec, PetscViewer);
 PETSC_INTERN PetscErrorCode VecView_Plex_Local_HDF5_Internal(Vec, PetscViewer);
@@ -247,17 +309,14 @@ PETSC_EXTERN PetscErrorCode VecViewPlex_ExodusII_Zonal_Internal(Vec, int, int);
 PETSC_EXTERN PetscErrorCode VecLoadPlex_ExodusII_Zonal_Internal(Vec, int, int);
 PETSC_INTERN PetscErrorCode DMPlexVTKGetCellType_Internal(DM,PetscInt,PetscInt,PetscInt*);
 PETSC_INTERN PetscErrorCode DMPlexGetAdjacency_Internal(DM,PetscInt,PetscBool,PetscBool,PetscBool,PetscInt*,PetscInt*[]);
-PETSC_INTERN PetscErrorCode DMPlexGetFaces_Internal(DM,PetscInt,PetscInt,PetscInt*,PetscInt*,const PetscInt*[]);
-PETSC_INTERN PetscErrorCode DMPlexGetRawFaces_Internal(DM,PetscInt,PetscInt,const PetscInt[], PetscInt*,PetscInt*,const PetscInt*[]);
-PETSC_INTERN PetscErrorCode DMPlexRestoreFaces_Internal(DM,PetscInt,PetscInt,PetscInt*,PetscInt*,const PetscInt*[]);
-PETSC_INTERN PetscErrorCode DMPlexRefineUniform_Internal(DM,CellRefiner,DM*);
-PETSC_INTERN PetscErrorCode DMPlexGetCellRefiner_Internal(DM,CellRefiner*);
-PETSC_INTERN PetscErrorCode CellRefinerGetAffineTransforms_Internal(CellRefiner, PetscInt *, PetscReal *[], PetscReal *[], PetscReal *[]);
-PETSC_INTERN PetscErrorCode CellRefinerRestoreAffineTransforms_Internal(CellRefiner, PetscInt *, PetscReal *[], PetscReal *[], PetscReal *[]);
-PETSC_INTERN PetscErrorCode CellRefinerInCellTest_Internal(CellRefiner, const PetscReal[], PetscBool *);
-PETSC_INTERN PetscErrorCode DMPlexInvertCell_Internal(PetscInt, PetscInt, PetscInt[]);
+PETSC_INTERN PetscErrorCode DMPlexGetRawFaces_Internal(DM,DMPolytopeType,const PetscInt[],PetscInt*,const DMPolytopeType*[],const PetscInt*[],const PetscInt*[]);
+PETSC_INTERN PetscErrorCode DMPlexRestoreRawFaces_Internal(DM,DMPolytopeType,const PetscInt[],PetscInt*,const DMPolytopeType*[],const PetscInt*[],const PetscInt*[]);
+PETSC_INTERN PetscErrorCode CellRefinerInCellTest_Internal(DMPolytopeType, const PetscReal[], PetscBool *);
+PETSC_INTERN PetscErrorCode DMPlexComputeCellType_Internal(DM, PetscInt, PetscInt, DMPolytopeType *);
+PETSC_INTERN PetscErrorCode DMPlexCreateCellTypeOrder_Internal(DMPolytopeType, PetscInt *[], PetscInt *[]);
 PETSC_INTERN PetscErrorCode DMPlexVecSetFieldClosure_Internal(DM, PetscSection, Vec, PetscBool[], PetscInt, PetscInt, const PetscInt[], const PetscScalar[], InsertMode);
 PETSC_INTERN PetscErrorCode DMPlexProjectConstraints_Internal(DM, Vec, Vec);
+PETSC_EXTERN PetscErrorCode DMPlexCreateReferenceTree_SetTree(DM, PetscSection, PetscInt[], PetscInt[]);
 PETSC_EXTERN PetscErrorCode DMPlexCreateReferenceTree_Union(DM,DM,const char *,DM*);
 PETSC_EXTERN PetscErrorCode DMPlexComputeInterpolatorTree(DM,DM,PetscSF,PetscInt *,Mat);
 PETSC_EXTERN PetscErrorCode DMPlexComputeInjectorTree(DM,DM,PetscSF,PetscInt *,Mat);
@@ -265,11 +324,20 @@ PETSC_EXTERN PetscErrorCode DMPlexAnchorsModifyMat(DM,PetscSection,PetscInt,Pets
 PETSC_EXTERN PetscErrorCode indicesPoint_private(PetscSection,PetscInt,PetscInt,PetscInt *,PetscBool,PetscInt,PetscInt []);
 PETSC_EXTERN PetscErrorCode indicesPointFields_private(PetscSection,PetscInt,PetscInt,PetscInt [],PetscBool,PetscInt,PetscInt []);
 PETSC_INTERN PetscErrorCode DMPlexLocatePoint_Internal(DM,PetscInt,const PetscScalar [],PetscInt,PetscInt *);
+/* these two are PETSC_EXTERN just because of src/dm/impls/plex/tests/ex18.c */
+PETSC_EXTERN PetscErrorCode DMPlexOrientCell_Internal(DM,PetscInt,PetscInt,PetscBool);
+PETSC_EXTERN PetscErrorCode DMPlexOrientInterface_Internal(DM);
+
+/* Applications may use this function */
+PETSC_EXTERN PetscErrorCode DMPlexCreateNumbering_Plex(DM, PetscInt, PetscInt, PetscInt, PetscInt *, PetscSF, IS *);
 
 PETSC_INTERN PetscErrorCode DMPlexCreateCellNumbering_Internal(DM, PetscBool, IS *);
 PETSC_INTERN PetscErrorCode DMPlexCreateVertexNumbering_Internal(DM, PetscBool, IS *);
 PETSC_INTERN PetscErrorCode DMPlexRefine_Internal(DM, DMLabel, DM *);
 PETSC_INTERN PetscErrorCode DMPlexCoarsen_Internal(DM, DMLabel, DM *);
+PETSC_INTERN PetscErrorCode DMCreateMatrix_Plex(DM, Mat*);
+
+PETSC_INTERN PetscErrorCode DMPlexGetOverlap_Plex(DM, PetscInt *);
 
 /* invert dihedral symmetry: return a^-1,
  * using the representation described in
@@ -298,10 +366,171 @@ PETSC_STATIC_INLINE PetscInt DihedralSwap(PetscInt N, PetscInt a, PetscInt b)
   return DihedralCompose(N,DihedralInvert(N,a),b);
 }
 
-PETSC_EXTERN PetscErrorCode DMPlexComputeResidual_Internal(DM, PetscInt, PetscInt, PetscReal, Vec, Vec, PetscReal, Vec, void *);
-PETSC_EXTERN PetscErrorCode DMPlexComputeJacobian_Internal(DM, PetscInt, PetscInt, PetscReal, PetscReal, Vec, Vec, Mat, Mat, void *);
+PETSC_EXTERN PetscErrorCode DMPlexComputeResidual_Internal(DM, IS , PetscReal, Vec, Vec, PetscReal, Vec, void *);
+PETSC_EXTERN PetscErrorCode DMPlexComputeJacobian_Internal(DM, IS, PetscReal, PetscReal, Vec, Vec, Mat, Mat, void *);
 PETSC_EXTERN PetscErrorCode DMPlexReconstructGradients_Internal(DM, PetscFV, PetscInt, PetscInt, Vec, Vec, Vec, Vec);
 
+/* Matvec with A in row-major storage, x and y can be aliased */
+PETSC_STATIC_INLINE void DMPlex_Mult2D_Internal(const PetscScalar A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[2];
+  z[0] = x[0]; z[1] = x[ldx];
+  y[0]   = A[0]*z[0] + A[1]*z[1];
+  y[ldx] = A[2]*z[0] + A[3]*z[1];
+  (void)PetscLogFlops(6.0);
+}
+PETSC_STATIC_INLINE void DMPlex_Mult3D_Internal(const PetscScalar A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[3];
+  z[0] = x[0]; z[1] = x[ldx]; z[2] = x[ldx*2];
+  y[0]     = A[0]*z[0] + A[1]*z[1] + A[2]*z[2];
+  y[ldx]   = A[3]*z[0] + A[4]*z[1] + A[5]*z[2];
+  y[ldx*2] = A[6]*z[0] + A[7]*z[1] + A[8]*z[2];
+  (void)PetscLogFlops(15.0);
+}
+PETSC_STATIC_INLINE void DMPlex_MultTranspose2D_Internal(const PetscScalar A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[2];
+  z[0] = x[0]; z[1] = x[ldx];
+  y[0]   = A[0]*z[0] + A[2]*z[1];
+  y[ldx] = A[1]*z[0] + A[3]*z[1];
+  (void)PetscLogFlops(6.0);
+}
+PETSC_STATIC_INLINE void DMPlex_MultTranspose3D_Internal(const PetscScalar A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[3];
+  z[0] = x[0]; z[1] = x[ldx]; z[2] = x[ldx*2];
+  y[0]     = A[0]*z[0] + A[3]*z[1] + A[6]*z[2];
+  y[ldx]   = A[1]*z[0] + A[4]*z[1] + A[7]*z[2];
+  y[ldx*2] = A[2]*z[0] + A[5]*z[1] + A[8]*z[2];
+  (void)PetscLogFlops(15.0);
+}
+PETSC_STATIC_INLINE void DMPlex_Mult2DReal_Internal(const PetscReal A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[2];
+  z[0] = x[0]; z[1] = x[ldx];
+  y[0]   = A[0]*z[0] + A[1]*z[1];
+  y[ldx] = A[2]*z[0] + A[3]*z[1];
+  (void)PetscLogFlops(6.0);
+}
+PETSC_STATIC_INLINE void DMPlex_Mult3DReal_Internal(const PetscReal A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[3];
+  z[0] = x[0]; z[1] = x[ldx]; z[2] = x[ldx*2];
+  y[0]     = A[0]*z[0] + A[1]*z[1] + A[2]*z[2];
+  y[ldx]   = A[3]*z[0] + A[4]*z[1] + A[5]*z[2];
+  y[ldx*2] = A[6]*z[0] + A[7]*z[1] + A[8]*z[2];
+  (void)PetscLogFlops(15.0);
+}
+PETSC_STATIC_INLINE void DMPlex_MultTranspose2DReal_Internal(const PetscReal A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[2];
+  z[0] = x[0]; z[1] = x[ldx];
+  y[0]   = A[0]*z[0] + A[2]*z[1];
+  y[ldx] = A[1]*z[0] + A[3]*z[1];
+  (void)PetscLogFlops(6.0);
+}
+PETSC_STATIC_INLINE void DMPlex_MultTranspose3DReal_Internal(const PetscReal A[], PetscInt ldx, const PetscScalar x[], PetscScalar y[])
+{
+  PetscScalar z[3];
+  z[0] = x[0]; z[1] = x[ldx]; z[2] = x[ldx*2];
+  y[0]     = A[0]*z[0] + A[3]*z[1] + A[6]*z[2];
+  y[ldx]   = A[1]*z[0] + A[4]*z[1] + A[7]*z[2];
+  y[ldx*2] = A[2]*z[0] + A[5]*z[1] + A[8]*z[2];
+  (void)PetscLogFlops(15.0);
+}
+
+PETSC_STATIC_INLINE void DMPlex_MatMult2D_Internal(const PetscScalar A[], PetscInt n, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < n; ++j) {
+    PetscScalar z[2];
+    z[0] = B[0+j]; z[1] = B[1*ldb+j];
+    DMPlex_Mult2D_Internal(A, 1, z, z);
+    C[0+j] = z[0]; C[1*ldb+j] = z[1];
+  }
+  (void)PetscLogFlops(8.0*n);
+}
+PETSC_STATIC_INLINE void DMPlex_MatMult3D_Internal(const PetscScalar A[], PetscInt n, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < n; ++j) {
+    PetscScalar z[3];
+    z[0] = B[0+j]; z[1] = B[1*ldb+j]; z[2] = B[2*ldb+j];
+    DMPlex_Mult3D_Internal(A, 1, z, z);
+    C[0+j] = z[0]; C[1*ldb+j] = z[1]; C[2*ldb+j] = z[2];
+  }
+  (void)PetscLogFlops(8.0*n);
+}
+PETSC_STATIC_INLINE void DMPlex_MatMultTranspose2D_Internal(const PetscScalar A[], PetscInt n, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < n; ++j) {
+    PetscScalar z[2];
+    z[0] = B[0+j]; z[1] = B[1*ldb+j];
+    DMPlex_MultTranspose2D_Internal(A, 1, z, z);
+    C[0+j] = z[0]; C[1*ldb+j] = z[1];
+  }
+  (void)PetscLogFlops(8.0*n);
+}
+PETSC_STATIC_INLINE void DMPlex_MatMultTranspose3D_Internal(const PetscScalar A[], PetscInt n, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < n; ++j) {
+    PetscScalar z[3];
+    z[0] = B[0+j]; z[1] = B[1*ldb+j]; z[2] = B[2*ldb+j];
+    DMPlex_MultTranspose3D_Internal(A, 1, z, z);
+    C[0+j] = z[0]; C[1*ldb+j] = z[1]; C[2*ldb+j] = z[2];
+  }
+  (void)PetscLogFlops(8.0*n);
+}
+
+PETSC_STATIC_INLINE void DMPlex_MatMultLeft2D_Internal(const PetscScalar A[], PetscInt m, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < m; ++j) {
+    DMPlex_MultTranspose2D_Internal(A, 1, &B[j*ldb], &C[j*ldb]);
+  }
+  (void)PetscLogFlops(8.0*m);
+}
+PETSC_STATIC_INLINE void DMPlex_MatMultLeft3D_Internal(const PetscScalar A[], PetscInt m, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < m; ++j) {
+    DMPlex_MultTranspose3D_Internal(A, 1, &B[j*ldb], &C[j*ldb]);
+  }
+  (void)PetscLogFlops(8.0*m);
+}
+PETSC_STATIC_INLINE void DMPlex_MatMultTransposeLeft2D_Internal(const PetscScalar A[], PetscInt m, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < m; ++j) {
+    DMPlex_Mult2D_Internal(A, 1, &B[j*ldb], &C[j*ldb]);
+  }
+  (void)PetscLogFlops(8.0*m);
+}
+PETSC_STATIC_INLINE void DMPlex_MatMultTransposeLeft3D_Internal(const PetscScalar A[], PetscInt m, PetscInt ldb, const PetscScalar B[], PetscScalar C[])
+{
+  PetscInt j;
+  for (j = 0; j < m; ++j) {
+    DMPlex_Mult3D_Internal(A, 1, &B[j*ldb], &C[j*ldb]);
+  }
+  (void)PetscLogFlops(8.0*m);
+}
+
+PETSC_STATIC_INLINE void DMPlex_Transpose2D_Internal(PetscScalar A[])
+{
+  PetscScalar tmp;
+  tmp = A[1]; A[1] = A[2]; A[2] = tmp;
+}
+PETSC_STATIC_INLINE void DMPlex_Transpose3D_Internal(PetscScalar A[])
+{
+  PetscScalar tmp;
+  tmp = A[1]; A[1] = A[3]; A[3] = tmp;
+  tmp = A[2]; A[2] = A[6]; A[6] = tmp;
+  tmp = A[5]; A[5] = A[7]; A[7] = tmp;
+}
+
 PETSC_STATIC_INLINE void DMPlex_Invert2D_Internal(PetscReal invJ[], PetscReal J[], PetscReal detJ)
 {
   const PetscReal invDet = 1.0/detJ;
@@ -365,126 +594,60 @@ PETSC_STATIC_INLINE PetscReal DMPlex_DotRealD_Internal(PetscInt dim, const Petsc
 
 PETSC_STATIC_INLINE PetscReal DMPlex_NormD_Internal(PetscInt dim, const PetscReal *x) {PetscReal sum = 0.0; PetscInt d; for (d = 0; d < dim; ++d) sum += x[d]*x[d]; return PetscSqrtReal(sum);}
 
-PETSC_STATIC_INLINE PetscErrorCode DMPlexGetLocalOffset_Private(DM dm, PetscInt point, PetscInt *start, PetscInt *end)
+PETSC_STATIC_INLINE PetscErrorCode DMPlexFixFaceOrientations_Translate_Private(PetscInt ornt, PetscInt *start, PetscBool *reverse)
 {
-  PetscFunctionBeginHot;
-#if defined(PETSC_USE_DEBUG)
-  {
-    PetscInt       dof;
-    PetscErrorCode ierr;
-    *start = *end = 0; /* Silence overzealous compiler warning */
-    if (!dm->defaultSection) SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_ARG_WRONG, "DM must have a default Section, see DMSetDefaultSection()");
-    ierr = PetscSectionGetOffset(dm->defaultSection, point, start);CHKERRQ(ierr);
-    ierr = PetscSectionGetDof(dm->defaultSection, point, &dof);CHKERRQ(ierr);
-    *end = *start + dof;
-  }
-#else
-  {
-    const PetscSection s = dm->defaultSection;
-    *start = s->atlasOff[point - s->pStart];
-    *end   = *start + s->atlasDof[point - s->pStart];
-  }
-#endif
+  PetscFunctionBegin;
+  *reverse = (ornt < 0) ? PETSC_TRUE : PETSC_FALSE;
+  *start = *reverse ? -(ornt+1) : ornt;
   PetscFunctionReturn(0);
 }
 
-PETSC_STATIC_INLINE PetscErrorCode DMPlexGetLocalFieldOffset_Private(DM dm, PetscInt point, PetscInt field, PetscInt *start, PetscInt *end)
+PETSC_STATIC_INLINE PetscErrorCode DMPlexFixFaceOrientations_Combine_Private(PetscInt coneSize, PetscInt origStart, PetscBool origReverse, PetscInt rotateStart, PetscBool rotateReverse, PetscInt *newStart, PetscBool *newReverse)
 {
   PetscFunctionBegin;
-#if defined(PETSC_USE_DEBUG)
-  {
-    PetscInt       dof;
-    PetscErrorCode ierr;
-    *start = *end = 0; /* Silence overzealous compiler warning */
-    if (!dm->defaultSection) SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_ARG_WRONG, "DM must have a default Section, see DMSetDefaultSection()");
-    ierr = PetscSectionGetFieldOffset(dm->defaultSection, point, field, start);CHKERRQ(ierr);
-    ierr = PetscSectionGetFieldDof(dm->defaultSection, point, field, &dof);CHKERRQ(ierr);
-    *end = *start + dof;
-  }
-#else
-  {
-    const PetscSection s = dm->defaultSection->field[field];
-    *start = s->atlasOff[point - s->pStart];
-    *end   = *start + s->atlasDof[point - s->pStart];
-  }
-#endif
+  *newReverse = (origReverse == rotateReverse) ? PETSC_FALSE : PETSC_TRUE;
+  *newStart = rotateReverse ? (coneSize + rotateStart - origStart) : (coneSize + origStart - rotateStart);
+  *newStart %= coneSize;
   PetscFunctionReturn(0);
 }
 
-PETSC_STATIC_INLINE PetscErrorCode DMPlexGetGlobalOffset_Private(DM dm, PetscInt point, PetscInt *start, PetscInt *end)
+PETSC_STATIC_INLINE PetscErrorCode DMPlexFixFaceOrientations_TranslateBack_Private(PetscInt coneSize, PetscInt start, PetscBool reverse, PetscInt *ornt)
 {
   PetscFunctionBegin;
-#if defined(PETSC_USE_DEBUG)
-  {
-    PetscErrorCode ierr;
-    PetscInt       dof,cdof;
-    *start = *end = 0; /* Silence overzealous compiler warning */
-    if (!dm->defaultSection) SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_ARG_WRONG, "DM must have a default Section, see DMSetDefaultSection()");
-    if (!dm->defaultGlobalSection) SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_ARG_WRONG, "DM must have a default global Section. It will be created automatically by DMGetDefaultGlobalSection()");
-    ierr = PetscSectionGetOffset(dm->defaultGlobalSection, point, start);CHKERRQ(ierr);
-    ierr = PetscSectionGetDof(dm->defaultGlobalSection, point, &dof);CHKERRQ(ierr);
-    ierr = PetscSectionGetConstraintDof(dm->defaultGlobalSection, point, &cdof);CHKERRQ(ierr);
-    *end = *start + dof - cdof + (dof < 0 ? 1 : 0);
+  if (coneSize < 3) {
+    /* edges just get flipped if start == 1 regardless direction */
+    *ornt = start ? -2 : 0;
+  } else {
+    *ornt = reverse ? -(start+1) : start;
   }
-#else
-  {
-    const PetscSection s    = dm->defaultGlobalSection;
-    const PetscInt     dof  = s->atlasDof[point - s->pStart];
-    const PetscInt     cdof = s->bc ? s->bc->atlasDof[point - s->bc->pStart] : 0;
-    *start = s->atlasOff[point - s->pStart];
-    *end   = *start + dof - cdof + (dof < 0 ? 1 : 0);
-  }
-#endif
   PetscFunctionReturn(0);
 }
 
-PETSC_STATIC_INLINE PetscErrorCode DMPlexGetGlobalFieldOffset_Private(DM dm, PetscInt point, PetscInt field, PetscInt *start, PetscInt *end)
+PETSC_STATIC_INLINE PetscErrorCode DMPlexFixFaceOrientations_Permute_Private(PetscInt n, const PetscInt arr[], PetscInt start, PetscBool reverse, PetscInt newarr[])
 {
-  PetscFunctionBegin;
-#if defined(PETSC_USE_DEBUG)
-  {
-    PetscInt       loff, lfoff, fdof, fcdof, ffcdof, f;
-    PetscErrorCode ierr;
-    *start = *end = 0; /* Silence overzealous compiler warning */
-    if (!dm->defaultSection) SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_ARG_WRONG, "DM must have a default Section, see DMSetDefaultSection()");
-    if (!dm->defaultGlobalSection) SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_ARG_WRONG, "DM must have a default global Section. It will be crated automatically by DMGetDefaultGlobalSection()");
-    ierr = PetscSectionGetOffset(dm->defaultGlobalSection, point, start);CHKERRQ(ierr);
-    ierr = PetscSectionGetOffset(dm->defaultSection, point, &loff);CHKERRQ(ierr);
-    ierr = PetscSectionGetFieldOffset(dm->defaultSection, point, field, &lfoff);CHKERRQ(ierr);
-    ierr = PetscSectionGetFieldDof(dm->defaultSection, point, field, &fdof);CHKERRQ(ierr);
-    ierr = PetscSectionGetFieldConstraintDof(dm->defaultSection, point, field, &fcdof);CHKERRQ(ierr);
-    *start = *start < 0 ? *start - (lfoff-loff) : *start + lfoff-loff;
-    for (f = 0; f < field; ++f) {
-      ierr = PetscSectionGetFieldConstraintDof(dm->defaultSection, point, f, &ffcdof);CHKERRQ(ierr);
-      *start = *start < 0 ? *start + ffcdof : *start - ffcdof;
-    }
-    *end   = *start < 0 ? *start - (fdof-fcdof) : *start + fdof-fcdof;
-  }
-#else
-  {
-    const PetscSection s     = dm->defaultSection;
-    const PetscSection fs    = dm->defaultSection->field[field];
-    const PetscSection gs    = dm->defaultGlobalSection;
-    const PetscInt     loff  = s->atlasOff[point - s->pStart];
-    const PetscInt     goff  = gs->atlasOff[point - s->pStart];
-    const PetscInt     lfoff = fs->atlasOff[point - s->pStart];
-    const PetscInt     fdof  = fs->atlasDof[point - s->pStart];
-    const PetscInt     fcdof = fs->bc ? fs->bc->atlasDof[point - fs->bc->pStart] : 0;
-    PetscInt           ffcdof = 0, f;
+  PetscInt i;
 
-    for (f = 0; f < field; ++f) {
-      const PetscSection ffs = dm->defaultSection->field[f];
-      ffcdof += ffs->bc ? ffs->bc->atlasDof[point - ffs->bc->pStart] : 0;
-    }
-    *start = goff + (goff < 0 ? loff-lfoff + ffcdof : lfoff-loff - ffcdof);
-    *end   = *start < 0 ? *start - (fdof-fcdof) : *start + fdof-fcdof;
-  }
-#endif
+  PetscFunctionBegin;
+  if (reverse) {for (i=0; i<n; i++) newarr[i] = arr[(n+start-i)%n];}
+  else         {for (i=0; i<n; i++) newarr[i] = arr[(start+i)%n];}
   PetscFunctionReturn(0);
 }
 
 PETSC_INTERN PetscErrorCode DMPlexGetPointDualSpaceFEM(DM,PetscInt,PetscInt,PetscDualSpace *);
-PETSC_INTERN PetscErrorCode DMPlexGetIndicesPoint_Internal(PetscSection,PetscInt,PetscInt,PetscInt *,PetscBool,const PetscInt[],PetscInt[]);
-PETSC_INTERN PetscErrorCode DMPlexGetIndicesPointFields_Internal(PetscSection,PetscInt,PetscInt,PetscInt[],PetscBool,const PetscInt***,PetscInt,PetscInt[]);
+PETSC_INTERN PetscErrorCode DMPlexGetIndicesPoint_Internal(PetscSection,PetscBool,PetscInt,PetscInt,PetscInt *,PetscBool,const PetscInt[],const PetscInt[],PetscInt[]);
+PETSC_INTERN PetscErrorCode DMPlexGetIndicesPointFields_Internal(PetscSection,PetscBool,PetscInt,PetscInt,PetscInt[],PetscBool,const PetscInt***,PetscInt,const PetscInt[],PetscInt[]);
+PETSC_INTERN PetscErrorCode DMPlexGetCompressedClosure(DM, PetscSection, PetscInt, PetscInt *, PetscInt **, PetscSection *, IS *, const PetscInt **);
+PETSC_INTERN PetscErrorCode DMPlexRestoreCompressedClosure(DM, PetscSection, PetscInt, PetscInt *, PetscInt **, PetscSection *, IS *, const PetscInt **);
+
+PETSC_EXTERN PetscErrorCode DMSNESGetFEGeom(DMField, IS, PetscQuadrature, PetscBool, PetscFEGeom **);
+PETSC_EXTERN PetscErrorCode DMSNESRestoreFEGeom(DMField, IS, PetscQuadrature, PetscBool, PetscFEGeom **);
+PETSC_EXTERN PetscErrorCode DMPlexComputeResidual_Patch_Internal(DM, PetscSection, IS, PetscReal, Vec, Vec, Vec, void *);
+PETSC_EXTERN PetscErrorCode DMPlexComputeJacobian_Patch_Internal(DM, PetscSection, PetscSection, IS, PetscReal, PetscReal, Vec, Vec, Mat, Mat, void *);
+PETSC_INTERN PetscErrorCode DMCreateSubDomainDM_Plex(DM,DMLabel,PetscInt,IS*,DM*);
+PETSC_INTERN PetscErrorCode DMPlexBasisTransformPoint_Internal(DM, DM, Vec, PetscInt, PetscBool[], PetscBool, PetscScalar *);
+PETSC_EXTERN PetscErrorCode DMPlexBasisTransformPointTensor_Internal(DM, DM, Vec, PetscInt, PetscBool, PetscInt, PetscScalar *);
+PETSC_INTERN PetscErrorCode DMPlexBasisTransformApplyReal_Internal(DM, const PetscReal[], PetscBool, PetscInt, const PetscReal *, PetscReal *, void *);
+PETSC_INTERN PetscErrorCode DMPlexBasisTransformApply_Internal(DM, const PetscReal[], PetscBool, PetscInt, const PetscScalar *, PetscScalar *, void *);
+PETSC_INTERN PetscErrorCode DMCreateNeumannOverlap_Plex(DM, IS*, Mat*, PetscErrorCode (**)(Mat, PetscReal, Vec, Vec, PetscReal, IS, void*), void **);
 
 #endif /* _PLEXIMPL_H */
