@@ -1,8 +1,8 @@
 /*
   Interface to the PETSc graphics
 */
-#if !defined(__PETSCDRAW_H)
-#define __PETSCDRAW_H
+#if !defined(PETSCDRAW_H)
+#define PETSCDRAW_H
 #include <petscsys.h>
 #include <petscdrawtypes.h>
 
@@ -22,7 +22,7 @@ PETSC_EXTERN PetscErrorCode PetscDrawSetSave(PetscDraw,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawSetSaveMovie(PetscDraw,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawSetSaveFinalImage(PetscDraw,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawView(PetscDraw,PetscViewer);
-PETSC_STATIC_INLINE PetscErrorCode PetscDrawViewFromOptions(PetscDraw A,PetscObject obj,const char name[]) {return PetscObjectViewFromOptions((PetscObject)A,obj,name);}
+PETSC_EXTERN PetscErrorCode PetscDrawViewFromOptions(PetscDraw,PetscObject,const char[]);
 
 /*
    Number of basic colors in the draw routines, the others are used
@@ -65,6 +65,7 @@ PETSC_STATIC_INLINE PetscErrorCode PetscDrawViewFromOptions(PetscDraw A,PetscObj
 #define PETSC_DRAW_LIMEGREEN       30
 #define PETSC_DRAW_LAVENDERBLUSH   31
 #define PETSC_DRAW_PLUM            32
+#define PETSC_DRAW_MAXCOLOR       256
 
 /*MC
 
@@ -83,7 +84,8 @@ PETSC_STATIC_INLINE PetscErrorCode PetscDrawViewFromOptions(PetscDraw A,PetscObj
 .  min - lower end of interval
 -  max - upper end of interval
 
-   Notes: Values outside the interval [min,max] are clipped.
+   Notes:
+    Values outside the interval [min,max] are clipped.
 
    Level: intermediate
 
@@ -97,7 +99,6 @@ PETSC_STATIC_INLINE int PetscDrawRealToColor(PetscReal value,PetscReal min,Petsc
 }
 
 PETSC_EXTERN PetscErrorCode PetscDrawOpenX(MPI_Comm,const char[],const char[],int,int,int,int,PetscDraw*);
-PETSC_EXTERN PetscErrorCode PetscDrawOpenGLUT(MPI_Comm,const char[],const char[],int,int,int,int,PetscDraw*);
 
 #define PETSC_DRAW_FULL_SIZE    -3
 #define PETSC_DRAW_HALF_SIZE    -4
@@ -157,7 +158,6 @@ PETSC_EXTERN PetscErrorCode PetscDrawTensorContour(PetscDraw,int,int,const Petsc
 PETSC_EXTERN PetscErrorCode PetscDrawString(PetscDraw,PetscReal,PetscReal,int,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawStringCentered(PetscDraw,PetscReal,PetscReal,int,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawStringBoxed(PetscDraw,PetscReal,PetscReal,int,int,const char[],PetscReal*,PetscReal*);
-PETSC_EXTERN PetscErrorCode PetscDrawStringBoxedSize(PetscDraw,const char[],PetscReal*,PetscReal*);
 PETSC_EXTERN PetscErrorCode PetscDrawStringVertical(PetscDraw,PetscReal,PetscReal,int,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawStringSetSize(PetscDraw,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawStringGetSize(PetscDraw,PetscReal*,PetscReal*);
@@ -171,7 +171,7 @@ PETSC_EXTERN PetscErrorCode PetscDrawGetCoordinates(PetscDraw,PetscReal*,PetscRe
 
 PETSC_EXTERN PetscErrorCode PetscDrawSetTitle(PetscDraw,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawAppendTitle(PetscDraw,const char[]);
-PETSC_EXTERN PetscErrorCode PetscDrawGetTitle(PetscDraw,char **);
+PETSC_EXTERN PetscErrorCode PetscDrawGetTitle(PetscDraw,const char *[]);
 
 PETSC_EXTERN PetscErrorCode PetscDrawSetPause(PetscDraw,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawGetPause(PetscDraw,PetscReal*);
@@ -215,8 +215,6 @@ PETSC_EXTERN PetscErrorCode PetscDrawZoom(PetscDraw,PetscErrorCode (*)(PetscDraw
 
    Level: intermediate
 
-  Concepts: graphics
-
 .seealso:  PetscDrawViewPortsCreate(), PetscDrawViewPortsSet()
 S*/
 typedef struct {
@@ -227,7 +225,6 @@ typedef struct {
   PetscReal *yr;
   PetscDraw draw;
   PetscReal port_xl,port_yl,port_xr,port_yr;   /* original port of parent PetscDraw */
-
 } PetscDrawViewPorts;
 PETSC_EXTERN PetscErrorCode PetscDrawViewPortsCreate(PetscDraw,PetscInt,PetscDrawViewPorts**);
 PETSC_EXTERN PetscErrorCode PetscDrawViewPortsCreateRect(PetscDraw,PetscInt,PetscInt,PetscDrawViewPorts**);
@@ -313,15 +310,6 @@ PETSC_EXTERN PetscErrorCode PetscDrawBarSetFromOptions(PetscDrawBar);
 PETSC_EXTERN PetscErrorCode PetscDrawBarGetAxis(PetscDrawBar,PetscDrawAxis*);
 PETSC_EXTERN PetscErrorCode PetscDrawBarGetDraw(PetscDrawBar,PetscDraw*);
 
-/*
-    PetscViewer routines that allow you to access underlying PetscDraw objects
-*/
-PETSC_EXTERN PetscErrorCode PetscViewerDrawGetDraw(PetscViewer,PetscInt,PetscDraw*);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawBaseAdd(PetscViewer,PetscInt);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawBaseSet(PetscViewer,PetscInt);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawGetDrawLG(PetscViewer,PetscInt,PetscDrawLG*);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawGetDrawAxis(PetscViewer,PetscInt,PetscDrawAxis*);
-
 PETSC_EXTERN PetscErrorCode PetscDrawUtilitySetCmap(const char[],int,unsigned char[],unsigned char[],unsigned char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawUtilitySetGamma(PetscReal);
 
@@ -338,15 +326,15 @@ PETSC_EXTERN_TYPEDEF typedef void (*PetscXIOErrorHandler)(void *);
 PETSC_EXTERN PetscXIOErrorHandler PetscSetXIOErrorHandler(PetscXIOErrorHandler);
 
 #define PetscDrawCollectiveBegin(draw) 0; do { \
-  PetscErrorCode       _Petsc_ierr; \
-  jmp_buf              _Petsc_jmpbuf; \
-  PetscXIOErrorHandler _Petsc_xioerrhdl = NULL; \
-  PetscBool            _Petsc_isdrawx, _Petsc_xioerr, _Petsc_xioerr_local = PETSC_FALSE; \
+  PetscErrorCode                _Petsc_ierr; \
+  jmp_buf                       _Petsc_jmpbuf; \
+  volatile PetscXIOErrorHandler _Petsc_xioerrhdl = NULL; \
+  PetscBool                     _Petsc_isdrawx, _Petsc_xioerr, _Petsc_xioerr_local = PETSC_FALSE; \
   _Petsc_ierr = PetscObjectTypeCompare((PetscObject)(draw),PETSC_DRAW_X,&_Petsc_isdrawx);CHKERRQ(_Petsc_ierr); \
   if (_Petsc_isdrawx) { \
     _Petsc_ierr = PetscMemcpy(&_Petsc_jmpbuf,&PetscXIOErrorHandlerJumpBuf,sizeof(_Petsc_jmpbuf));CHKERRQ(_Petsc_ierr); \
     _Petsc_xioerrhdl = PetscSetXIOErrorHandler(PetscXIOErrorHandlerJump); \
-    if (setjmp(PetscXIOErrorHandlerJumpBuf)) {_Petsc_xioerr_local = PETSC_TRUE; {_Petsc_ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(_Petsc_ierr);} \
+    if (setjmp(PetscXIOErrorHandlerJumpBuf)) {_Petsc_xioerr_local = PETSC_TRUE; do {_Petsc_ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(_Petsc_ierr);} \
   } do {} while(0)
 
 #define PetscDrawCollectiveEnd(draw) 0; \
