@@ -3,18 +3,18 @@
 
 #define PETSC_VERSION_RELEASE    1
 #define PETSC_VERSION_MAJOR      3
-#define PETSC_VERSION_MINOR      12
-#define PETSC_VERSION_SUBMINOR   5
+#define PETSC_VERSION_MINOR      13
+#define PETSC_VERSION_SUBMINOR   0
 #define PETSC_VERSION_PATCH      0
-#define PETSC_RELEASE_DATE       "Sep, 29, 2019"
-#define PETSC_VERSION_DATE       "Mar, 29, 2020"
+#define PETSC_RELEASE_DATE       "Mar 29, 2020"
+#define PETSC_VERSION_DATE       "Mar 29, 2020"
 
 #if !defined (PETSC_VERSION_GIT)
-#define PETSC_VERSION_GIT        "v3.12.5"
+#define PETSC_VERSION_GIT        "v3.13"
 #endif
 
 #if !defined(PETSC_VERSION_DATE_GIT)
-#define PETSC_VERSION_DATE_GIT   "2020-03-29 09:28:14 -0500"
+#define PETSC_VERSION_DATE_GIT   "2020-03-29 21:59:40 -0500"
 #endif
 
 #define PETSC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
