@@ -141,58 +141,6 @@ static PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-/*
-      M is number of grid points
-      m is number of processors
-
-*/
-PetscErrorCode  DMDASplitComm2d(MPI_Comm comm,PetscInt M,PetscInt N,PetscInt sw,MPI_Comm *outcomm)
-{
-  PetscErrorCode ierr;
-  PetscInt       m,n = 0,x = 0,y = 0;
-  PetscMPIInt    size,csize,rank;
-
-  PetscFunctionBegin;
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
-
-  csize = 4*size;
-  do {
-    if (csize % 4) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Cannot split communicator of size %d tried %d %D %D",size,csize,x,y);
-    csize = csize/4;
-
-    m = (PetscInt)(0.5 + PetscSqrtReal(((PetscReal)M)*((PetscReal)csize)/((PetscReal)N)));
-    if (!m) m = 1;
-    while (m > 0) {
-      n = csize/m;
-      if (m*n == csize) break;
-      m--;
-    }
-    if (M > N && m < n) {PetscInt _m = m; m = n; n = _m;}
-
-    x = M/m + ((M % m) > ((csize-1) % m));
-    y = (N + (csize-1)/m)/n;
-  } while ((x < 4 || y < 4) && csize > 1);
-  if (size != csize) {
-    MPI_Group   entire_group,sub_group;
-    PetscMPIInt i,*groupies;
-
-    ierr = MPI_Comm_group(comm,&entire_group);CHKERRQ(ierr);
-    ierr = PetscMalloc1(csize,&groupies);CHKERRQ(ierr);
-    for (i=0; i<csize; i++) {
-      groupies[i] = (rank/csize)*csize + i;
-    }
-    ierr = MPI_Group_incl(entire_group,csize,groupies,&sub_group);CHKERRQ(ierr);
-    ierr = PetscFree(groupies);CHKERRQ(ierr);
-    ierr = MPI_Comm_create(comm,sub_group,outcomm);CHKERRQ(ierr);
-    ierr = MPI_Group_free(&entire_group);CHKERRQ(ierr);
-    ierr = MPI_Group_free(&sub_group);CHKERRQ(ierr);
-    ierr = PetscInfo1(0,"DMDASplitComm2d:Creating redundant coarse problems of size %d\n",csize);CHKERRQ(ierr);
-  } else {
-    *outcomm = comm;
-  }
-  PetscFunctionReturn(0);
-}
 
 #if defined(new)
 /*
@@ -272,9 +220,6 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
   if (((PetscInt64) M)*((PetscInt64) N)*((PetscInt64) dof) > (PetscInt64) PETSC_MPI_INT_MAX) SETERRQ3(comm,PETSC_ERR_INT_OVERFLOW,"Mesh of %D by %D by %D (dof) is too large for 32 bit indices",M,N,dof);
 #endif
 
-  if (dof < 1) SETERRQ1(comm,PETSC_ERR_ARG_OUTOFRANGE,"Must have 1 or more degrees of freedom per node: %D",dof);
-  if (s < 0) SETERRQ1(comm,PETSC_ERR_ARG_OUTOFRANGE,"Stencil width cannot be negative: %D",s);
-
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
 
@@ -797,7 +742,7 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
    DMDACreate2d -  Creates an object that will manage the communication of  two-dimensional
    regular array data that is distributed across some processors.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm - MPI communicator
@@ -845,16 +790,15 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
    If you wish to use the options database to change values in the DMDA call DMSetFromOptions() after this call
    but before DMSetUp().
 
-.keywords: distributed array, create, two-dimensional
-
 .seealso: DMDestroy(), DMView(), DMDACreate1d(), DMDACreate3d(), DMGlobalToLocalBegin(), DMDAGetRefinementFactor(),
           DMGlobalToLocalEnd(), DMLocalToGlobalBegin(), DMLocalToLocalBegin(), DMLocalToLocalEnd(), DMDASetRefinementFactor(),
-          DMDAGetInfo(), DMCreateGlobalVector(), DMCreateLocalVector(), DMDACreateNaturalVector(), DMLoad(), DMDAGetOwnershipRanges()
+          DMDAGetInfo(), DMCreateGlobalVector(), DMCreateLocalVector(), DMDACreateNaturalVector(), DMLoad(), DMDAGetOwnershipRanges(),
+          DMStagCreate2d()
 
 @*/
 
 PetscErrorCode  DMDACreate2d(MPI_Comm comm,DMBoundaryType bx,DMBoundaryType by,DMDAStencilType stencil_type,
-                          PetscInt M,PetscInt N,PetscInt m,PetscInt n,PetscInt dof,PetscInt s,const PetscInt lx[],const PetscInt ly[],DM *da)
+                             PetscInt M,PetscInt N,PetscInt m,PetscInt n,PetscInt dof,PetscInt s,const PetscInt lx[],const PetscInt ly[],DM *da)
 {
   PetscErrorCode ierr;
 
