@@ -13,36 +13,28 @@
 
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 
-/*@
-    DMCreateInterpolationScale - Forms L = R*1/diag(R*1) - L.*v is like a coarse grid average of the
-      nearby fine grid points.
-
-  Input Parameters:
-+      dac - DM that defines a coarse mesh
-.      daf - DM that defines a fine mesh
--      mat - the restriction (or interpolation operator) from fine to coarse
-
-  Output Parameter:
-.    scale - the scaled vector
-
-  Level: developer
-
-.seealso: DMCreateInterpolation()
-
-@*/
-PetscErrorCode  DMCreateInterpolationScale(DM dac,DM daf,Mat mat,Vec *scale)
+/*
+   Since the interpolation uses MATMAIJ for dof > 0 we convert request for non-MATAIJ baseded matrices to MATAIJ.
+   This is a bit of a hack, the reason for it is partially because -dm_mat_type defines the
+   matrix type for both the operator matrices and the interpolation matrices so that users
+   can select matrix types of base MATAIJ for accelerators
+*/
+static PetscErrorCode ConvertToAIJ(MatType intype,MatType *outtype)
 {
   PetscErrorCode ierr;
-  Vec            fine;
-  PetscScalar    one = 1.0;
+  PetscInt       i;
+  char           const *types[3] = {MATAIJ,MATSEQAIJ,MATMPIAIJ};
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  ierr = DMCreateGlobalVector(daf,&fine);CHKERRQ(ierr);
-  ierr = DMCreateGlobalVector(dac,scale);CHKERRQ(ierr);
-  ierr = VecSet(fine,one);CHKERRQ(ierr);
-  ierr = MatRestrict(mat,fine,*scale);CHKERRQ(ierr);
-  ierr = VecDestroy(&fine);CHKERRQ(ierr);
-  ierr = VecReciprocal(*scale);CHKERRQ(ierr);
+  *outtype = MATAIJ;
+  for (i=0; i<3; i++) {
+    ierr = PetscStrbeginswith(intype,types[i],&flg);CHKERRQ(ierr);
+    if (flg) {
+      *outtype = intype;
+      PetscFunctionReturn(0);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
@@ -58,7 +50,7 @@ PetscErrorCode DMCreateInterpolation_DA_1D_Q1(DM dac,DM daf,Mat *A)
   Mat                    mat;
   DMBoundaryType         bx;
   ISLocalToGlobalMapping ltog_f,ltog_c;
-
+  MatType                mattype;
 
   PetscFunctionBegin;
   ierr = DMDAGetInfo(dac,0,&Mx,0,0,0,0,0,0,0,&bx,0,0,0);CHKERRQ(ierr);
@@ -83,8 +75,18 @@ PetscErrorCode DMCreateInterpolation_DA_1D_Q1(DM dac,DM daf,Mat *A)
 
   /* create interpolation matrix */
   ierr = MatCreate(PetscObjectComm((PetscObject)dac),&mat);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUDA)
+  /*
+     Temporary hack: Since the MAIJ matrix must be converted to AIJ before being used by the GPU
+     we don't want the original unconverted matrix copied to the GPU
+   */
+  if (dof > 1) {
+    ierr = MatBindToCPU(mat,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  #endif
   ierr = MatSetSizes(mat,m_f,m_c,mx,Mx);CHKERRQ(ierr);
-  ierr = MatSetType(mat,MATAIJ);CHKERRQ(ierr);
+  ierr = ConvertToAIJ(dac->mattype,&mattype);CHKERRQ(ierr);
+  ierr = MatSetType(mat,mattype);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(mat,2,NULL);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(mat,2,NULL,1,NULL);CHKERRQ(ierr);
 
@@ -189,14 +191,17 @@ PetscErrorCode DMCreateInterpolation_DA_1D_Q0(DM dac,DM daf,Mat *A)
   PetscScalar            v[2],x;
   Mat                    mat;
   DMBoundaryType         bx;
+  MatType                mattype;
 
   PetscFunctionBegin;
   ierr = DMDAGetInfo(dac,0,&Mx,0,0,0,0,0,0,0,&bx,0,0,0);CHKERRQ(ierr);
   ierr = DMDAGetInfo(daf,0,&mx,0,0,0,0,0,&dof,0,0,0,0,0);CHKERRQ(ierr);
   if (bx == DM_BOUNDARY_PERIODIC) {
+    if (!Mx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be positive",Mx);
     ratio = mx/Mx;
     if (ratio*Mx != mx) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: mx/Mx  must be integer: mx %D Mx %D",mx,Mx);
   } else {
+    if (Mx < 2) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be greater than 1",Mx);
     ratio = (mx-1)/(Mx-1);
     if (ratio*(Mx-1) != mx-1) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: (mx - 1)/(Mx - 1) must be integer: mx %D Mx %D",mx,Mx);
   }
@@ -213,8 +218,18 @@ PetscErrorCode DMCreateInterpolation_DA_1D_Q0(DM dac,DM daf,Mat *A)
 
   /* create interpolation matrix */
   ierr = MatCreate(PetscObjectComm((PetscObject)dac),&mat);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUDA)
+  /*
+     Temporary hack: Since the MAIJ matrix must be converted to AIJ before being used by the GPU
+     we don't want the original unconverted matrix copied to the GPU
+   */
+  if (dof > 1) {
+    ierr = MatBindToCPU(mat,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  #endif
   ierr = MatSetSizes(mat,m_f,m_c,mx,Mx);CHKERRQ(ierr);
-  ierr = MatSetType(mat,MATAIJ);CHKERRQ(ierr);
+  ierr = ConvertToAIJ(dac->mattype,&mattype);CHKERRQ(ierr);
+  ierr = MatSetType(mat,mattype);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(mat,2,NULL);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(mat,2,NULL,0,NULL);CHKERRQ(ierr);
 
@@ -266,26 +281,30 @@ PetscErrorCode DMCreateInterpolation_DA_2D_Q1(DM dac,DM daf,Mat *A)
   PetscScalar            v[4],x,y;
   Mat                    mat;
   DMBoundaryType         bx,by;
+  MatType                mattype;
 
   PetscFunctionBegin;
   ierr = DMDAGetInfo(dac,0,&Mx,&My,0,0,0,0,0,0,&bx,&by,0,0);CHKERRQ(ierr);
   ierr = DMDAGetInfo(daf,0,&mx,&my,0,0,0,0,&dof,0,0,0,0,0);CHKERRQ(ierr);
   if (bx == DM_BOUNDARY_PERIODIC) {
+    if (!Mx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be positive",Mx);
     ratioi = mx/Mx;
     if (ratioi*Mx != mx) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: mx/Mx  must be integer: mx %D Mx %D",mx,Mx);
   } else {
+    if (Mx < 2) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be greater than 1",Mx);
     ratioi = (mx-1)/(Mx-1);
     if (ratioi*(Mx-1) != mx-1) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: (mx - 1)/(Mx - 1) must be integer: mx %D Mx %D",mx,Mx);
   }
   if (by == DM_BOUNDARY_PERIODIC) {
+    if (!My) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of y coarse grid points %D must be positive",My);
     ratioj = my/My;
     if (ratioj*My != my) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: my/My  must be integer: my %D My %D",my,My);
   } else {
+    if (My < 2) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of y coarse grid points %D must be greater than 1",My);
     ratioj = (my-1)/(My-1);
     if (ratioj*(My-1) != my-1) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: (my - 1)/(My - 1) must be integer: my %D My %D",my,My);
   }
 
-
   ierr = DMDAGetCorners(daf,&i_start,&j_start,0,&m_f,&n_f,0);CHKERRQ(ierr);
   ierr = DMDAGetGhostCorners(daf,&i_start_ghost,&j_start_ghost,0,&m_ghost,&n_ghost,0);CHKERRQ(ierr);
   ierr = DMGetLocalToGlobalMapping(daf,&ltog_f);CHKERRQ(ierr);
@@ -345,8 +364,18 @@ PetscErrorCode DMCreateInterpolation_DA_2D_Q1(DM dac,DM daf,Mat *A)
     }
   }
   ierr = MatCreate(PetscObjectComm((PetscObject)daf),&mat);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUDA)
+  /*
+     Temporary hack: Since the MAIJ matrix must be converted to AIJ before being used by the GPU
+     we don't want the original unconverted matrix copied to the GPU
+  */
+  if (dof > 1) {
+    ierr = MatBindToCPU(mat,PETSC_TRUE);CHKERRQ(ierr);
+  }
+#endif
   ierr = MatSetSizes(mat,m_f*n_f,col_scale*m_c*n_c,mx*my,col_scale*Mx*My);CHKERRQ(ierr);
-  ierr = MatSetType(mat,MATAIJ);CHKERRQ(ierr);
+  ierr = ConvertToAIJ(dac->mattype,&mattype);CHKERRQ(ierr);
+  ierr = MatSetType(mat,mattype);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(mat,0,dnz);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(mat,0,dnz,0,onz);CHKERRQ(ierr);
   ierr = MatPreallocateFinalize(dnz,onz);CHKERRQ(ierr);
@@ -490,10 +519,13 @@ PetscErrorCode DMCreateInterpolation_DA_2D_Q0(DM dac,DM daf,Mat *A)
   PetscScalar            v[4];
   Mat                    mat;
   DMBoundaryType         bx,by;
+  MatType                mattype;
 
   PetscFunctionBegin;
   ierr = DMDAGetInfo(dac,0,&Mx,&My,0,0,0,0,0,0,&bx,&by,0,0);CHKERRQ(ierr);
   ierr = DMDAGetInfo(daf,0,&mx,&my,0,0,0,0,&dof,0,0,0,0,0);CHKERRQ(ierr);
+  if (!Mx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be positive",Mx);
+  if (!My) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of y coarse grid points %D must be positive",My);
   ratioi = mx/Mx;
   ratioj = my/My;
   if (ratioi*Mx != mx) SETERRQ(PetscObjectComm((PetscObject)daf),PETSC_ERR_ARG_WRONG,"Fine grid points must be multiple of coarse grid points in x");
@@ -554,8 +586,18 @@ PetscErrorCode DMCreateInterpolation_DA_2D_Q0(DM dac,DM daf,Mat *A)
     }
   }
   ierr = MatCreate(PetscObjectComm((PetscObject)daf),&mat);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUDA)
+  /*
+     Temporary hack: Since the MAIJ matrix must be converted to AIJ before being used by the GPU
+     we don't want the original unconverted matrix copied to the GPU
+  */
+  if (dof > 1) {
+    ierr = MatBindToCPU(mat,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  #endif
   ierr = MatSetSizes(mat,m_f*n_f,col_scale*m_c*n_c,mx*my,col_scale*Mx*My);CHKERRQ(ierr);
-  ierr = MatSetType(mat,MATAIJ);CHKERRQ(ierr);
+  ierr = ConvertToAIJ(dac->mattype,&mattype);CHKERRQ(ierr);
+  ierr = MatSetType(mat,mattype);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(mat,0,dnz);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(mat,0,dnz,0,onz);CHKERRQ(ierr);
   ierr = MatPreallocateFinalize(dnz,onz);CHKERRQ(ierr);
@@ -603,9 +645,13 @@ PetscErrorCode DMCreateInterpolation_DA_3D_Q0(DM dac,DM daf,Mat *A)
   PetscScalar            v[8];
   Mat                    mat;
   DMBoundaryType         bx,by,bz;
+  MatType                mattype;
 
   PetscFunctionBegin;
   ierr = DMDAGetInfo(dac,0,&Mx,&My,&Mz,0,0,0,0,0,&bx,&by,&bz,0);CHKERRQ(ierr);
+  if (!Mx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be positive",Mx);
+  if (!My) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of y coarse grid points %D must be positive",My);
+  if (!Mz) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of z coarse grid points %D must be positive",Mz);
   ierr = DMDAGetInfo(daf,0,&mx,&my,&mz,0,0,0,&dof,0,0,0,0,0);CHKERRQ(ierr);
   ratioi = mx/Mx;
   ratioj = my/My;
@@ -675,8 +721,18 @@ PetscErrorCode DMCreateInterpolation_DA_3D_Q0(DM dac,DM daf,Mat *A)
     }
   }
   ierr = MatCreate(PetscObjectComm((PetscObject)daf),&mat);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUDA)
+  /*
+     Temporary hack: Since the MAIJ matrix must be converted to AIJ before being used by the GPU
+     we don't want the original unconverted matrix copied to the GPU
+  */
+  if (dof > 1) {
+    ierr = MatBindToCPU(mat,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  #endif
   ierr = MatSetSizes(mat,m_f*n_f*p_f,col_scale*m_c*n_c*p_c,mx*my*mz,col_scale*Mx*My*Mz);CHKERRQ(ierr);
-  ierr = MatSetType(mat,MATAIJ);CHKERRQ(ierr);
+  ierr = ConvertToAIJ(dac->mattype,&mattype);CHKERRQ(ierr);
+  ierr = MatSetType(mat,mattype);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(mat,0,dnz);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(mat,0,dnz,0,onz);CHKERRQ(ierr);
   ierr = MatPreallocateFinalize(dnz,onz);CHKERRQ(ierr);
@@ -725,6 +781,7 @@ PetscErrorCode DMCreateInterpolation_DA_3D_Q1(DM dac,DM daf,Mat *A)
   PetscScalar            v[8],x,y,z;
   Mat                    mat;
   DMBoundaryType         bx,by,bz;
+  MatType                mattype;
 
   PetscFunctionBegin;
   ierr = DMDAGetInfo(dac,0,&Mx,&My,&Mz,0,0,0,0,0,&bx,&by,&bz,0);CHKERRQ(ierr);
@@ -732,27 +789,33 @@ PetscErrorCode DMCreateInterpolation_DA_3D_Q1(DM dac,DM daf,Mat *A)
   if (mx == Mx) {
     ratioi = 1;
   } else if (bx == DM_BOUNDARY_PERIODIC) {
+    if (!Mx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be positive",Mx);
     ratioi = mx/Mx;
     if (ratioi*Mx != mx) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: mx/Mx  must be integer: mx %D Mx %D",mx,Mx);
   } else {
+    if (Mx < 2) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of x coarse grid points %D must be greater than 1",Mx);
     ratioi = (mx-1)/(Mx-1);
     if (ratioi*(Mx-1) != mx-1) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: (mx - 1)/(Mx - 1) must be integer: mx %D Mx %D",mx,Mx);
   }
   if (my == My) {
     ratioj = 1;
   } else if (by == DM_BOUNDARY_PERIODIC) {
+    if (!My) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of y coarse grid points %D must be positive",My);
     ratioj = my/My;
     if (ratioj*My != my) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: my/My  must be integer: my %D My %D",my,My);
   } else {
+    if (My < 2) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of y coarse grid points %D must be greater than 1",My);
     ratioj = (my-1)/(My-1);
     if (ratioj*(My-1) != my-1) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: (my - 1)/(My - 1) must be integer: my %D My %D",my,My);
   }
   if (mz == Mz) {
     ratiok = 1;
   } else if (bz == DM_BOUNDARY_PERIODIC) {
+    if (!Mz) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of z coarse grid points %D must be positive",Mz);
     ratiok = mz/Mz;
     if (ratiok*Mz != mz) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: mz/Mz  must be integer: mz %D Mz %D",mz,Mz);
   } else {
+    if (Mz < 2) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of z coarse grid points %D must be greater than 1",Mz);
     ratiok = (mz-1)/(Mz-1);
     if (ratiok*(Mz-1) != mz-1) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Ratio between levels: (mz - 1)/(Mz - 1) must be integer: mz %D Mz %D",mz,Mz);
   }
@@ -819,8 +882,18 @@ PetscErrorCode DMCreateInterpolation_DA_3D_Q1(DM dac,DM daf,Mat *A)
     }
   }
   ierr = MatCreate(PetscObjectComm((PetscObject)dac),&mat);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUDA)
+  /*
+     Temporary hack: Since the MAIJ matrix must be converted to AIJ before being used by the GPU
+     we don't want the original unconverted matrix copied to the GPU
+  */
+  if (dof > 1) {
+    ierr = MatBindToCPU(mat,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  #endif
   ierr = MatSetSizes(mat,m_f*n_f*p_f,m_c*n_c*p_c,mx*my*mz,Mx*My*Mz);CHKERRQ(ierr);
-  ierr = MatSetType(mat,MATAIJ);CHKERRQ(ierr);
+  ierr = ConvertToAIJ(dac->mattype,&mattype);CHKERRQ(ierr);
+  ierr = MatSetType(mat,mattype);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(mat,0,dnz);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(mat,0,dnz,0,onz);CHKERRQ(ierr);
   ierr = MatPreallocateFinalize(dnz,onz);CHKERRQ(ierr);
@@ -1287,7 +1360,38 @@ PetscErrorCode  DMCreateInjection_DA(DM dac,DM daf,Mat *mat)
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode  DMCreateAggregates_DA(DM dac,DM daf,Mat *rest)
+/*@
+   DMCreateAggregates - Deprecated, see DMDACreateAggregates.
+
+   Level: intermediate
+@*/
+PetscErrorCode DMCreateAggregates(DM dac,DM daf,Mat *mat)
+{
+  return DMDACreateAggregates(dac,daf,mat);
+}
+
+/*@
+   DMDACreateAggregates - Gets the aggregates that map between
+   grids associated with two DMDAs.
+
+   Collective on dmc
+
+   Input Parameters:
++  dmc - the coarse grid DMDA
+-  dmf - the fine grid DMDA
+
+   Output Parameters:
+.  rest - the restriction matrix (transpose of the projection matrix)
+
+   Level: intermediate
+
+   Note: This routine is not used by PETSc.
+   It is not clear what its use case is and it may be removed in a future release.
+   Users should contact petsc-maint@mcs.anl.gov if they plan to use it.
+
+.seealso: DMRefine(), DMCreateInjection(), DMCreateInterpolation()
+@*/
+PetscErrorCode DMDACreateAggregates(DM dac,DM daf,Mat *rest)
 {
   PetscErrorCode         ierr;
   PetscInt               dimc,Mc,Nc,Pc,mc,nc,pc,dofc,sc;
@@ -1311,8 +1415,8 @@ PetscErrorCode  DMCreateAggregates_DA(DM dac,DM daf,Mat *rest)
   ISLocalToGlobalMapping ltogmf,ltogmc;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dac,DM_CLASSID,1);
-  PetscValidHeaderSpecific(daf,DM_CLASSID,2);
+  PetscValidHeaderSpecificType(dac,DM_CLASSID,1,DMDA);
+  PetscValidHeaderSpecificType(daf,DM_CLASSID,2,DMDA);
   PetscValidPointer(rest,3);
 
   ierr = DMDAGetInfo(dac,&dimc,&Mc,&Nc,&Pc,&mc,&nc,&pc,&dofc,&sc,&bxc,&byc,&bzc,&stc);CHKERRQ(ierr);
