@@ -1,4 +1,3 @@
-
 /*
   This file defines an additive Schwarz preconditioner for any Mat implementation.
 
@@ -21,7 +20,7 @@ typedef struct {
   VecScatter *lrestriction;       /* mapping from subregion to overlapping (process) subdomain */
   VecScatter *lprolongation;      /* mapping from non-overlapping subregion to overlapping (process) subdomain; used for restrict additive version of algorithms */
   Vec        lx, ly;              /* work vectors */
-  Vec        *x,*y;  		  /* work vectors */
+  Vec        *x,*y;               /* work vectors */
   IS         lis;                 /* index set that defines each overlapping multiplicative (process) subdomain */
   IS         *is;                 /* index set that defines each overlapping subdomain */
   IS         *is_local;           /* index set that defines each non-overlapping subdomain, may be NULL */
@@ -121,8 +120,10 @@ static PetscErrorCode PCASMPrintSubdomains(PC pc)
       ierr = ISGetLocalSize(osm->is[i],&nidx);CHKERRQ(ierr);
       ierr = ISGetIndices(osm->is[i],&idx);CHKERRQ(ierr);
       /* Print to a string viewer; no more than 15 characters per index plus 512 char for the header.*/
-      ierr = PetscMalloc1(16*(nidx+1)+512, &s);CHKERRQ(ierr);
-      ierr = PetscViewerStringOpen(PETSC_COMM_SELF, s, 16*(nidx+1)+512, &sviewer);CHKERRQ(ierr);
+#define len  16*(nidx+1)+512
+      ierr = PetscMalloc1(len,&s);CHKERRQ(ierr);
+      ierr = PetscViewerStringOpen(PETSC_COMM_SELF, s, len, &sviewer);CHKERRQ(ierr);
+#undef len
       ierr = PetscViewerStringSPrintf(sviewer, "[%D:%D] Subdomain %D with overlap:\n", rank, size, i);CHKERRQ(ierr);
       for (j=0; j<nidx; j++) {
         ierr = PetscViewerStringSPrintf(sviewer,"%D ",idx[j]);CHKERRQ(ierr);
@@ -137,8 +138,10 @@ static PetscErrorCode PCASMPrintSubdomains(PC pc)
       ierr = PetscFree(s);CHKERRQ(ierr);
       if (osm->is_local) {
         /* Print to a string viewer; no more than 15 characters per index plus 512 char for the header.*/
-        ierr = PetscMalloc1(16*(nidx+1)+512, &s);CHKERRQ(ierr);
-        ierr = PetscViewerStringOpen(PETSC_COMM_SELF, s, 16*(nidx+1)+512, &sviewer);CHKERRQ(ierr);
+#define len  16*(nidx+1)+512
+        ierr = PetscMalloc1(len, &s);CHKERRQ(ierr);
+        ierr = PetscViewerStringOpen(PETSC_COMM_SELF, s, len, &sviewer);CHKERRQ(ierr);
+#undef len
         ierr = PetscViewerStringSPrintf(sviewer, "[%D:%D] Subdomain %D without overlap:\n", rank, size, i);CHKERRQ(ierr);
         ierr = ISGetLocalSize(osm->is_local[i],&nidx);CHKERRQ(ierr);
         ierr = ISGetIndices(osm->is_local[i],&idx);CHKERRQ(ierr);
@@ -176,7 +179,7 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
 {
   PC_ASM         *osm = (PC_ASM*)pc->data;
   PetscErrorCode ierr;
-  PetscBool      symset,flg;
+  PetscBool      flg;
   PetscInt       i,m,m_local;
   MatReuse       scall = MAT_REUSE_MATRIX;
   IS             isl;
@@ -190,11 +193,6 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
   if (!pc->setupcalled) {
     PetscInt m;
 
-    if (!osm->type_set) {
-      ierr = MatIsSymmetricKnown(pc->pmat,&symset,&flg);CHKERRQ(ierr);
-      if (symset && flg) osm->type = PC_ASM_BASIC;
-    }
-
     /* Note: if subdomains have been set either via PCASMSetTotalSubdomains() or via PCASMSetLocalSubdomains(), osm->n_local_true will not be PETSC_DECIDE */
     if (osm->n_local_true == PETSC_DECIDE) {
       /* no subdomains given */
@@ -204,9 +202,9 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
         char      **domain_names;
         IS        *inner_domain_is, *outer_domain_is;
         ierr = DMCreateDomainDecomposition(pc->dm, &num_domains, &domain_names, &inner_domain_is, &outer_domain_is, &domain_dm);CHKERRQ(ierr);
-        osm->overlap = -1; /* We do not want to increase the overlap of the IS. 
-                              A future improvement of this code might allow one to use 
-                              DM-defined subdomains and also increase the overlap, 
+        osm->overlap = -1; /* We do not want to increase the overlap of the IS.
+                              A future improvement of this code might allow one to use
+                              DM-defined subdomains and also increase the overlap,
                               but that is not currently supported */
         if (num_domains) {
           ierr = PCASMSetLocalSubdomains(pc, num_domains, outer_domain_is, inner_domain_is);CHKERRQ(ierr);
@@ -239,7 +237,7 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
       if (outwork.max == 1 && outwork.sum == size) {
         /* osm->n_local_true = 1 on all processes, set this option may enable use of optimized MatCreateSubMatrices() implementation */
         ierr = MatSetOption(pc->pmat,MAT_SUBMAT_SINGLEIS,PETSC_TRUE);CHKERRQ(ierr);
-      } 
+      }
     }
     if (!osm->is) { /* create the index sets */
       ierr = PCASMCreateSubdomains(pc->pmat,osm->n_local_true,&osm->is);CHKERRQ(ierr);
@@ -301,13 +299,13 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
         ierr = PetscFree(domain_dm);CHKERRQ(ierr);
       }
     }
-       
+
     ierr = ISConcatenate(PETSC_COMM_SELF, osm->n_local_true, osm->is, &osm->lis);CHKERRQ(ierr);
     ierr = ISSortRemoveDups(osm->lis);CHKERRQ(ierr);
     ierr = ISGetLocalSize(osm->lis, &m);CHKERRQ(ierr);
     ierr = VecCreateSeq(PETSC_COMM_SELF, m, &osm->lx);CHKERRQ(ierr);
     ierr = VecDuplicate(osm->lx, &osm->ly);CHKERRQ(ierr);
-    
+
     scall = MAT_INITIAL_MATRIX;
   } else {
     /*
@@ -341,21 +339,21 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
   if(!pc->setupcalled){
     /* Create the local work vectors (from the local matrices) and scatter contexts */
     ierr = MatCreateVecs(pc->pmat,&vec,0);CHKERRQ(ierr);
-    
-    if (osm->is_local && (osm->type == PC_ASM_INTERPOLATE || osm->type == PC_ASM_NONE )) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"Cannot use interpolate or none PCASMType if is_local was provided to PCASMSetLocalSubdomains()"); 
+
+    if (osm->is_local && (osm->type == PC_ASM_INTERPOLATE || osm->type == PC_ASM_NONE )) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"Cannot use interpolate or none PCASMType if is_local was provided to PCASMSetLocalSubdomains()");
     if (osm->is_local && osm->type == PC_ASM_RESTRICT && osm->loctype == PC_COMPOSITE_ADDITIVE) {
       ierr = PetscMalloc1(osm->n_local_true,&osm->lprolongation);CHKERRQ(ierr);
     }
     ierr = PetscMalloc1(osm->n_local_true,&osm->lrestriction);CHKERRQ(ierr);
     ierr = PetscMalloc1(osm->n_local_true,&osm->x);CHKERRQ(ierr);
     ierr = PetscMalloc1(osm->n_local_true,&osm->y);CHKERRQ(ierr);
-        
+
     ierr = ISGetLocalSize(osm->lis,&m);CHKERRQ(ierr);
     ierr = ISCreateStride(PETSC_COMM_SELF,m,0,1,&isl);CHKERRQ(ierr);
     ierr = VecScatterCreate(vec,osm->lis,osm->lx,isl,&osm->restriction);CHKERRQ(ierr);
     ierr = ISDestroy(&isl);CHKERRQ(ierr);
-    
-    
+
+
     for (i=0; i<osm->n_local_true; ++i) {
       ISLocalToGlobalMapping ltog;
       IS                     isll;
@@ -365,7 +363,7 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
       ierr = ISGetLocalSize(osm->is[i],&m);CHKERRQ(ierr);
       ierr = MatCreateVecs(osm->pmat[i],&osm->x[i],NULL);CHKERRQ(ierr);
       ierr = VecDuplicate(osm->x[i],&osm->y[i]);CHKERRQ(ierr);
-      
+
       /* generate a scatter from ly to y[i] picking all the overlapping is[i] entries */
       ierr = ISLocalToGlobalMappingCreateIS(osm->lis,&ltog);CHKERRQ(ierr);
       ierr = ISGetLocalSize(osm->is[i],&m);CHKERRQ(ierr);
@@ -381,34 +379,34 @@ static PetscErrorCode PCSetUp_ASM(PC pc)
       ierr = ISDestroy(&isll);CHKERRQ(ierr);
       ierr = ISDestroy(&isl);CHKERRQ(ierr);
       if (osm->lprolongation) { /* generate a scatter from y[i] to ly picking only the the non-overalapping is_local[i] entries */
-	ISLocalToGlobalMapping ltog;
+        ISLocalToGlobalMapping ltog;
         IS                     isll,isll_local;
         const PetscInt         *idx_local;
         PetscInt               *idx1, *idx2, nout;
-        
+
         ierr = ISGetLocalSize(osm->is_local[i],&m_local);CHKERRQ(ierr);
         ierr = ISGetIndices(osm->is_local[i], &idx_local);CHKERRQ(ierr);
-        
-	ierr = ISLocalToGlobalMappingCreateIS(osm->is[i],&ltog);CHKERRQ(ierr);
-	ierr = PetscMalloc1(m_local,&idx1);CHKERRQ(ierr);
-	ierr = ISGlobalToLocalMappingApply(ltog,IS_GTOLM_DROP,m_local,idx_local,&nout,idx1);CHKERRQ(ierr);
+
+        ierr = ISLocalToGlobalMappingCreateIS(osm->is[i],&ltog);CHKERRQ(ierr);
+        ierr = PetscMalloc1(m_local,&idx1);CHKERRQ(ierr);
+        ierr = ISGlobalToLocalMappingApply(ltog,IS_GTOLM_DROP,m_local,idx_local,&nout,idx1);CHKERRQ(ierr);
         ierr = ISLocalToGlobalMappingDestroy(&ltog);CHKERRQ(ierr);
         if (nout != m_local) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"is_local not a subset of is");
         ierr = ISCreateGeneral(PETSC_COMM_SELF,m_local,idx1,PETSC_OWN_POINTER,&isll);CHKERRQ(ierr);
-	
-	ierr = ISLocalToGlobalMappingCreateIS(osm->lis,&ltog);CHKERRQ(ierr);
-	ierr = PetscMalloc1(m_local,&idx2);CHKERRQ(ierr);
-      	ierr = ISGlobalToLocalMappingApply(ltog,IS_GTOLM_DROP,m_local,idx_local,&nout,idx2);CHKERRQ(ierr);
+
+        ierr = ISLocalToGlobalMappingCreateIS(osm->lis,&ltog);CHKERRQ(ierr);
+        ierr = PetscMalloc1(m_local,&idx2);CHKERRQ(ierr);
+        ierr = ISGlobalToLocalMappingApply(ltog,IS_GTOLM_DROP,m_local,idx_local,&nout,idx2);CHKERRQ(ierr);
         ierr = ISLocalToGlobalMappingDestroy(&ltog);CHKERRQ(ierr);
         if (nout != m_local) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"is_local not a subset of lis");
         ierr = ISCreateGeneral(PETSC_COMM_SELF,m_local,idx2,PETSC_OWN_POINTER,&isll_local);CHKERRQ(ierr);
-	
-	ierr = ISRestoreIndices(osm->is_local[i], &idx_local);CHKERRQ(ierr);
-	ierr = VecScatterCreate(osm->y[i],isll,osm->ly,isll_local,&osm->lprolongation[i]);CHKERRQ(ierr);
-	
+
+        ierr = ISRestoreIndices(osm->is_local[i], &idx_local);CHKERRQ(ierr);
+        ierr = VecScatterCreate(osm->y[i],isll,osm->ly,isll_local,&osm->lprolongation[i]);CHKERRQ(ierr);
+
         ierr = ISDestroy(&isll);CHKERRQ(ierr);
-	ierr = ISDestroy(&isll_local);CHKERRQ(ierr);
-      }    
+        ierr = ISDestroy(&isll_local);CHKERRQ(ierr);
+      }
     }
     ierr = VecDestroy(&vec);CHKERRQ(ierr);
   }
@@ -450,7 +448,7 @@ static PetscErrorCode PCSetUpOnBlocks_ASM(PC pc)
   for (i=0; i<osm->n_local_true; i++) {
     ierr = KSPSetUp(osm->ksp[i]);CHKERRQ(ierr);
     ierr = KSPGetConvergedReason(osm->ksp[i],&reason);CHKERRQ(ierr);
-    if (reason == KSP_DIVERGED_PCSETUP_FAILED) {
+    if (reason == KSP_DIVERGED_PC_FAILED) {
       pc->failedreason = PC_SUBPC_ERROR;
     }
   }
@@ -477,45 +475,48 @@ static PetscErrorCode PCApply_ASM(PC pc,Vec x,Vec y)
   if (!(osm->type & PC_ASM_INTERPOLATE)) {
     reverse = SCATTER_REVERSE_LOCAL;
   }
-    
+
   if(osm->loctype == PC_COMPOSITE_MULTIPLICATIVE || osm->loctype == PC_COMPOSITE_ADDITIVE){
     /* zero the global and the local solutions */
     ierr = VecZeroEntries(y);CHKERRQ(ierr);
     ierr = VecSet(osm->ly, 0.0);CHKERRQ(ierr);
-    
+
     /* Copy the global RHS to local RHS including the ghost nodes */
     ierr = VecScatterBegin(osm->restriction, x, osm->lx, INSERT_VALUES, forward);CHKERRQ(ierr);
     ierr = VecScatterEnd(osm->restriction, x, osm->lx, INSERT_VALUES, forward);CHKERRQ(ierr);
-    
+
     /* Restrict local RHS to the overlapping 0-block RHS */
     ierr = VecScatterBegin(osm->lrestriction[0], osm->lx, osm->x[0], INSERT_VALUES, forward);CHKERRQ(ierr);
     ierr = VecScatterEnd(osm->lrestriction[0], osm->lx, osm->x[0],  INSERT_VALUES, forward);CHKERRQ(ierr);
-    
+
     /* do the local solves */
     for (i = 0; i < n_local_true; ++i) {
-      
+
       /* solve the overlapping i-block */
+      ierr = PetscLogEventBegin(PC_ApplyOnBlocks,osm->ksp[i],osm->x[i],osm->y[i],0);CHKERRQ(ierr);
       ierr = KSPSolve(osm->ksp[i], osm->x[i], osm->y[i]);CHKERRQ(ierr);
+      ierr = KSPCheckSolve(osm->ksp[i],pc,osm->y[i]);CHKERRQ(ierr);
+      ierr = PetscLogEventEnd(PC_ApplyOnBlocks,osm->ksp[i],osm->x[i],osm->y[i],0);CHKERRQ(ierr);
 
       if (osm->lprolongation) { /* interpolate the non-overalapping i-block solution to the local solution (only for restrictive additive) */
- 	ierr = VecScatterBegin(osm->lprolongation[i], osm->y[i], osm->ly, ADD_VALUES, forward);CHKERRQ(ierr);
- 	ierr = VecScatterEnd(osm->lprolongation[i], osm->y[i], osm->ly, ADD_VALUES, forward);CHKERRQ(ierr);
+        ierr = VecScatterBegin(osm->lprolongation[i], osm->y[i], osm->ly, ADD_VALUES, forward);CHKERRQ(ierr);
+        ierr = VecScatterEnd(osm->lprolongation[i], osm->y[i], osm->ly, ADD_VALUES, forward);CHKERRQ(ierr);
       }
       else{ /* interpolate the overalapping i-block solution to the local solution */
         ierr = VecScatterBegin(osm->lrestriction[i], osm->y[i], osm->ly, ADD_VALUES, reverse);CHKERRQ(ierr);
-	ierr = VecScatterEnd(osm->lrestriction[i], osm->y[i], osm->ly, ADD_VALUES, reverse);CHKERRQ(ierr);
+        ierr = VecScatterEnd(osm->lrestriction[i], osm->y[i], osm->ly, ADD_VALUES, reverse);CHKERRQ(ierr);
       }
-            
+
       if (i < n_local_true-1) {
-	/* Restrict local RHS to the overlapping (i+1)-block RHS */
-	ierr = VecScatterBegin(osm->lrestriction[i+1], osm->lx, osm->x[i+1], INSERT_VALUES, forward);CHKERRQ(ierr);
-	ierr = VecScatterEnd(osm->lrestriction[i+1], osm->lx, osm->x[i+1], INSERT_VALUES, forward);CHKERRQ(ierr);
-		
-	if ( osm->loctype == PC_COMPOSITE_MULTIPLICATIVE){
-	  /* udpdate the overlapping (i+1)-block RHS using the current local solution */
-	  ierr = MatMult(osm->lmats[i+1], osm->ly, osm->y[i+1]);CHKERRQ(ierr);
-	  ierr = VecAXPBY(osm->x[i+1],-1.,1., osm->y[i+1]); CHKERRQ(ierr);
-	}
+        /* Restrict local RHS to the overlapping (i+1)-block RHS */
+        ierr = VecScatterBegin(osm->lrestriction[i+1], osm->lx, osm->x[i+1], INSERT_VALUES, forward);CHKERRQ(ierr);
+        ierr = VecScatterEnd(osm->lrestriction[i+1], osm->lx, osm->x[i+1], INSERT_VALUES, forward);CHKERRQ(ierr);
+
+        if ( osm->loctype == PC_COMPOSITE_MULTIPLICATIVE){
+          /* update the overlapping (i+1)-block RHS using the current local solution */
+          ierr = MatMult(osm->lmats[i+1], osm->ly, osm->y[i+1]);CHKERRQ(ierr);
+          ierr = VecAXPBY(osm->x[i+1],-1.,1., osm->y[i+1]); CHKERRQ(ierr);
+        }
       }
     }
     /* Add the local solution to the global solution including the ghost nodes */
@@ -542,7 +543,7 @@ static PetscErrorCode PCApplyTranspose_ASM(PC pc,Vec x,Vec y)
      Note: these are reversed from the PCApply_ASM() because we are applying the
      transpose of the three terms
   */
-  
+
   if (!(osm->type & PC_ASM_INTERPOLATE)) {
     forward = SCATTER_FORWARD_LOCAL;
     /* have to zero the work RHS since scatter may leave some slots empty */
@@ -553,20 +554,23 @@ static PetscErrorCode PCApplyTranspose_ASM(PC pc,Vec x,Vec y)
   /* zero the global and the local solutions */
   ierr = VecZeroEntries(y);CHKERRQ(ierr);
   ierr = VecSet(osm->ly, 0.0);CHKERRQ(ierr);
-    
+
   /* Copy the global RHS to local RHS including the ghost nodes */
   ierr = VecScatterBegin(osm->restriction, x, osm->lx, INSERT_VALUES, forward);CHKERRQ(ierr);
   ierr = VecScatterEnd(osm->restriction, x, osm->lx, INSERT_VALUES, forward);CHKERRQ(ierr);
-    
+
   /* Restrict local RHS to the overlapping 0-block RHS */
   ierr = VecScatterBegin(osm->lrestriction[0], osm->lx, osm->x[0], INSERT_VALUES, forward);CHKERRQ(ierr);
   ierr = VecScatterEnd(osm->lrestriction[0], osm->lx, osm->x[0],  INSERT_VALUES, forward);CHKERRQ(ierr);
-    
+
   /* do the local solves */
   for (i = 0; i < n_local_true; ++i) {
-      
+
     /* solve the overlapping i-block */
-    ierr = KSPSolve(osm->ksp[i], osm->x[i], osm->y[i]);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(PC_ApplyOnBlocks,osm->ksp[i],osm->x[i],osm->y[i],0);CHKERRQ(ierr);
+    ierr = KSPSolveTranspose(osm->ksp[i], osm->x[i], osm->y[i]);CHKERRQ(ierr);
+    ierr = KSPCheckSolve(osm->ksp[i],pc,osm->y[i]);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(PC_ApplyOnBlocks,osm->ksp[i],osm->x[i],osm->y[i],0);CHKERRQ(ierr);
 
     if (osm->lprolongation) { /* interpolate the non-overalapping i-block solution to the local solution */
      ierr = VecScatterBegin(osm->lprolongation[i], osm->y[i], osm->ly, ADD_VALUES, forward);CHKERRQ(ierr);
@@ -576,7 +580,7 @@ static PetscErrorCode PCApplyTranspose_ASM(PC pc,Vec x,Vec y)
       ierr = VecScatterBegin(osm->lrestriction[i], osm->y[i], osm->ly, ADD_VALUES, reverse);CHKERRQ(ierr);
       ierr = VecScatterEnd(osm->lrestriction[i], osm->y[i], osm->ly, ADD_VALUES, reverse);CHKERRQ(ierr);
     }
-            
+
     if (i < n_local_true-1) {
       /* Restrict local RHS to the overlapping (i+1)-block RHS */
       ierr = VecScatterBegin(osm->lrestriction[i+1], osm->lx, osm->x[i+1], INSERT_VALUES, forward);CHKERRQ(ierr);
@@ -586,9 +590,9 @@ static PetscErrorCode PCApplyTranspose_ASM(PC pc,Vec x,Vec y)
   /* Add the local solution to the global solution including the ghost nodes */
   ierr = VecScatterBegin(osm->restriction, osm->ly, y,  ADD_VALUES, reverse);CHKERRQ(ierr);
   ierr = VecScatterEnd(osm->restriction,  osm->ly, y, ADD_VALUES, reverse);CHKERRQ(ierr);
-  
+
   PetscFunctionReturn(0);
-    
+
 }
 
 static PetscErrorCode PCReset_ASM(PC pc)
@@ -620,7 +624,7 @@ static PetscErrorCode PCReset_ASM(PC pc)
     if (osm->lprolongation) {ierr = PetscFree(osm->lprolongation);CHKERRQ(ierr);}
     ierr = PetscFree(osm->x);CHKERRQ(ierr);
     ierr = PetscFree(osm->y);CHKERRQ(ierr);
-    
+
   }
   ierr = PCASMDestroySubdomains(osm->n_local_true,osm->is,osm->is_local);CHKERRQ(ierr);
   ierr = ISDestroy(&osm->lis);CHKERRQ(ierr);
@@ -660,17 +664,12 @@ static PetscErrorCode PCSetFromOptions_ASM(PetscOptionItems *PetscOptionsObject,
   PC_ASM         *osm = (PC_ASM*)pc->data;
   PetscErrorCode ierr;
   PetscInt       blocks,ovl;
-  PetscBool      symset,flg;
+  PetscBool      flg;
   PCASMType      asmtype;
   PCCompositeType loctype;
   char           sub_mat_type[256];
 
   PetscFunctionBegin;
-  /* set the type to symmetric if matrix is symmetric */
-  if (!osm->type_set && pc->pmat) {
-    ierr = MatIsSymmetricKnown(pc->pmat,&symset,&flg);CHKERRQ(ierr);
-    if (symset && flg) osm->type = PC_ASM_BASIC;
-  }
   ierr = PetscOptionsHead(PetscOptionsObject,"Additive Schwarz options");CHKERRQ(ierr);
   ierr = PetscOptionsBool("-pc_asm_dm_subdomains","Use DMCreateDomainDecomposition() to define subdomains","PCASMSetDMSubdomains",osm->dm_subdomains,&osm->dm_subdomains,&flg);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-pc_asm_blocks","Number of subdomains","PCASMSetTotalSubdomains",osm->n,&blocks,&flg);CHKERRQ(ierr);
@@ -678,6 +677,11 @@ static PetscErrorCode PCSetFromOptions_ASM(PetscOptionItems *PetscOptionsObject,
     ierr = PCASMSetTotalSubdomains(pc,blocks,NULL,NULL);CHKERRQ(ierr);
     osm->dm_subdomains = PETSC_FALSE;
   }
+  ierr = PetscOptionsInt("-pc_asm_local_blocks","Number of local subdomains","PCASMSetLocalSubdomains",osm->n_local_true,&blocks,&flg);CHKERRQ(ierr);
+  if (flg) {
+    ierr = PCASMSetLocalSubdomains(pc,blocks,NULL,NULL);CHKERRQ(ierr);
+    osm->dm_subdomains = PETSC_FALSE;
+  }
   ierr = PetscOptionsInt("-pc_asm_overlap","Number of grid points overlap","PCASMSetOverlap",osm->overlap,&ovl,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PCASMSetOverlap(pc,ovl);CHKERRQ(ierr);
@@ -840,7 +844,7 @@ static PetscErrorCode  PCASMGetSubKSP_ASM(PC pc,PetscInt *n_local,PetscInt *firs
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (osm->n_local_true < 1) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ORDER,"Need to call PCSetUP() on PC (or KSPSetUp() on the outer KSP object) before calling here");
+  if (osm->n_local_true < 1) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ORDER,"Need to call PCSetUp() on PC (or KSPSetUp() on the outer KSP object) before calling here");
 
   if (n_local) *n_local = osm->n_local_true;
   if (first_local) {
@@ -882,7 +886,7 @@ static PetscErrorCode PCASMSetSubMatType_ASM(PC pc,MatType sub_mat_type)
 /*@C
     PCASMSetLocalSubdomains - Sets the local subdomains (for this processor only) for the additive Schwarz preconditioner.
 
-    Collective on PC
+    Collective on pc
 
     Input Parameters:
 +   pc - the preconditioner context
@@ -892,6 +896,11 @@ static PetscErrorCode PCASMSetSubMatType_ASM(PC pc,MatType sub_mat_type)
 -   is_local - the index sets that define the local part of the subdomains for this processor, not used unless PCASMType is PC_ASM_RESTRICT
          (or NULL to not provide these)
 
+    Options Database Key:
+    To set the total number of subdomain blocks rather than specify the
+    index sets, use the option
+.    -pc_asm_local_blocks <blks> - Sets local blocks
+
     Notes:
     The IS numbering is in the parallel, global numbering of the vector for both is and is_local
 
@@ -907,8 +916,6 @@ static PetscErrorCode PCASMSetSubMatType_ASM(PC pc,MatType sub_mat_type)
 
     Level: advanced
 
-.keywords: PC, ASM, set, local, subdomains, additive Schwarz
-
 .seealso: PCASMSetTotalSubdomains(), PCASMSetOverlap(), PCASMGetSubKSP(),
           PCASMCreateSubdomains2D(), PCASMGetLocalSubdomains(), PCASMType, PCASMSetType()
 @*/
@@ -927,7 +934,7 @@ PetscErrorCode  PCASMSetLocalSubdomains(PC pc,PetscInt n,IS is[],IS is_local[])
     additive Schwarz preconditioner.  Either all or no processors in the
     PC communicator must call this routine, with the same index sets.
 
-    Collective on PC
+    Collective on pc
 
     Input Parameters:
 +   pc - the preconditioner context
@@ -956,8 +963,6 @@ PetscErrorCode  PCASMSetLocalSubdomains(PC pc,PetscInt n,IS is[],IS is_local[])
 
     Level: advanced
 
-.keywords: PC, ASM, set, total, global, subdomains, additive Schwarz
-
 .seealso: PCASMSetLocalSubdomains(), PCASMSetOverlap(), PCASMGetSubKSP(),
           PCASMCreateSubdomains2D()
 @*/
@@ -976,7 +981,7 @@ PetscErrorCode  PCASMSetTotalSubdomains(PC pc,PetscInt N,IS is[],IS is_local[])
     additive Schwarz preconditioner.  Either all or no processors in the
     PC communicator must call this routine.
 
-    Logically Collective on PC
+    Logically Collective on pc
 
     Input Parameters:
 +   pc  - the preconditioner context
@@ -1008,8 +1013,6 @@ PetscErrorCode  PCASMSetTotalSubdomains(PC pc,PetscInt N,IS is[],IS is_local[])
 
     Level: intermediate
 
-.keywords: PC, ASM, set, overlap
-
 .seealso: PCASMSetTotalSubdomains(), PCASMSetLocalSubdomains(), PCASMGetSubKSP(),
           PCASMCreateSubdomains2D(), PCASMGetLocalSubdomains(), MatIncreaseOverlap()
 @*/
@@ -1028,7 +1031,7 @@ PetscErrorCode  PCASMSetOverlap(PC pc,PetscInt ovl)
     PCASMSetType - Sets the type of restriction and interpolation used
     for local problems in the additive Schwarz method.
 
-    Logically Collective on PC
+    Logically Collective on pc
 
     Input Parameters:
 +   pc  - the preconditioner context
@@ -1043,13 +1046,12 @@ PetscErrorCode  PCASMSetOverlap(PC pc,PetscInt ovl)
     Options Database Key:
 .   -pc_asm_type [basic,restrict,interpolate,none] - Sets ASM type
 
-    Notes: if the is_local arguments are passed to PCASMSetLocalSubdomains() then they are used when PC_ASM_RESTRICT has been selected
+    Notes:
+    if the is_local arguments are passed to PCASMSetLocalSubdomains() then they are used when PC_ASM_RESTRICT has been selected
     to limit the local processor interpolation
 
     Level: intermediate
 
-.keywords: PC, ASM, set, type
-
 .seealso: PCASMSetTotalSubdomains(), PCASMSetTotalSubdomains(), PCASMGetSubKSP(),
           PCASMCreateSubdomains2D(), PCASMType, PCASMSetLocalType(), PCASMGetLocalType()
 @*/
@@ -1068,7 +1070,7 @@ PetscErrorCode  PCASMSetType(PC pc,PCASMType type)
     PCASMGetType - Gets the type of restriction and interpolation used
     for local problems in the additive Schwarz method.
 
-    Logically Collective on PC
+    Logically Collective on pc
 
     Input Parameter:
 .   pc  - the preconditioner context
@@ -1088,8 +1090,6 @@ PetscErrorCode  PCASMSetType(PC pc,PCASMType type)
 
     Level: intermediate
 
-.keywords: PC, ASM, set, type
-
 .seealso: PCASMSetTotalSubdomains(), PCASMSetTotalSubdomains(), PCASMGetSubKSP(),
           PCASMCreateSubdomains2D(), PCASMType, PCASMSetType(), PCASMSetLocalType(), PCASMGetLocalType()
 @*/
@@ -1106,7 +1106,7 @@ PetscErrorCode  PCASMGetType(PC pc,PCASMType *type)
 /*@
   PCASMSetLocalType - Sets the type of composition used for local problems in the additive Schwarz method.
 
-  Logically Collective on PC
+  Logically Collective on pc
 
   Input Parameters:
 + pc  - the preconditioner context
@@ -1137,7 +1137,7 @@ PetscErrorCode PCASMSetLocalType(PC pc, PCCompositeType type)
 /*@
   PCASMGetLocalType - Gets the type of composition used for local problems in the additive Schwarz method.
 
-  Logically Collective on PC
+  Logically Collective on pc
 
   Input Parameter:
 . pc  - the preconditioner context
@@ -1170,7 +1170,7 @@ PetscErrorCode PCASMGetLocalType(PC pc, PCCompositeType *type)
 /*@
     PCASMSetSortIndices - Determines whether subdomain indices are sorted.
 
-    Logically Collective on PC
+    Logically Collective on pc
 
     Input Parameters:
 +   pc  - the preconditioner context
@@ -1178,8 +1178,6 @@ PetscErrorCode PCASMGetLocalType(PC pc, PCCompositeType *type)
 
     Level: intermediate
 
-.keywords: PC, ASM, set, type
-
 .seealso: PCASMSetLocalSubdomains(), PCASMSetTotalSubdomains(), PCASMGetSubKSP(),
           PCASMCreateSubdomains2D()
 @*/
@@ -1198,7 +1196,7 @@ PetscErrorCode  PCASMSetSortIndices(PC pc,PetscBool doSort)
    PCASMGetSubKSP - Gets the local KSP contexts for all blocks on
    this processor.
 
-   Collective on PC iff first_local is requested
+   Collective on pc iff first_local is requested
 
    Input Parameter:
 .  pc - the preconditioner context
@@ -1219,8 +1217,6 @@ PetscErrorCode  PCASMSetSortIndices(PC pc,PetscBool doSort)
 
    Level: advanced
 
-.keywords: PC, ASM, additive Schwarz, get, sub, KSP, context
-
 .seealso: PCASMSetTotalSubdomains(), PCASMSetTotalSubdomains(), PCASMSetOverlap(),
           PCASMCreateSubdomains2D(),
 @*/
@@ -1249,7 +1245,8 @@ PetscErrorCode  PCASMGetSubKSP(PC pc,PetscInt *n_local,PetscInt *first_local,KSP
       will get a different convergence rate due to the default option of -pc_asm_type restrict. Use
       -pc_asm_type basic to use the standard ASM.
 
-   Notes: Each processor can have one or more blocks, but a block cannot be shared by more
+   Notes:
+    Each processor can have one or more blocks, but a block cannot be shared by more
      than one processor. Use PCGASM for subdomains shared by multiple processes. Defaults to one block per processor.
 
      To set options on the solvers for each block append -sub_ to all the KSP, and PC
@@ -1261,17 +1258,15 @@ PetscErrorCode  PCASMGetSubKSP(PC pc,PetscInt *n_local,PetscInt *first_local,KSP
 
    Level: beginner
 
-   Concepts: additive Schwarz method
-
     References:
 +   1. - M Dryja, OB Widlund, An additive variant of the Schwarz alternating method for the case of many subregions
      Courant Institute, New York University Technical report
--   1. - Barry Smith, Petter Bjorstad, and William Gropp, Domain Decompositions: Parallel Multilevel Methods for Elliptic Partial Differential Equations,
+-   2. - Barry Smith, Petter Bjorstad, and William Gropp, Domain Decompositions: Parallel Multilevel Methods for Elliptic Partial Differential Equations,
     Cambridge University Press.
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC,
            PCBJACOBI, PCASMGetSubKSP(), PCASMSetLocalSubdomains(), PCASMType, PCASMGetType(), PCASMSetLocalType(), PCASMGetLocalType()
-           PCASMSetTotalSubdomains(), PCSetModifySubmatrices(), PCASMSetOverlap(), PCASMSetType(), PCCompositeType
+           PCASMSetTotalSubdomains(), PCSetModifySubMatrices(), PCASMSetOverlap(), PCASMSetType(), PCCompositeType
 
 M*/
 
@@ -1349,8 +1344,6 @@ PETSC_EXTERN PetscErrorCode PCCreate_ASM(PC pc)
 
     In the Fortran version you must provide the array outis[] already allocated of length n.
 
-.keywords: PC, ASM, additive Schwarz, create, subdomains, unstructured grid
-
 .seealso: PCASMSetLocalSubdomains(), PCASMDestroySubdomains()
 @*/
 PetscErrorCode  PCASMCreateSubdomains(Mat A, PetscInt n, IS* outis[])
@@ -1380,8 +1373,8 @@ PetscErrorCode  PCASMCreateSubdomains(Mat A, PetscInt n, IS* outis[])
     ierr = MatGetDiagonalBlock(A,&Ad);CHKERRQ(ierr);
   }
   if (Ad) {
-    ierr = PetscObjectTypeCompare((PetscObject)Ad,MATSEQBAIJ,&isbaij);CHKERRQ(ierr);
-    if (!isbaij) {ierr = PetscObjectTypeCompare((PetscObject)Ad,MATSEQSBAIJ,&isbaij);CHKERRQ(ierr);}
+    ierr = PetscObjectBaseTypeCompare((PetscObject)Ad,MATSEQBAIJ,&isbaij);CHKERRQ(ierr);
+    if (!isbaij) {ierr = PetscObjectBaseTypeCompare((PetscObject)Ad,MATSEQSBAIJ,&isbaij);CHKERRQ(ierr);}
   }
   if (Ad && n > 1) {
     PetscBool match,done;
@@ -1521,8 +1514,6 @@ PetscErrorCode  PCASMCreateSubdomains(Mat A, PetscInt n, IS* outis[])
 
    Level: advanced
 
-.keywords: PC, ASM, additive Schwarz, create, subdomains, unstructured grid
-
 .seealso: PCASMCreateSubdomains(), PCASMSetLocalSubdomains()
 @*/
 PetscErrorCode  PCASMDestroySubdomains(PetscInt n, IS is[], IS is_local[])
@@ -1569,8 +1560,6 @@ PetscErrorCode  PCASMDestroySubdomains(PetscInt n, IS is[], IS is_local[])
 
    Level: advanced
 
-.keywords: PC, ASM, additive Schwarz, create, subdomains, 2D, regular grid
-
 .seealso: PCASMSetTotalSubdomains(), PCASMSetLocalSubdomains(), PCASMGetSubKSP(),
           PCASMSetOverlap()
 @*/
@@ -1649,8 +1638,6 @@ PetscErrorCode  PCASMCreateSubdomains2D(PetscInt m,PetscInt n,PetscInt M,PetscIn
 
     Level: advanced
 
-.keywords: PC, ASM, set, local, subdomains, additive Schwarz
-
 .seealso: PCASMSetTotalSubdomains(), PCASMSetOverlap(), PCASMGetSubKSP(),
           PCASMCreateSubdomains2D(), PCASMSetLocalSubdomains(), PCASMGetLocalSubmatrices()
 @*/
@@ -1687,14 +1674,13 @@ PetscErrorCode  PCASMGetLocalSubdomains(PC pc,PetscInt *n,IS *is[],IS *is_local[
 
     Level: advanced
 
-    Notes: Call after PCSetUp() (or KSPSetUp()) but before PCApply() (or KSPApply()) and before PCSetUpOnBlocks())
+    Notes:
+    Call after PCSetUp() (or KSPSetUp()) but before PCApply() and before PCSetUpOnBlocks())
 
-           Usually one would use PCSetModifySubmatrices() to change the submatrices in building the preconditioner.
-
-.keywords: PC, ASM, set, local, subdomains, additive Schwarz, block Jacobi
+           Usually one would use PCSetModifySubMatrices() to change the submatrices in building the preconditioner.
 
 .seealso: PCASMSetTotalSubdomains(), PCASMSetOverlap(), PCASMGetSubKSP(),
-          PCASMCreateSubdomains2D(), PCASMSetLocalSubdomains(), PCASMGetLocalSubdomains(), PCSetModifySubmatrices()
+          PCASMCreateSubdomains2D(), PCASMSetLocalSubdomains(), PCASMGetLocalSubdomains(), PCSetModifySubMatrices()
 @*/
 PetscErrorCode  PCASMGetLocalSubmatrices(PC pc,PetscInt *n,Mat *mat[])
 {
@@ -1706,7 +1692,7 @@ PetscErrorCode  PCASMGetLocalSubmatrices(PC pc,PetscInt *n,Mat *mat[])
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   PetscValidIntPointer(n,2);
   if (mat) PetscValidPointer(mat,3);
-  if (!pc->setupcalled) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must call after KSPSetUP() or PCSetUp().");
+  if (!pc->setupcalled) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must call after KSPSetUp() or PCSetUp().");
   ierr = PetscObjectTypeCompare((PetscObject)pc,PCASM,&match);CHKERRQ(ierr);
   if (!match) {
     if (n) *n = 0;
@@ -1737,8 +1723,6 @@ PetscErrorCode  PCASMGetLocalSubmatrices(PC pc,PetscInt *n,Mat *mat[])
     PCASMSetTotalSubdomains() and PCASMSetOverlap() take precedence over PCASMSetDMSubdomains(),
     so setting either of the first two effectively turns the latter off.
 
-.keywords: PC, ASM, DM, set, subdomains, additive Schwarz
-
 .seealso: PCASMGetDMSubdomains(), PCASMSetTotalSubdomains(), PCASMSetOverlap()
           PCASMCreateSubdomains2D(), PCASMSetLocalSubdomains(), PCASMGetLocalSubdomains()
 @*/
@@ -1771,8 +1755,6 @@ PetscErrorCode  PCASMSetDMSubdomains(PC pc,PetscBool flg)
 
     Level: intermediate
 
-.keywords: PC, ASM, DM, set, subdomains, additive Schwarz
-
 .seealso: PCASMSetDMSubdomains(), PCASMSetTotalSubdomains(), PCASMSetOverlap()
           PCASMCreateSubdomains2D(), PCASMSetLocalSubdomains(), PCASMGetLocalSubdomains()
 @*/
@@ -1784,7 +1766,7 @@ PetscErrorCode  PCASMGetDMSubdomains(PC pc,PetscBool* flg)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  PetscValidPointer(flg,2);
+  PetscValidBoolPointer(flg,2);
   ierr = PetscObjectTypeCompare((PetscObject)pc,PCASM,&match);CHKERRQ(ierr);
   if (match) {
     if (flg) *flg = osm->dm_subdomains;
@@ -1805,8 +1787,6 @@ PetscErrorCode  PCASMGetDMSubdomains(PC pc,PetscBool* flg)
 
    Level: advanced
 
-.keywords: PC, PCASM, MatType, set
-
 .seealso: PCASMSetSubMatType(), PCASM, PCSetType(), VecSetType(), MatType, Mat
 @*/
 PetscErrorCode  PCASMGetSubMatType(PC pc,MatType *sub_mat_type){
@@ -1833,8 +1813,6 @@ PetscErrorCode  PCASMGetSubMatType(PC pc,MatType *sub_mat_type){
 
   Level: advanced
 
-.keywords: PC, PCASM, MatType, set
-
 .seealso: PCASMGetSubMatType(), PCASM, PCSetType(), VecSetType(), MatType, Mat
 @*/
 PetscErrorCode PCASMSetSubMatType(PC pc,MatType sub_mat_type)
