@@ -1,5 +1,5 @@
-#include "private/fortranimpl.h"
-#include "petscksp.h"
+#include <petsc/private/fortranimpl.h>
+#include <petscksp.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define pcbjacobigetsubksp_        PCBJACOBIGETSUBKSP
@@ -7,21 +7,18 @@
 #define pcbjacobigetsubksp_        pcbjacobigetsubksp
 #endif
 
-EXTERN_C_BEGIN
-void PETSC_STDCALL pcbjacobigetsubksp_(PC *pc,PetscInt *n_local,PetscInt *first_local,KSP *ksp,PetscErrorCode *ierr)
+PETSC_EXTERN void pcbjacobigetsubksp_(PC *pc,PetscInt *n_local,PetscInt *first_local,KSP *ksp,PetscErrorCode *ierr)
 {
-  KSP *tksp;
-  PetscInt  i,nloc;
+  KSP      *tksp;
+  PetscInt i,nloc;
   CHKFORTRANNULLINTEGER(n_local);
   CHKFORTRANNULLINTEGER(first_local);
-  *ierr = PCBJacobiGetSubKSP(*pc,&nloc,first_local,&tksp);
+  *ierr = PCBJacobiGetSubKSP(*pc,&nloc,first_local,&tksp); if (*ierr) return;
   if (n_local) *n_local = nloc;
   CHKFORTRANNULLOBJECT(ksp);
   if (ksp) {
-    for (i=0; i<nloc; i++){
+    for (i=0; i<nloc; i++) {
       ksp[i] = tksp[i];
     }
   }
 }
-
-EXTERN_C_END
