@@ -22,17 +22,6 @@ static PetscErrorCode PCSetFromOptions_Cholesky(PetscOptionItems *PetscOptionsOb
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode PCView_Cholesky(PC pc,PetscViewer viewer)
-{
-  PetscErrorCode ierr;
-  PetscBool      iascii;
-
-  PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
-  ierr = PCView_Factor(pc,viewer);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 static PetscErrorCode PCSetUp_Cholesky(PC pc)
 {
   PetscErrorCode         ierr;
@@ -178,6 +167,34 @@ static PetscErrorCode PCApply_Cholesky(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode PCApplySymmetricLeft_Cholesky(PC pc,Vec x,Vec y)
+{
+  PC_Cholesky    *dir = (PC_Cholesky*)pc->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (dir->hdr.inplace) {
+    ierr = MatForwardSolve(pc->pmat,x,y);CHKERRQ(ierr);
+  } else {
+    ierr = MatForwardSolve(((PC_Factor*)dir)->fact,x,y);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCApplySymmetricRight_Cholesky(PC pc,Vec x,Vec y)
+{
+  PC_Cholesky    *dir = (PC_Cholesky*)pc->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (dir->hdr.inplace) {
+    ierr = MatBackwardSolve(pc->pmat,x,y);CHKERRQ(ierr);
+  } else {
+    ierr = MatBackwardSolve(((PC_Factor*)dir)->fact,x,y);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode PCApplyTranspose_Cholesky(PC pc,Vec x,Vec y)
 {
   PC_Cholesky    *dir = (PC_Cholesky*)pc->data;
@@ -212,8 +229,6 @@ static PetscErrorCode PCApplyTranspose_Cholesky(PC pc,Vec x,Vec y)
 
    Level: intermediate
 
-.keywords: PC, levels, reordering, factorization, incomplete, LU
-
 .seealso: PCFactorSetReuseFill()
 @*/
 PetscErrorCode  PCFactorSetReuseOrdering(PC pc,PetscBool flag)
@@ -238,13 +253,13 @@ PetscErrorCode  PCFactorSetReuseOrdering(PC pc,PetscBool flag)
 .  -pc_factor_in_place - Activates in-place factorization
 -  -pc_factor_mat_ordering_type <nd,rcm,...> - Sets ordering routine
 
-   Notes: Not all options work for all matrix formats
+   Notes:
+    Not all options work for all matrix formats
 
    Level: beginner
 
-   Concepts: Cholesky factorization, direct solver
-
-   Notes: Usually this will compute an "exact" solution in one iteration and does
+   Notes:
+    Usually this will compute an "exact" solution in one iteration and does
           not need a Krylov method (i.e. you can use -ksp_type preonly, or
           KSPSetType(ksp,KSPPREONLY) for the Krylov method
 
@@ -271,15 +286,18 @@ PETSC_EXTERN PetscErrorCode PCCreate_Cholesky(PC pc)
   dir->col = 0;
   dir->row = 0;
 
-  ierr = PetscStrallocpy(MATORDERINGNATURAL,(char**)&((PC_Factor*)dir)->ordering);CHKERRQ(ierr);
+  /* MATORDERINGNATURAL_OR_ND allows selecting type based on matrix type sbaij or aij */
+  ierr = PetscStrallocpy(MATORDERINGNATURAL_OR_ND,(char**)&((PC_Factor*)dir)->ordering);CHKERRQ(ierr);
 
-  pc->ops->destroy           = PCDestroy_Cholesky;
-  pc->ops->reset             = PCReset_Cholesky;
-  pc->ops->apply             = PCApply_Cholesky;
-  pc->ops->applytranspose    = PCApplyTranspose_Cholesky;
-  pc->ops->setup             = PCSetUp_Cholesky;
-  pc->ops->setfromoptions    = PCSetFromOptions_Cholesky;
-  pc->ops->view              = PCView_Cholesky;
-  pc->ops->applyrichardson   = 0;
+  pc->ops->destroy             = PCDestroy_Cholesky;
+  pc->ops->reset               = PCReset_Cholesky;
+  pc->ops->apply               = PCApply_Cholesky;
+  pc->ops->applysymmetricleft  = PCApplySymmetricLeft_Cholesky;
+  pc->ops->applysymmetricright = PCApplySymmetricRight_Cholesky;
+  pc->ops->applytranspose      = PCApplyTranspose_Cholesky;
+  pc->ops->setup               = PCSetUp_Cholesky;
+  pc->ops->setfromoptions      = PCSetFromOptions_Cholesky;
+  pc->ops->view                = PCView_Factor;
+  pc->ops->applyrichardson     = 0;
   PetscFunctionReturn(0);
 }
