@@ -1,5 +1,4 @@
 #include <../src/ksp/pc/impls/gamg/gamg.h>        /*I "petscpc.h" I*/
-#include <petsc/private/kspimpl.h>
 #include <petscsf.h>
 
 PetscFunctionList PCGAMGClassicalProlongatorList    = NULL;
@@ -11,8 +10,6 @@ typedef struct {
   PetscInt  nsmooths;         /* number of jacobi smoothings on the prolongator */
 } PC_GAMG_Classical;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGClassicalSetType"
 /*@C
    PCGAMGClassicalSetType - Sets the type of classical interpolation to use
 
@@ -34,12 +31,10 @@ PetscErrorCode PCGAMGClassicalSetType(PC pc, PCGAMGClassicalType type)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  ierr = PetscTryMethod(pc,"PCGAMGClassicalSetType_C",(PC,PCGAMGType),(pc,type));CHKERRQ(ierr);
+  ierr = PetscTryMethod(pc,"PCGAMGClassicalSetType_C",(PC,PCGAMGClassicalType),(pc,type));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGClassicalGetType"
 /*@C
    PCGAMGClassicalGetType - Gets the type of classical interpolation to use
 
@@ -61,12 +56,10 @@ PetscErrorCode PCGAMGClassicalGetType(PC pc, PCGAMGClassicalType *type)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  ierr = PetscUseMethod(pc,"PCGAMGClassicalGetType_C",(PC,PCGAMGType*),(pc,type));CHKERRQ(ierr);
+  ierr = PetscUseMethod(pc,"PCGAMGClassicalGetType_C",(PC,PCGAMGClassicalType*),(pc,type));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGClassicalSetType_GAMG"
 static PetscErrorCode PCGAMGClassicalSetType_GAMG(PC pc, PCGAMGClassicalType type)
 {
   PetscErrorCode    ierr;
@@ -79,8 +72,6 @@ static PetscErrorCode PCGAMGClassicalSetType_GAMG(PC pc, PCGAMGClassicalType typ
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGClassicalGetType_GAMG"
 static PetscErrorCode PCGAMGClassicalGetType_GAMG(PC pc, PCGAMGClassicalType *type)
 {
   PC_MG             *mg          = (PC_MG*)pc->data;
@@ -92,8 +83,6 @@ static PetscErrorCode PCGAMGClassicalGetType_GAMG(PC pc, PCGAMGClassicalType *ty
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGGraph_Classical"
 PetscErrorCode PCGAMGGraph_Classical(PC pc,Mat A,Mat *G)
 {
   PetscInt          s,f,n,idx,lidx,gidx;
@@ -104,17 +93,14 @@ PetscErrorCode PCGAMGGraph_Classical(PC pc,Mat A,Mat *G)
   PetscScalar       *gval;
   PetscReal         rmax;
   PetscInt          cmax = 0;
-  PC_MG             *mg;
-  PC_GAMG           *gamg;
+  PC_MG             *mg = (PC_MG *)pc->data;
+  PC_GAMG           *gamg = (PC_GAMG *)mg->innerctx;
   PetscErrorCode    ierr;
   PetscInt          *gsparse,*lsparse;
   PetscScalar       *Amax;
   MatType           mtype;
 
   PetscFunctionBegin;
-  mg   = (PC_MG *)pc->data;
-  gamg = (PC_GAMG *)mg->innerctx;
-
   ierr = MatGetOwnershipRange(A,&s,&f);CHKERRQ(ierr);
   n=f-s;
   ierr = PetscMalloc3(n,&lsparse,n,&gsparse,n,&Amax);CHKERRQ(ierr);
@@ -139,7 +125,7 @@ PetscErrorCode PCGAMGGraph_Classical(PC pc,Mat A,Mat *G)
     gidx = 0;
     /* create the local and global sparsity patterns */
     for (c = 0; c < ncols; c++) {
-      if (PetscRealPart(-rval[c]) > gamg->threshold*PetscRealPart(Amax[r-s]) || rcol[c] == r) {
+      if (PetscRealPart(-rval[c]) > gamg->threshold[0]*PetscRealPart(Amax[r-s]) || rcol[c] == r) {
         if (rcol[c] < f && rcol[c] >= s) {
           lidx++;
         } else {
@@ -164,7 +150,7 @@ PetscErrorCode PCGAMGGraph_Classical(PC pc,Mat A,Mat *G)
     idx = 0;
     for (c = 0; c < ncols; c++) {
       /* classical strength of connection */
-      if (PetscRealPart(-rval[c]) > gamg->threshold*PetscRealPart(Amax[r-s]) || rcol[c] == r) {
+      if (PetscRealPart(-rval[c]) > gamg->threshold[0]*PetscRealPart(Amax[r-s]) || rcol[c] == r) {
         gcol[idx] = rcol[c];
         gval[idx] = rval[c];
         idx++;
@@ -182,8 +168,6 @@ PetscErrorCode PCGAMGGraph_Classical(PC pc,Mat A,Mat *G)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGCoarsen_Classical"
 PetscErrorCode PCGAMGCoarsen_Classical(PC pc,Mat *G,PetscCoarsenData **agg_lists)
 {
   PetscErrorCode   ierr;
@@ -200,12 +184,9 @@ PetscErrorCode PCGAMGCoarsen_Classical(PC pc,Mat *G,PetscCoarsenData **agg_lists
   ierr = MatCoarsenApply(crs);CHKERRQ(ierr);
   ierr = MatCoarsenGetData(crs,agg_lists);CHKERRQ(ierr);
   ierr = MatCoarsenDestroy(&crs);CHKERRQ(ierr);
-
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGProlongator_Classical_Direct"
 PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoarsenData *agg_lists,Mat *P)
 {
   PetscErrorCode    ierr;
@@ -279,7 +260,7 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
     ierr = PetscSFBcastEnd(sf,MPIU_INT,lcid,gcid);CHKERRQ(ierr);
   }
 
-  /* determine the biggest off-diagonal entries in each row */
+  /* determine the largest off-diagonal entries in each row */
   for (i=fs;i<fe;i++) {
     Amax_pos[i-fs] = 0.;
     Amax_neg[i-fs] = 0.;
@@ -306,7 +287,7 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
       ierr = MatGetRow(lA,i,&ncols,&rcol,&rval);CHKERRQ(ierr);
       for (j = 0;j < ncols;j++) {
         col = rcol[j];
-        if (lcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold*Amax_neg[i])) {
+        if (lcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold[0]*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold[0]*Amax_neg[i])) {
           lsparse[i] += 1;
         }
       }
@@ -316,7 +297,7 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
         ierr = MatGetRow(gA,i,&ncols,&rcol,&rval);CHKERRQ(ierr);
         for (j = 0; j < ncols; j++) {
           col = rcol[j];
-          if (gcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold*Amax_neg[i])) {
+          if (gcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold[0]*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold[0]*Amax_neg[i])) {
             gsparse[i] += 1;
           }
         }
@@ -346,13 +327,13 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
       g_neg = 0.;
       a_pos = 0.;
       a_neg = 0.;
-      diag = 0.;
+      diag  = 0.;
 
       /* local connections */
       ierr = MatGetRow(lA,i,&ncols,&rcol,&rval);CHKERRQ(ierr);
       for (j = 0; j < ncols; j++) {
         col = rcol[j];
-        if (lcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold*Amax_neg[i])) {
+        if (lcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold[0]*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold[0]*Amax_neg[i])) {
           if (PetscRealPart(rval[j]) > 0.) {
             g_pos += rval[j];
           } else {
@@ -376,7 +357,7 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
         ierr = MatGetRow(gA,i,&ncols,&rcol,&rval);CHKERRQ(ierr);
         for (j = 0; j < ncols; j++) {
           col = rcol[j];
-          if (gcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold*Amax_neg[i])) {
+          if (gcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold[0]*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold[0]*Amax_neg[i])) {
             if (PetscRealPart(rval[j]) > 0.) {
               g_pos += rval[j];
             } else {
@@ -412,7 +393,7 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
       idx = 0;
       for (j = 0;j < ncols;j++) {
         col = rcol[j];
-        if (lcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold*Amax_neg[i])) {
+        if (lcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold[0]*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold[0]*Amax_neg[i])) {
           row_f = i + fs;
           row_c = lcid[col];
           /* set the values for on-processor ones */
@@ -434,7 +415,7 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
         ierr = MatGetRow(gA,i,&ncols,&rcol,&rval);CHKERRQ(ierr);
         for (j = 0; j < ncols; j++) {
           col = rcol[j];
-          if (gcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold*Amax_neg[i])) {
+          if (gcid[col] >= 0 && (PetscRealPart(rval[j]) > gamg->threshold[0]*Amax_pos[i] || PetscRealPart(-rval[j]) > gamg->threshold[0]*Amax_neg[i])) {
             row_f = i + fs;
             row_c = gcid[col];
             /* set the values for on-processor ones */
@@ -466,12 +447,9 @@ PetscErrorCode PCGAMGProlongator_Classical_Direct(PC pc, Mat A, Mat G, PetscCoar
     ierr = PetscSFDestroy(&sf);CHKERRQ(ierr);
     ierr = PetscFree(gcid);CHKERRQ(ierr);
   }
-
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGTruncateProlongator_Private"
 PetscErrorCode PCGAMGTruncateProlongator_Private(PC pc,Mat *P)
 {
   PetscInt          j,i,ps,pf,pn,pcs,pcf,pcn,idx,cmax;
@@ -590,8 +568,6 @@ PetscErrorCode PCGAMGTruncateProlongator_Private(PC pc,Mat *P)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGProlongator_Classical_Standard"
 PetscErrorCode PCGAMGProlongator_Classical_Standard(PC pc, Mat A, Mat G, PetscCoarsenData *agg_lists,Mat *P)
 {
   PetscErrorCode    ierr;
@@ -623,7 +599,7 @@ PetscErrorCode PCGAMGProlongator_Classical_Standard(PC pc, Mat A, Mat G, PetscCo
     ierr = MatIncreaseOverlap(A,1,&lis,2);CHKERRQ(ierr);
     ierr = ISSort(lis);CHKERRQ(ierr);
     /* get the local part of A */
-    ierr = MatGetSubMatrices(A,1,&lis,&lis,MAT_INITIAL_MATRIX,&lAs);CHKERRQ(ierr);
+    ierr = MatCreateSubMatrices(A,1,&lis,&lis,MAT_INITIAL_MATRIX,&lAs);CHKERRQ(ierr);
     lA = lAs[0];
     /* build an SF out of it */
     ierr = ISGetLocalSize(lis,&nl);CHKERRQ(ierr);
@@ -846,8 +822,6 @@ PetscErrorCode PCGAMGProlongator_Classical_Standard(PC pc, Mat A, Mat G, PetscCo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGOptProlongator_Classical_Jacobi"
 PetscErrorCode PCGAMGOptProlongator_Classical_Jacobi(PC pc,Mat A,Mat *P)
 {
 
@@ -903,8 +877,6 @@ PetscErrorCode PCGAMGOptProlongator_Classical_Jacobi(PC pc,Mat A,Mat *P)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGProlongator_Classical"
 PetscErrorCode PCGAMGProlongator_Classical(PC pc, Mat A, Mat G, PetscCoarsenData *agg_lists,Mat *P)
 {
   PetscErrorCode    ierr;
@@ -920,8 +892,6 @@ PetscErrorCode PCGAMGProlongator_Classical(PC pc, Mat A, Mat G, PetscCoarsenData
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGDestroy_Classical"
 PetscErrorCode PCGAMGDestroy_Classical(PC pc)
 {
   PetscErrorCode ierr;
@@ -935,8 +905,6 @@ PetscErrorCode PCGAMGDestroy_Classical(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGSetFromOptions_Classical"
 PetscErrorCode PCGAMGSetFromOptions_Classical(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PC_MG             *mg          = (PC_MG*)pc->data;
@@ -958,8 +926,6 @@ PetscErrorCode PCGAMGSetFromOptions_Classical(PetscOptionItems *PetscOptionsObje
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGSetData_Classical"
 PetscErrorCode PCGAMGSetData_Classical(PC pc, Mat A)
 {
   PC_MG          *mg      = (PC_MG*)pc->data;
@@ -967,7 +933,7 @@ PetscErrorCode PCGAMGSetData_Classical(PC pc, Mat A)
 
   PetscFunctionBegin;
   /* no data for classical AMG */
-  pc_gamg->data = NULL;
+  pc_gamg->data           = NULL;
   pc_gamg->data_cell_cols = 0;
   pc_gamg->data_cell_rows = 0;
   pc_gamg->data_sz        = 0;
@@ -975,8 +941,6 @@ PetscErrorCode PCGAMGSetData_Classical(PC pc, Mat A)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGClassicalFinalizePackage"
 PetscErrorCode PCGAMGClassicalFinalizePackage(void)
 {
   PetscErrorCode ierr;
@@ -987,8 +951,6 @@ PetscErrorCode PCGAMGClassicalFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGClassicalInitializePackage"
 PetscErrorCode PCGAMGClassicalInitializePackage(void)
 {
   PetscErrorCode ierr;
@@ -1006,8 +968,6 @@ PetscErrorCode PCGAMGClassicalInitializePackage(void)
    PCCreateGAMG_Classical
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCCreateGAMG_Classical"
 PetscErrorCode  PCCreateGAMG_Classical(PC pc)
 {
   PetscErrorCode ierr;
