@@ -22,11 +22,12 @@ PetscErrorCode MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void
   PetscInt          bs=J->rmap->bs;
 
   PetscFunctionBegin;
+  ierr = VecBindToCPU(x1,PETSC_TRUE);CHKERRQ(ierr);
   /* (1) Set w1 = F(x1) */
   if (!coloring->fset) {
-    ierr = PetscLogEventBegin(MAT_FDColoringFunction,0,0,0,0);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(MAT_FDColoringFunction,coloring,0,0,0);CHKERRQ(ierr);
     ierr = (*f)(sctx,x1,w1,fctx);CHKERRQ(ierr);
-    ierr = PetscLogEventEnd(MAT_FDColoringFunction,0,0,0,0);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(MAT_FDColoringFunction,coloring,0,0,0);CHKERRQ(ierr);
   } else {
     coloring->fset = PETSC_FALSE;
   }
@@ -60,6 +61,8 @@ PetscErrorCode MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void
   /* (3) Loop over each color */
   if (!coloring->w3) {
     ierr = VecDuplicate(x1,&coloring->w3);CHKERRQ(ierr);
+    /* Vec is used instensively in particular piece of scalar CPU code; won't benifit from bouncing back and forth to the GPU */
+    ierr = VecBindToCPU(coloring->w3,PETSC_TRUE);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)coloring,(PetscObject)coloring->w3);CHKERRQ(ierr);
   }
   w3 = coloring->w3;
@@ -68,7 +71,7 @@ PetscErrorCode MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void
   if (vscale) {
     ierr = VecGetArray(vscale,&vscale_array);CHKERRQ(ierr);
   }
-  nz   = 0;
+  nz = 0;
   for (k=0; k<ncolors; k++) {
     coloring->currentcolor = k;
 
@@ -152,6 +155,7 @@ PetscErrorCode MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void
   }
 
   coloring->currentcolor = -1;
+  ierr = VecBindToCPU(x1,PETSC_FALSE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -174,6 +178,7 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
   const PetscInt    ncolors=coloring->ncolors,*ncolumns=coloring->ncolumns,*nrows=coloring->nrows;
 
   PetscFunctionBegin;
+  ierr = VecBindToCPU(x1,PETSC_TRUE);CHKERRQ(ierr);
   if ((ctype == IS_COLORING_LOCAL) && (J->ops->fdcoloringapply == MatFDColoringApply_AIJ)) SETERRQ(PetscObjectComm((PetscObject)J),PETSC_ERR_SUP,"Must call MatColoringUseDM() with IS_COLORING_LOCAL");
   /* (1) Set w1 = F(x1) */
   if (!coloring->fset) {
@@ -221,7 +226,7 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
   if (vscale) {
     ierr = VecGetArray(vscale,&vscale_array);CHKERRQ(ierr);
   }
-  nz   = 0;
+  nz = 0;
 
   if (coloring->bcols > 1) { /* use blocked insertion of Jentry */
     PetscInt    i,m=J->rmap->n,nbcols,bcols=coloring->bcols;
@@ -276,7 +281,6 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
        (3-3) Loop over block rows of vector, putting results into Jacobian matrix
        */
       nrows_k = nrows[nbcols++];
-      ierr = VecGetArray(w2,&y);CHKERRQ(ierr);
 
       if (coloring->htype[0] == 'w') {
         for (l=0; l<nrows_k; l++) {
@@ -290,7 +294,6 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
           nz++;
         }
       }
-      ierr = VecRestoreArray(w2,&y);CHKERRQ(ierr);
     }
   } else { /* bcols == 1 */
     for (k=0; k<ncolors; k++) {
@@ -349,12 +352,16 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
     }
   }
 
+#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_CUDA)
+  if (J->offloadmask != PETSC_OFFLOAD_UNALLOCATED) J->offloadmask = PETSC_OFFLOAD_CPU;
+#endif
   ierr = MatAssemblyBegin(J,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(J,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   if (vscale) {
     ierr = VecRestoreArray(vscale,&vscale_array);CHKERRQ(ierr);
   }
   coloring->currentcolor = -1;
+  ierr = VecBindToCPU(x1,PETSC_FALSE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -364,8 +371,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
   PetscMPIInt            size,*ncolsonproc,*disp,nn;
   PetscInt               i,n,nrows,nrows_i,j,k,m,ncols,col,*rowhit,cstart,cend,colb;
   const PetscInt         *is,*A_ci,*A_cj,*B_ci,*B_cj,*row=NULL,*ltog=NULL;
-  PetscInt               nis=iscoloring->n,nctot,*cols;
-  IS                     *isa;
+  PetscInt               nis=iscoloring->n,nctot,*cols,tmp = 0;
   ISLocalToGlobalMapping map=mat->cmap->mapping;
   PetscInt               ctype=c->ctype,*spidxA,*spidxB,nz,bs,bs2,spidx;
   Mat                    A,B;
@@ -387,7 +393,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
   }
 
   ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPIBAIJ,&isBAIJ);CHKERRQ(ierr);
+  ierr = PetscObjectBaseTypeCompare((PetscObject)mat,MATMPIBAIJ,&isBAIJ);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPISELL,&isSELL);CHKERRQ(ierr);
   if (isBAIJ) {
     Mat_MPIBAIJ *baij=(Mat_MPIBAIJ*)mat->data;
@@ -411,6 +417,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
         }
       }
       ierr = VecCreateGhost(PetscObjectComm((PetscObject)mat),mat->cmap->n,PETSC_DETERMINE,B->cmap->n,garray,&c->vscale);CHKERRQ(ierr);
+      ierr = VecBindToCPU(c->vscale,PETSC_TRUE);CHKERRQ(ierr);
       ierr = PetscFree(garray);CHKERRQ(ierr);
     }
   } else if (isSELL) {
@@ -432,6 +439,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
 
     if (ctype == IS_COLORING_GLOBAL && c->htype[0] == 'd') { /* create vscale for storing dx */
       ierr = VecCreateGhost(PetscObjectComm((PetscObject)mat),mat->cmap->n,PETSC_DETERMINE,B->cmap->n,sell->garray,&c->vscale);CHKERRQ(ierr);
+      ierr = VecBindToCPU(c->vscale,PETSC_TRUE);CHKERRQ(ierr);
     }
   } else {
     Mat_MPIAIJ *aij=(Mat_MPIAIJ*)mat->data;
@@ -452,49 +460,46 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
 
     if (ctype == IS_COLORING_GLOBAL && c->htype[0] == 'd') { /* create vscale for storing dx */
       ierr = VecCreateGhost(PetscObjectComm((PetscObject)mat),mat->cmap->n,PETSC_DETERMINE,B->cmap->n,aij->garray,&c->vscale);CHKERRQ(ierr);
+      ierr = VecBindToCPU(c->vscale,PETSC_TRUE);CHKERRQ(ierr);
     }
   }
 
-  m         = mat->rmap->n/bs;
-  cstart    = mat->cmap->rstart/bs;
-  cend      = mat->cmap->rend/bs;
+  m      = mat->rmap->n/bs;
+  cstart = mat->cmap->rstart/bs;
+  cend   = mat->cmap->rend/bs;
 
-  ierr       = PetscMalloc1(nis,&c->ncolumns);CHKERRQ(ierr);
-  ierr       = PetscMalloc1(nis,&c->columns);CHKERRQ(ierr);
-  ierr       = PetscCalloc1(nis,&c->nrows);CHKERRQ(ierr);
-  ierr       = PetscLogObjectMemory((PetscObject)c,3*nis*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscMalloc2(nis,&c->ncolumns,nis,&c->columns);CHKERRQ(ierr);
+  ierr = PetscMalloc1(nis,&c->nrows);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)c,3*nis*sizeof(PetscInt));CHKERRQ(ierr);
 
   if (c->htype[0] == 'd') {
-    ierr       = PetscMalloc1(nz,&Jentry);CHKERRQ(ierr);
-    ierr       = PetscLogObjectMemory((PetscObject)c,nz*sizeof(MatEntry));CHKERRQ(ierr);
+    ierr        = PetscMalloc1(nz,&Jentry);CHKERRQ(ierr);
+    ierr        = PetscLogObjectMemory((PetscObject)c,nz*sizeof(MatEntry));CHKERRQ(ierr);
     c->matentry = Jentry;
   } else if (c->htype[0] == 'w') {
-    ierr       = PetscMalloc1(nz,&Jentry2);CHKERRQ(ierr);
-    ierr       = PetscLogObjectMemory((PetscObject)c,nz*sizeof(MatEntry2));CHKERRQ(ierr);
+    ierr         = PetscMalloc1(nz,&Jentry2);CHKERRQ(ierr);
+    ierr         = PetscLogObjectMemory((PetscObject)c,nz*sizeof(MatEntry2));CHKERRQ(ierr);
     c->matentry2 = Jentry2;
   } else SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"htype is not supported");
 
   ierr = PetscMalloc2(m+1,&rowhit,m+1,&valaddrhit);CHKERRQ(ierr);
-  nz = 0;
-  ierr = ISColoringGetIS(iscoloring,PETSC_IGNORE,&isa);CHKERRQ(ierr);
+  nz   = 0;
+  ierr = ISColoringGetIS(iscoloring,PETSC_OWN_POINTER, PETSC_IGNORE,&c->isa);CHKERRQ(ierr);
+
+  if (ctype == IS_COLORING_GLOBAL) {
+    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)mat),&size);CHKERRQ(ierr);
+    ierr = PetscMalloc2(size,&ncolsonproc,size,&disp);CHKERRQ(ierr);
+  }
+
   for (i=0; i<nis; i++) { /* for each local color */
-    ierr = ISGetLocalSize(isa[i],&n);CHKERRQ(ierr);
-    ierr = ISGetIndices(isa[i],&is);CHKERRQ(ierr);
+    ierr = ISGetLocalSize(c->isa[i],&n);CHKERRQ(ierr);
+    ierr = ISGetIndices(c->isa[i],&is);CHKERRQ(ierr);
 
     c->ncolumns[i] = n; /* local number of columns of this color on this process */
-    if (n) {
-      ierr = PetscMalloc1(n,&c->columns[i]);CHKERRQ(ierr);
-      ierr = PetscLogObjectMemory((PetscObject)c,n*sizeof(PetscInt));CHKERRQ(ierr);
-      ierr = PetscMemcpy(c->columns[i],is,n*sizeof(PetscInt));CHKERRQ(ierr);
-    } else {
-      c->columns[i] = 0;
-    }
+    c->columns[i]  = (PetscInt*)is;
 
     if (ctype == IS_COLORING_GLOBAL) {
       /* Determine nctot, the total (parallel) number of columns of this color */
-      ierr = MPI_Comm_size(PetscObjectComm((PetscObject)mat),&size);CHKERRQ(ierr);
-      ierr = PetscMalloc2(size,&ncolsonproc,size,&disp);CHKERRQ(ierr);
-
       /* ncolsonproc[j]: local ncolumns on proc[j] of this color */
       ierr  = PetscMPIIntCast(n,&nn);CHKERRQ(ierr);
       ierr  = MPI_Allgather(&nn,1,MPI_INT,ncolsonproc,1,MPI_INT,PetscObjectComm((PetscObject)mat));CHKERRQ(ierr);
@@ -511,16 +516,14 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
       /* Get cols, the complete list of columns for this color on each process */
       ierr = PetscMalloc1(nctot+1,&cols);CHKERRQ(ierr);
       ierr = MPI_Allgatherv((void*)is,n,MPIU_INT,cols,ncolsonproc,disp,MPIU_INT,PetscObjectComm((PetscObject)mat));CHKERRQ(ierr);
-      ierr = PetscFree2(ncolsonproc,disp);CHKERRQ(ierr);
     } else if (ctype == IS_COLORING_LOCAL) {
       /* Determine local number of columns of this color on this process, including ghost points */
       nctot = n;
-      ierr  = PetscMalloc1(nctot+1,&cols);CHKERRQ(ierr);
-      ierr  = PetscMemcpy(cols,is,n*sizeof(PetscInt));CHKERRQ(ierr);
+      cols  = (PetscInt*)is;
     } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not provided for this MatFDColoring type");
 
     /* Mark all rows affect by these columns */
-    ierr    = PetscMemzero(rowhit,m*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr    = PetscArrayzero(rowhit,m);CHKERRQ(ierr);
     bs2     = bs*bs;
     nrows_i = 0;
     for (j=0; j<nctot; j++) { /* loop over columns*/
@@ -530,13 +533,15 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
         col = cols[j];
       }
       if (col >= cstart && col < cend) { /* column is in A, diagonal block of mat */
-        row      = A_cj + A_ci[col-cstart];
-        nrows    = A_ci[col-cstart+1] - A_ci[col-cstart];
+        tmp      = A_ci[col-cstart];
+        row      = A_cj + tmp;
+        nrows    = A_ci[col-cstart+1] - tmp;
         nrows_i += nrows;
+
         /* loop over columns of A marking them in rowhit */
         for (k=0; k<nrows; k++) {
           /* set valaddrhit for part A */
-          spidx            = bs2*spidxA[A_ci[col-cstart] + k];
+          spidx            = bs2*spidxA[tmp + k];
           valaddrhit[*row] = &A_val[spidx];
           rowhit[*row++]   = col - cstart + 1; /* local column index */
         }
@@ -551,14 +556,15 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
           nrows = 0;
         } else {
           colb  = colb/bs;
-          row   = B_cj + B_ci[colb];
-          nrows = B_ci[colb+1] - B_ci[colb];
+          tmp   = B_ci[colb];
+          row   = B_cj + tmp;
+          nrows = B_ci[colb+1] - tmp;
         }
         nrows_i += nrows;
         /* loop over columns of B marking them in rowhit */
         for (k=0; k<nrows; k++) {
           /* set valaddrhit for part B */
-          spidx            = bs2*spidxB[B_ci[colb] + k];
+          spidx            = bs2*spidxB[tmp + k];
           valaddrhit[*row] = &B_val[spidx];
           rowhit[*row++]   = colb + 1 + cend - cstart; /* local column index */
         }
@@ -584,7 +590,12 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
         }
       }
     }
-    ierr = PetscFree(cols);CHKERRQ(ierr);
+    if (ctype == IS_COLORING_GLOBAL) {
+      ierr = PetscFree(cols);CHKERRQ(ierr);
+    }
+  }
+  if (ctype == IS_COLORING_GLOBAL) {
+    ierr = PetscFree2(ncolsonproc,disp);CHKERRQ(ierr);
   }
 
   if (bcols > 1) { /* reorder Jentry for faster MatFDColoringApply() */
@@ -598,12 +609,12 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
   } else if (isSELL) {
     ierr = MatRestoreColumnIJ_SeqSELL_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
     ierr = MatRestoreColumnIJ_SeqSELL_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
-  }else {
+  } else {
     ierr = MatRestoreColumnIJ_SeqAIJ_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
     ierr = MatRestoreColumnIJ_SeqAIJ_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
   }
 
-  ierr = ISColoringRestoreIS(iscoloring,&isa);CHKERRQ(ierr);
+  ierr = ISColoringRestoreIS(iscoloring,PETSC_OWN_POINTER,&c->isa);CHKERRQ(ierr);
   ierr = PetscFree2(rowhit,valaddrhit);CHKERRQ(ierr);
 
   if (ctype == IS_COLORING_LOCAL) {
@@ -623,7 +634,7 @@ PetscErrorCode MatFDColoringCreate_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCo
   /* set default brows and bcols for speedup inserting the dense matrix into sparse Jacobian;
    bcols is chosen s.t. dy-array takes 50% of memory space as mat */
   ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPIBAIJ,&isBAIJ);CHKERRQ(ierr);
+  ierr = PetscObjectBaseTypeCompare((PetscObject)mat,MATMPIBAIJ,&isBAIJ);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPISELL,&isSELL);CHKERRQ(ierr);
   if (isBAIJ || m == 0) {
     c->brows = m;
@@ -677,3 +688,55 @@ PetscErrorCode MatFDColoringCreate_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCo
   c->ncolors = nis;
   PetscFunctionReturn(0);
 }
+
+/*@C
+
+    MatFDColoringSetValues - takes a matrix in compressed color format and enters the matrix into a PETSc Mat
+
+   Collective on J
+
+   Input Parameters:
++    J - the sparse matrix
+.    coloring - created with MatFDColoringCreate() and a local coloring
+-    y - column major storage of matrix values with one color of values per column, the number of rows of y should match
+         the number of local rows of J and the number of columns is the number of colors.
+
+   Level: intermediate
+
+   Notes: the matrix in compressed color format may come from an AD code
+
+   The code will be slightly faster if MatFDColoringSetBlockSize(coloring,PETSC_DEFAULT,nc); is called immediately after creating the coloring
+
+.seealso: MatFDColoringCreate(), ISColoring, ISColoringCreate(), ISColoringSetType(), IS_COLORING_LOCAL, MatFDColoringSetBlockSize()
+
+@*/
+PetscErrorCode  MatFDColoringSetValues(Mat J,MatFDColoring coloring,const PetscScalar *y)
+{
+  PetscErrorCode    ierr;
+  MatEntry2         *Jentry2;
+  PetscInt          row,i,nrows_k,l,ncolors,nz = 0,bcols,nbcols = 0;
+  const PetscInt    *nrows;
+  PetscBool         eq;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(J,MAT_CLASSID,1);
+  PetscValidHeaderSpecific(coloring,MAT_FDCOLORING_CLASSID,2);
+  ierr = PetscObjectCompareId((PetscObject)J,coloring->matid,&eq);CHKERRQ(ierr);
+  if (!eq) SETERRQ(PetscObjectComm((PetscObject)J),PETSC_ERR_ARG_WRONG,"Matrix used with MatFDColoringSetValues() must be that used with MatFDColoringCreate()");
+  Jentry2 = coloring->matentry2;
+  nrows   = coloring->nrows;
+  ncolors = coloring->ncolors;
+  bcols   = coloring->bcols;
+
+  for (i=0; i<ncolors; i+=bcols) {
+    nrows_k = nrows[nbcols++];
+    for (l=0; l<nrows_k; l++) {
+      row                      = Jentry2[nz].row;   /* local row index */
+      *(Jentry2[nz++].valaddr) = y[row];
+    }
+    y += bcols*coloring->m;
+  }
+  ierr = MatAssemblyBegin(J,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = MatAssemblyEnd(J,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
