@@ -9,61 +9,77 @@
 #include <petscbt.h>
 #include <petsctime.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPtAP_SeqAIJ_SeqAIJ"
-PETSC_INTERN PetscErrorCode MatPtAP_SeqAIJ_SeqAIJ(Mat A,Mat P,MatReuse scall,PetscReal fill,Mat *C)
+#if defined(PETSC_HAVE_HYPRE)
+PETSC_INTERN PetscErrorCode MatPtAPSymbolic_AIJ_AIJ_wHYPRE(Mat,Mat,PetscReal,Mat);
+#endif
+
+PetscErrorCode MatProductSymbolic_PtAP_SeqAIJ_SeqAIJ(Mat C)
 {
-  PetscErrorCode ierr;
-  const char     *algTypes[2] = {"scalable","nonscalable"};
-  PetscInt       alg=0; /* set default algorithm */
+  PetscErrorCode      ierr;
+  Mat_Product         *product = C->product;
+  Mat                 A=product->A,P=product->B;
+  MatProductAlgorithm alg=product->alg;
+  PetscReal           fill=product->fill;
+  PetscBool           flg;
+  Mat                 Pt;
+  Mat_MatTransMatMult *atb;
+  Mat_SeqAIJ          *c;
 
   PetscFunctionBegin;
-  if (scall == MAT_INITIAL_MATRIX) {
-    /* 
-     Alg 'scalable' determines which implementations to be used:
-       "nonscalable": do dense axpy in MatPtAPNumeric() - fastest, but requires storage of struct A*P;
-       "scalable":    do two sparse axpy in MatPtAPNumeric() - might slow, does not store structure of A*P. 
-     */
-    ierr = PetscObjectOptionsBegin((PetscObject)A);CHKERRQ(ierr);
-    ierr = PetscOptionsEList("-matptap_via","Algorithmic approach","MatPtAP",algTypes,2,algTypes[0],&alg,NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsEnd();CHKERRQ(ierr);
-    ierr = PetscLogEventBegin(MAT_PtAPSymbolic,A,P,0,0);CHKERRQ(ierr);
-    switch (alg) {
-    case 1:
-      ierr = MatPtAPSymbolic_SeqAIJ_SeqAIJ_DenseAxpy(A,P,fill,C);CHKERRQ(ierr);
-      break;
-    default:
-      ierr = MatPtAPSymbolic_SeqAIJ_SeqAIJ_SparseAxpy(A,P,fill,C);CHKERRQ(ierr);
-      break;
-    }
-    ierr = PetscLogEventEnd(MAT_PtAPSymbolic,A,P,0,0);CHKERRQ(ierr);
+  /* "scalable" */
+  ierr = PetscStrcmp(alg,"scalable",&flg);CHKERRQ(ierr);
+  if (flg) {
+    ierr = MatPtAPSymbolic_SeqAIJ_SeqAIJ_SparseAxpy(A,P,fill,C);CHKERRQ(ierr);
+    C->ops->productnumeric = MatProductNumeric_PtAP;
+    PetscFunctionReturn(0);
   }
-  ierr = PetscLogEventBegin(MAT_PtAPNumeric,A,P,0,0);CHKERRQ(ierr);
-  ierr = (*(*C)->ops->ptapnumeric)(A,P,*C);CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(MAT_PtAPNumeric,A,P,0,0);CHKERRQ(ierr);
+
+  /* "rap" */
+  ierr = PetscStrcmp(alg,"rap",&flg);CHKERRQ(ierr);
+  if (flg) { /* Set default algorithm */
+    ierr = PetscNew(&atb);CHKERRQ(ierr);
+    ierr = MatTranspose_SeqAIJ(P,MAT_INITIAL_MATRIX,&Pt);CHKERRQ(ierr);
+    ierr = MatMatMatMultSymbolic_SeqAIJ_SeqAIJ_SeqAIJ(Pt,A,P,fill,C);CHKERRQ(ierr);
+
+    c                      = (Mat_SeqAIJ*)C->data;
+    c->atb                 = atb;
+    atb->At                = Pt;
+    atb->destroy           = C->ops->destroy;
+    C->ops->destroy        = MatDestroy_SeqAIJ_MatTransMatMult;
+    C->ops->ptapnumeric    = MatPtAPNumeric_SeqAIJ_SeqAIJ;
+    C->ops->productnumeric = MatProductNumeric_PtAP;
+    PetscFunctionReturn(0);
+  }
+
+  /* hypre */
+#if defined(PETSC_HAVE_HYPRE)
+  ierr = PetscStrcmp(alg,"hypre",&flg);CHKERRQ(ierr);
+  if (flg) {
+    ierr = MatPtAPSymbolic_AIJ_AIJ_wHYPRE(A,P,fill,C);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
+#endif
+
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"MatProductType is not supported");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_SeqAIJ_PtAP"
 PetscErrorCode MatDestroy_SeqAIJ_PtAP(Mat A)
 {
   PetscErrorCode ierr;
-  Mat_SeqAIJ     *a    = (Mat_SeqAIJ*)A->data;
-  Mat_PtAP       *ptap = a->ptap;
+  Mat_SeqAIJ     *a  = (Mat_SeqAIJ*)A->data;
+  Mat_AP         *ap = a->ap;
 
   PetscFunctionBegin;
-  ierr = PetscFree(ptap->apa);CHKERRQ(ierr);
-  ierr = PetscFree(ptap->api);CHKERRQ(ierr);
-  ierr = PetscFree(ptap->apj);CHKERRQ(ierr);
-  ierr = (ptap->destroy)(A);CHKERRQ(ierr);
-  ierr = PetscFree(ptap);CHKERRQ(ierr);
+  ierr = PetscFree(ap->apa);CHKERRQ(ierr);
+  ierr = PetscFree(ap->api);CHKERRQ(ierr);
+  ierr = PetscFree(ap->apj);CHKERRQ(ierr);
+  ierr = (ap->destroy)(A);CHKERRQ(ierr);
+  ierr = PetscFree(ap);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPtAPSymbolic_SeqAIJ_SeqAIJ_SparseAxpy"
-PetscErrorCode MatPtAPSymbolic_SeqAIJ_SeqAIJ_SparseAxpy(Mat A,Mat P,PetscReal fill,Mat *C)
+PetscErrorCode MatPtAPSymbolic_SeqAIJ_SeqAIJ_SparseAxpy(Mat A,Mat P,PetscReal fill,Mat C)
 {
   PetscErrorCode     ierr;
   PetscFreeSpaceList free_space=NULL,current_space=NULL;
@@ -156,41 +172,40 @@ PetscErrorCode MatPtAPSymbolic_SeqAIJ_SeqAIJ_SparseAxpy(Mat A,Mat P,PetscReal fi
   ierr = PetscCalloc1(ci[pn]+1,&ca);CHKERRQ(ierr);
 
   /* put together the new matrix */
-  ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),pn,pn,ci,cj,ca,C);CHKERRQ(ierr);
-  ierr = MatSetBlockSizes(*C,PetscAbs(P->cmap->bs),PetscAbs(P->cmap->bs));CHKERRQ(ierr);
+  ierr = MatSetSeqAIJWithArrays_private(PetscObjectComm((PetscObject)A),pn,pn,ci,cj,ca,C);CHKERRQ(ierr);
+  ierr = MatSetBlockSizes(C,PetscAbs(P->cmap->bs),PetscAbs(P->cmap->bs));CHKERRQ(ierr);
+  ierr = MatSetType(C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* Since these are PETSc arrays, change flags to free them as necessary. */
-  c          = (Mat_SeqAIJ*)((*C)->data);
+  c          = (Mat_SeqAIJ*)((C)->data);
   c->free_a  = PETSC_TRUE;
   c->free_ij = PETSC_TRUE;
   c->nonew   = 0;
-  (*C)->ops->ptapnumeric = MatPtAPNumeric_SeqAIJ_SeqAIJ_SparseAxpy;
+  C->ops->ptapnumeric    = MatPtAPNumeric_SeqAIJ_SeqAIJ_SparseAxpy;
 
   /* set MatInfo */
   afill = (PetscReal)ci[pn]/(ai[am]+pi[pm] + 1.e-5);
   if (afill < 1.0) afill = 1.0;
   c->maxnz                     = ci[pn];
   c->nz                        = ci[pn];
-  (*C)->info.mallocs           = nspacedouble;
-  (*C)->info.fill_ratio_given  = fill;
-  (*C)->info.fill_ratio_needed = afill;
+  C->info.mallocs           = nspacedouble;
+  C->info.fill_ratio_given  = fill;
+  C->info.fill_ratio_needed = afill;
 
   /* Clean up. */
   ierr = MatRestoreSymbolicTranspose_SeqAIJ(P,&pti,&ptj);CHKERRQ(ierr);
 #if defined(PETSC_USE_INFO)
   if (ci[pn] != 0) {
-    ierr = PetscInfo3((*C),"Reallocs %D; Fill ratio: given %g needed %g.\n",nspacedouble,(double)fill,(double)afill);CHKERRQ(ierr);
-    ierr = PetscInfo1((*C),"Use MatPtAP(A,P,MatReuse,%g,&C) for best performance.\n",(double)afill);CHKERRQ(ierr);
+    ierr = PetscInfo3(C,"Reallocs %D; Fill ratio: given %g needed %g.\n",nspacedouble,(double)fill,(double)afill);CHKERRQ(ierr);
+    ierr = PetscInfo1(C,"Use MatPtAP(A,P,MatReuse,%g,&C) for best performance.\n",(double)afill);CHKERRQ(ierr);
   } else {
-    ierr = PetscInfo((*C),"Empty matrix product\n");CHKERRQ(ierr);
+    ierr = PetscInfo(C,"Empty matrix product\n");CHKERRQ(ierr);
   }
 #endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPtAPNumeric_SeqAIJ_SeqAIJ_SparseAxpy"
 PetscErrorCode MatPtAPNumeric_SeqAIJ_SeqAIJ_SparseAxpy(Mat A,Mat P,Mat C)
 {
   PetscErrorCode ierr;
@@ -205,12 +220,11 @@ PetscErrorCode MatPtAPNumeric_SeqAIJ_SeqAIJ_SparseAxpy(Mat A,Mat P,Mat C)
 
   PetscFunctionBegin;
   /* Allocate temporary array for storage of one row of A*P (cn: non-scalable) */
-  ierr = PetscMalloc3(cn,&apa,cn,&apjdense,c->rmax,&apj);CHKERRQ(ierr);
-  ierr = PetscMemzero(apa,cn*sizeof(MatScalar));CHKERRQ(ierr);
-  ierr = PetscMemzero(apjdense,cn*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscCalloc2(cn,&apa,cn,&apjdense);CHKERRQ(ierr);
+  ierr = PetscMalloc1(cn,&apj);CHKERRQ(ierr);
 
   /* Clear old values in C */
-  ierr = PetscMemzero(ca,ci[cm]*sizeof(MatScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(ca,ci[cm]);CHKERRQ(ierr);
 
   for (i=0; i<am; i++) {
     /* Form sparse row of A*P */
@@ -267,157 +281,20 @@ PetscErrorCode MatPtAPNumeric_SeqAIJ_SeqAIJ_SparseAxpy(Mat A,Mat P,Mat C)
   ierr = MatAssemblyBegin(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
 
-  ierr = PetscFree3(apa,apjdense,apj);CHKERRQ(ierr);
+  ierr = PetscFree2(apa,apjdense);CHKERRQ(ierr);
+  ierr = PetscFree(apj);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPtAPSymbolic_SeqAIJ_SeqAIJ_DenseAxpy"
-PetscErrorCode MatPtAPSymbolic_SeqAIJ_SeqAIJ_DenseAxpy(Mat A,Mat P,PetscReal fill,Mat *C)
-{
-  PetscErrorCode ierr;
-  Mat_SeqAIJ     *ap,*c;
-  PetscInt       *api,*apj,*ci,pn=P->cmap->N;
-  MatScalar      *ca;
-  Mat_PtAP       *ptap;
-  Mat            Pt,AP;
-
-  PetscFunctionBegin;
-  /* Get symbolic Pt = P^T */
-  ierr = MatTransposeSymbolic_SeqAIJ(P,&Pt);CHKERRQ(ierr);
-
-  /* Get symbolic AP = A*P */
-  ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ(A,P,fill,&AP);CHKERRQ(ierr);
-
-  ap          = (Mat_SeqAIJ*)AP->data;
-  api         = ap->i;
-  apj         = ap->j;
-  ap->free_ij = PETSC_FALSE; /* api and apj are kept in struct ptap, cannot be destroyed with AP */
-
-  /* Get C = Pt*AP */
-  ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ(Pt,AP,fill,C);CHKERRQ(ierr);
-
-  c         = (Mat_SeqAIJ*)(*C)->data;
-  ci        = c->i;
-  ierr      = PetscCalloc1(ci[pn]+1,&ca);CHKERRQ(ierr);
-  c->a      = ca;
-  c->free_a = PETSC_TRUE;
-
-  /* Create a supporting struct for reuse by MatPtAPNumeric() */
-  ierr = PetscNew(&ptap);CHKERRQ(ierr);
-
-  c->ptap            = ptap;
-  ptap->destroy      = (*C)->ops->destroy;
-  (*C)->ops->destroy = MatDestroy_SeqAIJ_PtAP;
-
-  /* Allocate temporary array for storage of one row of A*P */
-  ierr = PetscCalloc1(pn+1,&ptap->apa);CHKERRQ(ierr);
-
-  (*C)->ops->ptapnumeric = MatPtAPNumeric_SeqAIJ_SeqAIJ;
-
-  ptap->api = api;
-  ptap->apj = apj;
-
-  /* Clean up. */
-  ierr = MatDestroy(&Pt);CHKERRQ(ierr);
-  ierr = MatDestroy(&AP);CHKERRQ(ierr);
-#if defined(PETSC_USE_INFO)
-  ierr = PetscInfo1((*C),"given fill %g\n",(double)fill);CHKERRQ(ierr);
-#endif
-  PetscFunctionReturn(0);
-}
-
-/* #define PROFILE_MatPtAPNumeric */
-#undef __FUNCT__
-#define __FUNCT__ "MatPtAPNumeric_SeqAIJ_SeqAIJ"
 PetscErrorCode MatPtAPNumeric_SeqAIJ_SeqAIJ(Mat A,Mat P,Mat C)
 {
-  PetscErrorCode    ierr;
-  Mat_SeqAIJ        *a = (Mat_SeqAIJ*) A->data;
-  Mat_SeqAIJ        *p = (Mat_SeqAIJ*) P->data;
-  Mat_SeqAIJ        *c = (Mat_SeqAIJ*) C->data;
-  const PetscInt    *ai=a->i,*aj=a->j,*pi=p->i,*pj=p->j,*ci=c->i,*cj=c->j;
-  const PetscScalar *aa=a->a,*pa=p->a,*pval;
-  const PetscInt    *apj,*pcol,*cjj;
-  const PetscInt    am=A->rmap->N,cm=C->rmap->N;
-  PetscInt          i,j,k,anz,apnz,pnz,prow,crow,cnz;
-  PetscScalar       *apa,*ca=c->a,*caj,pvalj;
-  Mat_PtAP          *ptap = c->ptap;
-#if defined(PROFILE_MatPtAPNumeric)
-  PetscLogDouble t0,tf,time_Cseq0=0.0,time_Cseq1=0.0;
-  PetscInt       flops0=0,flops1=0;
-#endif
+  PetscErrorCode      ierr;
+  Mat_SeqAIJ          *c = (Mat_SeqAIJ*)C->data;
+  Mat_MatTransMatMult *atb = c->atb;
+  Mat                 Pt = atb->At;
 
   PetscFunctionBegin;
-  /* Get temporary array for storage of one row of A*P */
-  apa = ptap->apa;
-
-  /* Clear old values in C */
-  ierr = PetscMemzero(ca,ci[cm]*sizeof(MatScalar));CHKERRQ(ierr);
-
-  for (i=0; i<am; i++) {
-    /* Form sparse row of AP[i,:] = A[i,:]*P */
-#if defined(PROFILE_MatPtAPNumeric)
-    ierr = PetscTime(&t0);CHKERRQ(ierr);
-#endif
-    anz  = ai[i+1] - ai[i];
-    for (j=0; j<anz; j++) {
-      prow = aj[j];
-      pnz  = pi[prow+1] - pi[prow];
-      pcol = pj + pi[prow];
-      pval = pa + pi[prow];
-      for (k=0; k<pnz; k++) {
-        apa[pcol[k]] += aa[j]*pval[k];
-      }
-      ierr = PetscLogFlops(2.0*pnz);CHKERRQ(ierr);
-#if defined(PROFILE_MatPtAPNumeric)
-      flops0 += 2.0*pnz;
-#endif
-    }
-    aj += anz; aa += anz;
-#if defined(PROFILE_MatPtAPNumeric)
-    ierr = PetscTime(&tf);CHKERRQ(ierr);
-
-    time_Cseq0 += tf - t0;
-#endif
-
-    /* Compute P^T*A*P using outer product P[i,:]^T*AP[i,:]. */
-#if defined(PROFILE_MatPtAPNumeric)
-    ierr = PetscTime(&t0);CHKERRQ(ierr);
-#endif
-    apj  = ptap->apj + ptap->api[i];
-    apnz = ptap->api[i+1] - ptap->api[i];
-    pnz  = pi[i+1] - pi[i];
-    pcol = pj + pi[i];
-    pval = pa + pi[i];
-
-    /* Perform dense axpy */
-    for (j=0; j<pnz; j++) {
-      crow  = pcol[j];
-      cjj   = cj + ci[crow];
-      caj   = ca + ci[crow];
-      pvalj = pval[j];
-      cnz   = ci[crow+1] - ci[crow];
-      for (k=0; k<cnz; k++) caj[k] += pvalj*apa[cjj[k]];
-      ierr = PetscLogFlops(2.0*cnz);CHKERRQ(ierr);
-#if defined(PROFILE_MatPtAPNumeric)
-      flops1 += 2.0*cnz;
-#endif
-    }
-#if defined(PROFILE_MatPtAPNumeric)
-    ierr        = PetscTime(&tf);CHKERRQ(ierr);
-    time_Cseq1 += tf - t0;
-#endif
-
-    /* Zero the current row info for A*P */
-    for (j=0; j<apnz; j++) apa[apj[j]] = 0.0;
-  }
-
-  /* Assemble the final matrix and clean up */
-  ierr = MatAssemblyBegin(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-  ierr = MatAssemblyEnd(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-#if defined(PROFILE_MatPtAPNumeric)
-  printf("PtAPNumeric_SeqAIJ time %g + %g, flops %d %d\n",time_Cseq0,time_Cseq1,flops0,flops1);
-#endif
+  ierr = MatTranspose_SeqAIJ(P,MAT_REUSE_MATRIX,&Pt);CHKERRQ(ierr);
+  ierr = (C->ops->matmatmultnumeric)(Pt,A,P,C);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
