@@ -15,7 +15,6 @@ static PetscBool MatMFFDPackageInitialized = PETSC_FALSE;
 
   Level: developer
 
-.keywords: Petsc, destroy, package
 .seealso: PetscFinalize(), MatCreateMFFD(), MatCreateSNESMF()
 @*/
 PetscErrorCode  MatMFFDFinalizePackage(void)
@@ -31,19 +30,16 @@ PetscErrorCode  MatMFFDFinalizePackage(void)
 
 /*@C
   MatMFFDInitializePackage - This function initializes everything in the MatMFFD package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to MatCreate_MFFD()
-  when using static libraries.
+  from MatInitializePackage().
 
   Level: developer
 
-.keywords: Vec, initialize, package
 .seealso: PetscInitialize()
 @*/
 PetscErrorCode  MatMFFDInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -54,28 +50,52 @@ PetscErrorCode  MatMFFDInitializePackage(void)
   /* Register Constructors */
   ierr = MatMFFDRegisterAll();CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister("MatMult MF",          MATMFFD_CLASSID,&MATMFFD_Mult);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("MatMult MF",MATMFFD_CLASSID,&MATMFFD_Mult);CHKERRQ(ierr);
+ /* Process Info */
+  {
+    PetscClassId  classids[1];
 
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL, "-info_exclude", logList, 256, &opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrstr(logList, "matmffd", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(MATMFFD_CLASSID);CHKERRQ(ierr);
-    }
+    classids[0] = MATMFFD_CLASSID;
+    ierr = PetscInfoProcessClass("matmffd", 1, classids);CHKERRQ(ierr);
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL, "-log_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "matmffd", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(MATMFFD_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("matmffd",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscLogEventExcludeClass(MATMFFD_CLASSID);CHKERRQ(ierr);}
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(MatMFFDFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode  MatMFFDSetType_MFFD(Mat mat,MatMFFDType ftype)
+{
+  PetscErrorCode ierr,(*r)(MatMFFD);
+  MatMFFD        ctx;
+  PetscBool      match;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+  PetscValidCharPointer(ftype,2);
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
+
+  /* already set, so just return */
+  ierr = PetscObjectTypeCompare((PetscObject)ctx,ftype,&match);CHKERRQ(ierr);
+  if (match) PetscFunctionReturn(0);
+
+  /* destroy the old one if it exists */
+  if (ctx->ops->destroy) {
+    ierr = (*ctx->ops->destroy)(ctx);CHKERRQ(ierr);
+  }
+
+  ierr =  PetscFunctionListFind(MatMFFDList,ftype,&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown MatMFFD type %s given",ftype);
+  ierr = (*r)(ctx);CHKERRQ(ierr);
+  ierr = PetscObjectChangeTypeName((PetscObject)ctx,ftype);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
     MatMFFDSetType - Sets the method that is used to compute the
     differencing parameter for finite differene matrix-free formulations.
@@ -99,30 +119,12 @@ PetscErrorCode  MatMFFDInitializePackage(void)
 @*/
 PetscErrorCode  MatMFFDSetType(Mat mat,MatMFFDType ftype)
 {
-  PetscErrorCode ierr,(*r)(MatMFFD);
-  MatMFFD        ctx = (MatMFFD)mat->data;
-  PetscBool      match;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   PetscValidCharPointer(ftype,2);
-
-  ierr = PetscObjectTypeCompare((PetscObject)mat,MATMFFD,&match);CHKERRQ(ierr);
-  if (!match) PetscFunctionReturn(0);
-
-  /* already set, so just return */
-  ierr = PetscObjectTypeCompare((PetscObject)ctx,ftype,&match);CHKERRQ(ierr);
-  if (match) PetscFunctionReturn(0);
-
-  /* destroy the old one if it exists */
-  if (ctx->ops->destroy) {
-    ierr = (*ctx->ops->destroy)(ctx);CHKERRQ(ierr);
-  }
-
-  ierr =  PetscFunctionListFind(MatMFFDList,ftype,&r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown MatMFFD type %s given",ftype);
-  ierr = (*r)(ctx);CHKERRQ(ierr);
-  ierr = PetscObjectChangeTypeName((PetscObject)ctx,ftype);CHKERRQ(ierr);
+  ierr = PetscTryMethod(mat,"MatMFFDSetType_C",(Mat,MatMFFDType),(mat,ftype));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -131,38 +133,46 @@ static PetscErrorCode MatGetDiagonal_MFFD(Mat,Vec);
 typedef PetscErrorCode (*FCN1)(void*,Vec); /* force argument to next function to not be extern C*/
 static PetscErrorCode  MatMFFDSetFunctioniBase_MFFD(Mat mat,FCN1 func)
 {
-  MatMFFD ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   ctx->funcisetbase = func;
-  /* allow users to compose their own getdiagonal and allow MatHasOperation
-     to return false if the two functions pointers are not set */
-  if (!mat->ops->getdiagonal && func) {
-    mat->ops->getdiagonal = MatGetDiagonal_MFFD;
-  }
   PetscFunctionReturn(0);
 }
 
 typedef PetscErrorCode (*FCN2)(void*,PetscInt,Vec,PetscScalar*); /* force argument to next function to not be extern C*/
 static PetscErrorCode  MatMFFDSetFunctioni_MFFD(Mat mat,FCN2 funci)
 {
-  MatMFFD ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   ctx->funci = funci;
-  /* allow users to compose their own getdiagonal and allow MatHasOperation
-     to return false if the two functions pointers are not set */
-  if (!mat->ops->getdiagonal && funci) {
-    mat->ops->getdiagonal = MatGetDiagonal_MFFD;
-  }
+  ierr = MatShellSetOperation(mat,MATOP_GET_DIAGONAL,(void (*)(void))MatGetDiagonal_MFFD);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatMFFDGetH_MFFD(Mat mat,PetscScalar *h)
+{
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
+  *h = ctx->currenth;
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode  MatMFFDResetHHistory_MFFD(Mat J)
 {
-  MatMFFD ctx = (MatMFFD)J->data;
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(J,&ctx);CHKERRQ(ierr);
   ctx->ncurrenth = 0;
   PetscFunctionReturn(0);
 }
@@ -191,8 +201,6 @@ $     MatMFFDSetType(mfctx,"my_h")
    or at runtime via the option
 $     -mat_mffd_type my_h
 
-.keywords: MatMFFD, register
-
 .seealso: MatMFFDRegisterAll(), MatMFFDRegisterDestroy()
  @*/
 PetscErrorCode  MatMFFDRegister(const char sname[],PetscErrorCode (*function)(MatMFFD))
@@ -200,6 +208,7 @@ PetscErrorCode  MatMFFDRegister(const char sname[],PetscErrorCode (*function)(Ma
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&MatMFFDList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -208,21 +217,17 @@ PetscErrorCode  MatMFFDRegister(const char sname[],PetscErrorCode (*function)(Ma
 static PetscErrorCode MatDestroy_MFFD(Mat mat)
 {
   PetscErrorCode ierr;
-  MatMFFD        ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->w);CHKERRQ(ierr);
-  ierr = VecDestroy(&ctx->drscale);CHKERRQ(ierr);
-  ierr = VecDestroy(&ctx->dlscale);CHKERRQ(ierr);
-  ierr = VecDestroy(&ctx->dshift);CHKERRQ(ierr);
-  ierr = VecDestroy(&ctx->dshiftw);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->current_u);CHKERRQ(ierr);
   if (ctx->current_f_allocated) {
     ierr = VecDestroy(&ctx->current_f);CHKERRQ(ierr);
   }
   if (ctx->ops->destroy) {ierr = (*ctx->ops->destroy)(ctx);CHKERRQ(ierr);}
-  ierr      = PetscHeaderDestroy(&ctx);CHKERRQ(ierr);
-  mat->data = 0;
+  ierr = PetscHeaderDestroy(&ctx);CHKERRQ(ierr);
 
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetBase_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetFunctioniBase_C",NULL);CHKERRQ(ierr);
@@ -232,6 +237,9 @@ static PetscErrorCode MatDestroy_MFFD(Mat mat)
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetCheckh_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetPeriod_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDResetHHistory_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetHHistory_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDGetH_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -242,11 +250,12 @@ static PetscErrorCode MatDestroy_MFFD(Mat mat)
 static PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  MatMFFD        ctx = (MatMFFD)J->data;
+  MatMFFD        ctx;
   PetscBool      iascii, viewbase, viewfunction;
   const char     *prefix;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(J,&ctx);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"Matrix-free approximation:\n");CHKERRQ(ierr);
@@ -257,6 +266,11 @@ static PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"Using %s compute h routine\n",((PetscObject)ctx)->type_name);CHKERRQ(ierr);
     }
+#if defined(PETSC_USE_COMPLEX)
+    if (ctx->usecomplex) {
+      ierr = PetscViewerASCIIPrintf(viewer,"Using Lyness complex number trick to compute the matrix-vector product\n");CHKERRQ(ierr);
+    }
+#endif
     if (ctx->ops->view) {
       ierr = (*ctx->ops->view)(ctx,viewer);CHKERRQ(ierr);
     }
@@ -290,12 +304,11 @@ static PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
 PETSC_EXTERN PetscErrorCode MatAssemblyEnd_MFFD(Mat J,MatAssemblyType mt)
 {
   PetscErrorCode ierr;
-  MatMFFD        j = (MatMFFD)J->data;
+  MatMFFD        j;
 
   PetscFunctionBegin;
-  ierr      = MatMFFDResetHHistory(J);CHKERRQ(ierr);
-  j->vshift = 0.0;
-  j->vscale = 1.0;
+  ierr = MatShellGetContext(J,&j);CHKERRQ(ierr);
+  ierr = MatMFFDResetHHistory(J);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -309,13 +322,14 @@ PETSC_EXTERN PetscErrorCode MatAssemblyEnd_MFFD(Mat J,MatAssemblyType mt)
 */
 static PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
 {
-  MatMFFD        ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
   PetscScalar    h;
   Vec            w,U,F;
   PetscErrorCode ierr;
   PetscBool      zeroa;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   if (!ctx->current_u) SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_ARG_WRONGSTATE,"MatMFFDSetBase() has not been called, this is often caused by forgetting to call \n\t\tMatAssemblyBegin/End on the first Mat in the SNES compute function");
   /* We log matrix-free matrix-vector products separately, so that we can
      separate the performance monitoring from the cases that use conventional
@@ -356,13 +370,12 @@ static PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
   }
   ctx->ncurrenth++;
 
+#if defined(PETSC_USE_COMPLEX)
+  if (ctx->usecomplex) h = PETSC_i*h;
+#endif
+
   /* w = u + ha */
-  if (ctx->drscale) {
-    ierr = VecPointwiseMult(ctx->drscale,a,U);CHKERRQ(ierr);
-    ierr = VecAYPX(U,h,w);CHKERRQ(ierr);
-  } else {
-    ierr = VecWAXPY(w,h,a,U);CHKERRQ(ierr);
-  }
+  ierr = VecWAXPY(w,h,a,U);CHKERRQ(ierr);
 
   /* compute func(U) as base for differencing; only needed first time in and not when provided by user */
   if (ctx->ncurrenth == 1 && ctx->current_f_allocated) {
@@ -370,23 +383,17 @@ static PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
   }
   ierr = (*ctx->func)(ctx->funcctx,w,y);CHKERRQ(ierr);
 
+#if defined(PETSC_USE_COMPLEX)  
+  if (ctx->usecomplex) {
+    ierr = VecImaginaryPart(y);CHKERRQ(ierr);
+    h    = PetscImaginaryPart(h);
+  } else {
+    ierr = VecAXPY(y,-1.0,F);CHKERRQ(ierr);
+  }
+#else
   ierr = VecAXPY(y,-1.0,F);CHKERRQ(ierr);
+#endif
   ierr = VecScale(y,1.0/h);CHKERRQ(ierr);
-
-  if ((ctx->vshift != 0.0) || (ctx->vscale != 1.0)) {
-    ierr = VecAXPBY(y,ctx->vshift,ctx->vscale,a);CHKERRQ(ierr);
-  }
-  if (ctx->dlscale) {
-    ierr = VecPointwiseMult(y,ctx->dlscale,y);CHKERRQ(ierr);
-  }
-  if (ctx->dshift) {
-    if (!ctx->dshiftw) {
-      ierr = VecDuplicate(y,&ctx->dshiftw);CHKERRQ(ierr);
-    }
-    ierr = VecPointwiseMult(ctx->dshift,a,ctx->dshiftw);CHKERRQ(ierr);
-    ierr = VecAXPY(y,1.0,ctx->dshiftw);CHKERRQ(ierr);
-  }
-
   if (mat->nullsp) {ierr = MatNullSpaceRemove(mat->nullsp,y);CHKERRQ(ierr);}
 
   ierr = PetscLogEventEnd(MATMFFD_Mult,a,y,0,0);CHKERRQ(ierr);
@@ -403,7 +410,7 @@ static PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
 */
 PetscErrorCode MatGetDiagonal_MFFD(Mat mat,Vec a)
 {
-  MatMFFD        ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
   PetscScalar    h,*aa,*ww,v;
   PetscReal      epsilon = PETSC_SQRT_MACHINE_EPSILON,umin = 100.0*PETSC_SQRT_MACHINE_EPSILON;
   Vec            w,U;
@@ -411,13 +418,15 @@ PetscErrorCode MatGetDiagonal_MFFD(Mat mat,Vec a)
   PetscInt       i,rstart,rend;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   if (!ctx->func) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Requires calling MatMFFDSetFunction() first");
   if (!ctx->funci) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Requires calling MatMFFDSetFunctioni() first");
-  if (!ctx->funcisetbase) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Requires calling MatMFFDSetFunctioniBase() first");
   w    = ctx->w;
   U    = ctx->current_u;
   ierr = (*ctx->func)(ctx->funcctx,U,a);CHKERRQ(ierr);
-  ierr = (*ctx->funcisetbase)(ctx->funcctx,U);CHKERRQ(ierr);
+  if (ctx->funcisetbase) {
+    ierr = (*ctx->funcisetbase)(ctx->funcctx,U);CHKERRQ(ierr);
+  }
   ierr = VecCopy(U,w);CHKERRQ(ierr);
 
   ierr = VecGetOwnershipRange(a,&rstart,&rend);CHKERRQ(ierr);
@@ -435,11 +444,6 @@ PetscErrorCode MatGetDiagonal_MFFD(Mat mat,Vec a)
     ierr          = (*ctx->funci)(ctx->funcctx,i,w,&v);CHKERRQ(ierr);
     aa[i-rstart]  = (v - aa[i-rstart])/h;
 
-    /* possibly shift and scale result */
-    if ((ctx->vshift != 0.0) || (ctx->vscale != 1.0)) {
-      aa[i - rstart] = ctx->vshift + ctx->vscale*aa[i-rstart];
-    }
-
     ierr          = VecGetArray(w,&ww);CHKERRQ(ierr);
     ww[i-rstart] -= h;
     ierr          = VecRestoreArray(w,&ww);CHKERRQ(ierr);
@@ -448,73 +452,21 @@ PetscErrorCode MatGetDiagonal_MFFD(Mat mat,Vec a)
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode MatDiagonalScale_MFFD(Mat mat,Vec ll,Vec rr)
-{
-  MatMFFD        aij = (MatMFFD)mat->data;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  if (ll && !aij->dlscale) {
-    ierr = VecDuplicate(ll,&aij->dlscale);CHKERRQ(ierr);
-  }
-  if (rr && !aij->drscale) {
-    ierr = VecDuplicate(rr,&aij->drscale);CHKERRQ(ierr);
-  }
-  if (ll) {
-    ierr = VecCopy(ll,aij->dlscale);CHKERRQ(ierr);
-  }
-  if (rr) {
-    ierr = VecCopy(rr,aij->drscale);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-static PetscErrorCode MatDiagonalSet_MFFD(Mat mat,Vec ll,InsertMode mode)
-{
-  MatMFFD        aij = (MatMFFD)mat->data;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  if (mode == INSERT_VALUES) SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"No diagonal set with INSERT_VALUES");
-  if (!aij->dshift) {
-    ierr = VecDuplicate(ll,&aij->dshift);CHKERRQ(ierr);
-  }
-  ierr = VecAXPY(aij->dshift,1.0,ll);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-static PetscErrorCode MatShift_MFFD(Mat Y,PetscScalar a)
-{
-  MatMFFD shell = (MatMFFD)Y->data;
-
-  PetscFunctionBegin;
-  shell->vshift += a;
-  PetscFunctionReturn(0);
-}
-
-static PetscErrorCode MatScale_MFFD(Mat Y,PetscScalar a)
-{
-  MatMFFD shell = (MatMFFD)Y->data;
-
-  PetscFunctionBegin;
-  shell->vscale *= a;
-  PetscFunctionReturn(0);
-}
-
 PETSC_EXTERN PetscErrorCode MatMFFDSetBase_MFFD(Mat J,Vec U,Vec F)
 {
   PetscErrorCode ierr;
-  MatMFFD        ctx = (MatMFFD)J->data;
+  MatMFFD        ctx;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(J,&ctx);CHKERRQ(ierr);
   ierr = MatMFFDResetHHistory(J);CHKERRQ(ierr);
   if (!ctx->current_u) {
     ierr = VecDuplicate(U,&ctx->current_u);CHKERRQ(ierr);
-    ierr = VecLockPush(ctx->current_u);CHKERRQ(ierr);
+    ierr = VecLockReadPush(ctx->current_u);CHKERRQ(ierr);
   }
-  ierr = VecLockPop(ctx->current_u);CHKERRQ(ierr);
+  ierr = VecLockReadPop(ctx->current_u);CHKERRQ(ierr);
   ierr = VecCopy(U,ctx->current_u);CHKERRQ(ierr);
-  ierr = VecLockPush(ctx->current_u);CHKERRQ(ierr);
+  ierr = VecLockReadPush(ctx->current_u);CHKERRQ(ierr);
   if (F) {
     if (ctx->current_f_allocated) {ierr = VecDestroy(&ctx->current_f);CHKERRQ(ierr);}
     ctx->current_f           = F;
@@ -535,9 +487,11 @@ typedef PetscErrorCode (*FCN3)(void*,Vec,Vec,PetscScalar*); /* force argument to
 
 static PetscErrorCode  MatMFFDSetCheckh_MFFD(Mat J,FCN3 fun,void *ectx)
 {
-  MatMFFD ctx = (MatMFFD)J->data;
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(J,&ctx);CHKERRQ(ierr);
   ctx->checkh    = fun;
   ctx->checkhctx = ectx;
   PetscFunctionReturn(0);
@@ -559,18 +513,17 @@ static PetscErrorCode  MatMFFDSetCheckh_MFFD(Mat J,FCN3 fun,void *ectx)
 
    Level: advanced
 
-.keywords: SNES, matrix-free, parameters
-
 .seealso: MatSetFromOptions(), MatCreateSNESMF(), MatCreateMFFD()
 @*/
 PetscErrorCode  MatMFFDSetOptionsPrefix(Mat mat,const char prefix[])
 
 {
-  MatMFFD        mfctx = mat ? (MatMFFD)mat->data : (MatMFFD)NULL;
+  MatMFFD        mfctx;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+  ierr = MatShellGetContext(mat,&mfctx);CHKERRQ(ierr);
   PetscValidHeaderSpecific(mfctx,MATMFFD_CLASSID,1);
   ierr = PetscObjectSetOptionsPrefix((PetscObject)mfctx,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -578,13 +531,14 @@ PetscErrorCode  MatMFFDSetOptionsPrefix(Mat mat,const char prefix[])
 
 static PetscErrorCode  MatSetFromOptions_MFFD(PetscOptionItems *PetscOptionsObject,Mat mat)
 {
-  MatMFFD        mfctx = (MatMFFD)mat->data;
+  MatMFFD        mfctx;
   PetscErrorCode ierr;
   PetscBool      flg;
   char           ftype[256];
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+  ierr = MatShellGetContext(mat,&mfctx);CHKERRQ(ierr);
   PetscValidHeaderSpecific(mfctx,MATMFFD_CLASSID,1);
   ierr = PetscObjectOptionsBegin((PetscObject)mfctx);CHKERRQ(ierr);
   ierr = PetscOptionsFList("-mat_mffd_type","Matrix free type","MatMFFDSetType",MatMFFDList,((PetscObject)mfctx)->type_name,ftype,256,&flg);CHKERRQ(ierr);
@@ -600,6 +554,9 @@ static PetscErrorCode  MatSetFromOptions_MFFD(PetscOptionItems *PetscOptionsObje
   if (flg) {
     ierr = MatMFFDSetCheckh(mat,MatMFFDCheckPositivity,0);CHKERRQ(ierr);
   }
+#if defined(PETSC_USE_COMPLEX)
+  ierr = PetscOptionsBool("-mat_mffd_complex","Use Lyness complex number trick to compute the matrix-vector product","None",mfctx->usecomplex,&mfctx->usecomplex,NULL);CHKERRQ(ierr);
+#endif
   if (mfctx->ops->setfromoptions) {
     ierr = (*mfctx->ops->setfromoptions)(PetscOptionsObject,mfctx);CHKERRQ(ierr);
   }
@@ -609,18 +566,22 @@ static PetscErrorCode  MatSetFromOptions_MFFD(PetscOptionItems *PetscOptionsObje
 
 static PetscErrorCode  MatMFFDSetPeriod_MFFD(Mat mat,PetscInt period)
 {
-  MatMFFD ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   ctx->recomputeperiod = period;
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode  MatMFFDSetFunction_MFFD(Mat mat,PetscErrorCode (*func)(void*,Vec,Vec),void *funcctx)
 {
-  MatMFFD ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   ctx->func    = func;
   ctx->funcctx = funcctx;
   PetscFunctionReturn(0);
@@ -628,17 +589,25 @@ static PetscErrorCode  MatMFFDSetFunction_MFFD(Mat mat,PetscErrorCode (*func)(vo
 
 static PetscErrorCode  MatMFFDSetFunctionError_MFFD(Mat mat,PetscReal error)
 {
-  MatMFFD ctx = (MatMFFD)mat->data;
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   if (error != PETSC_DEFAULT) ctx->error_rel = error;
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode MatMissingDiagonal_MFFD(Mat A,PetscBool  *missing,PetscInt *d)
+PetscErrorCode  MatMFFDSetHHistory_MFFD(Mat J,PetscScalar history[],PetscInt nhistory)
 {
+  MatMFFD        ctx;
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  *missing = PETSC_FALSE;
+  ierr = MatShellGetContext(J,&ctx);CHKERRQ(ierr);
+  ctx->historyh    = history;
+  ctx->maxcurrenth = nhistory;
+  ctx->currenth    = 0.;
   PetscFunctionReturn(0);
 }
 
@@ -647,6 +616,8 @@ static PetscErrorCode MatMissingDiagonal_MFFD(Mat A,PetscBool  *missing,PetscInt
 
   Level: advanced
 
+  Developers Note: This is implemented on top of MATSHELL to get support for scaling and shifting without requiring duplicate code
+
 .seealso: MatCreateMFFD(), MatCreateSNESMF(), MatMFFDSetFunction(), MatMFFDSetType(),  
           MatMFFDSetFunctionError(), MatMFFDDSSetUmin(), MatMFFDSetFunction()
           MatMFFDSetHHistory(), MatMFFDResetHHistory(), MatCreateSNESMF(),
@@ -660,7 +631,7 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
   PetscFunctionBegin;
   ierr = MatMFFDInitializePackage();CHKERRQ(ierr);
 
-  ierr = PetscHeaderCreate(mfctx,MATMFFD_CLASSID,"MatMFFD","Matrix-free Finite Differencing","Mat",PetscObjectComm((PetscObject)A),MatDestroy_MFFD,MatView_MFFD);CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(mfctx,MATMFFD_CLASSID,"MatMFFD","Matrix-free Finite Differencing","Mat",PetscObjectComm((PetscObject)A),NULL,NULL);CHKERRQ(ierr);
 
   mfctx->error_rel                = PETSC_SQRT_MACHINE_EPSILON;
   mfctx->recomputeperiod          = 1;
@@ -671,9 +642,6 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
   mfctx->maxcurrenth              = 0;
   ((PetscObject)mfctx)->type_name = 0;
 
-  mfctx->vshift = 0.0;
-  mfctx->vscale = 1.0;
-
   /*
      Create the empty data structure to contain compute-h routines.
      These will be filled in below from the command line options or
@@ -689,20 +657,15 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
   mfctx->func    = 0;
   mfctx->funcctx = 0;
   mfctx->w       = NULL;
+  mfctx->mat     = A;
 
-  A->data = mfctx;
-
-  A->ops->mult            = MatMult_MFFD;
-  A->ops->destroy         = MatDestroy_MFFD;
-  A->ops->view            = MatView_MFFD;
-  A->ops->assemblyend     = MatAssemblyEnd_MFFD;
-  A->ops->scale           = MatScale_MFFD;
-  A->ops->shift           = MatShift_MFFD;
-  A->ops->diagonalscale   = MatDiagonalScale_MFFD;
-  A->ops->diagonalset     = MatDiagonalSet_MFFD;
-  A->ops->setfromoptions  = MatSetFromOptions_MFFD;
-  A->ops->missingdiagonal = MatMissingDiagonal_MFFD;
-  A->assembled            = PETSC_TRUE;
+  ierr = MatSetType(A,MATSHELL);CHKERRQ(ierr);
+  ierr = MatShellSetContext(A,mfctx);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(A,MATOP_MULT,(void (*)(void))MatMult_MFFD);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(A,MATOP_DESTROY,(void (*)(void))MatDestroy_MFFD);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(A,MATOP_VIEW,(void (*)(void))MatView_MFFD);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(A,MATOP_ASSEMBLY_END,(void (*)(void))MatAssemblyEnd_MFFD);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(A,MATOP_SET_FROM_OPTIONS,(void (*)(void))MatSetFromOptions_MFFD);CHKERRQ(ierr);
 
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetBase_C",MatMFFDSetBase_MFFD);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetFunctioniBase_C",MatMFFDSetFunctioniBase_MFFD);CHKERRQ(ierr);
@@ -712,9 +675,9 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetPeriod_C",MatMFFDSetPeriod_MFFD);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetFunctionError_C",MatMFFDSetFunctionError_MFFD);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDResetHHistory_C",MatMFFDResetHHistory_MFFD);CHKERRQ(ierr);
-
-  mfctx->mat = A;
-
+  ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetHHistory_C",MatMFFDSetHHistory_MFFD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetType_C",MatMFFDSetType_MFFD);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDGetH_C",MatMFFDGetH_MFFD);CHKERRQ(ierr);
   ierr = PetscObjectChangeTypeName((PetscObject)A,MATMFFD);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -741,8 +704,11 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
 
    Options Database Keys: call MatSetFromOptions() to trigger these
 +  -mat_mffd_type - wp or ds (see MATMFFD_WP or MATMFFD_DS)
--  -mat_mffd_err - square root of estimated relative error in function evaluation
--  -mat_mffd_period - how often h is recomputed, defaults to 1, everytime
+.  -mat_mffd_err - square root of estimated relative error in function evaluation
+.  -mat_mffd_period - how often h is recomputed, defaults to 1, everytime
+.  -mat_mffd_check_positivity - possibly decrease h until U + h*a has only positive values
+-  -mat_mffd_complex - use the Lyness trick with complex numbers to compute the matrix-vector product instead of differencing
+                       (requires real valued functions but that PETSc be configured for complex numbers)
 
 
    Level: advanced
@@ -777,8 +743,6 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
 .  -mat_mffd_unim <umin> - Sets umin (for default PETSc routine that computes h only)
 -  -mat_mffd_check_positivity
 
-.keywords: default, matrix-free, create, matrix
-
 .seealso: MatDestroy(), MatMFFDSetFunctionError(), MatMFFDDSSetUmin(), MatMFFDSetFunction()
           MatMFFDSetHHistory(), MatMFFDResetHHistory(), MatCreateSNESMF(),
           MatMFFDGetH(), MatMFFDRegister(), MatMFFDComputeJacobian()
@@ -805,28 +769,21 @@ PetscErrorCode  MatCreateMFFD(MPI_Comm comm,PetscInt m,PetscInt n,PetscInt M,Pet
    Input Parameters:
 .  mat - the matrix obtained with MatCreateSNESMF()
 
-   Output Paramter:
+   Output Parameter:
 .  h - the differencing step size
 
    Level: advanced
 
-.keywords: SNES, matrix-free, parameters
-
 .seealso: MatCreateSNESMF(),MatMFFDSetHHistory(), MatCreateMFFD(), MATMFFD, MatMFFDResetHHistory()
 @*/
 PetscErrorCode  MatMFFDGetH(Mat mat,PetscScalar *h)
 {
-  MatMFFD        ctx = (MatMFFD)mat->data;
   PetscErrorCode ierr;
-  PetscBool      match;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   PetscValidPointer(h,2);
-  ierr = PetscObjectTypeCompare((PetscObject)mat,MATMFFD,&match);CHKERRQ(ierr);
-  if (!match) SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_ARG_WRONG,"Not a MFFD matrix");
-
-  *h = ctx->currenth;
+  ierr = PetscUseMethod(mat,"MatMFFDGetH_C",(Mat,PetscScalar*),(mat,h));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -855,8 +812,6 @@ $     func (void *funcctx, Vec x, Vec f)
 
     If this is not set then it will use the function set with SNESSetFunction() if MatCreateSNESMF() was used.
 
-.keywords: SNES, matrix-free, function
-
 .seealso: MatCreateSNESMF(),MatMFFDGetH(), MatCreateMFFD(), MATMFFD,
           MatMFFDSetHHistory(), MatMFFDResetHHistory(), SNESetFunction()
 @*/
@@ -887,8 +842,6 @@ PetscErrorCode  MatMFFDSetFunction(Mat mat,PetscErrorCode (*func)(void*,Vec,Vec)
     This function is necessary to compute the diagonal of the matrix.
     funci must not contain any MPI call as it is called inside a loop on the local portion of the vector.
 
-.keywords: SNES, matrix-free, function
-
 .seealso: MatCreateSNESMF(),MatMFFDGetH(), MatMFFDSetHHistory(), MatMFFDResetHHistory(), SNESetFunction(), MatGetDiagonal()
 
 @*/
@@ -919,8 +872,6 @@ PetscErrorCode  MatMFFDSetFunctioni(Mat mat,PetscErrorCode (*funci)(void*,PetscI
     This function is necessary to compute the diagonal of the matrix.
 
 
-.keywords: SNES, matrix-free, function
-
 .seealso: MatCreateSNESMF(),MatMFFDGetH(), MatCreateMFFD(), MATMFFD
           MatMFFDSetHHistory(), MatMFFDResetHHistory(), SNESetFunction(), MatGetDiagonal()
 @*/
@@ -944,13 +895,11 @@ PetscErrorCode  MatMFFDSetFunctioniBase(Mat mat,PetscErrorCode (*func)(void*,Vec
 -  period - 1 for everytime, 2 for every second etc
 
    Options Database Keys:
-+  -mat_mffd_period <period>
+.  -mat_mffd_period <period>
 
    Level: advanced
 
 
-.keywords: SNES, matrix-free, parameters
-
 .seealso: MatCreateSNESMF(),MatMFFDGetH(),
           MatMFFDSetHHistory(), MatMFFDResetHHistory()
 @*/
@@ -977,7 +926,7 @@ PetscErrorCode  MatMFFDSetPeriod(Mat mat,PetscInt period)
                the relative error in the function evaluations)
 
    Options Database Keys:
-+  -mat_mffd_err <error_rel> - Sets error_rel
+.  -mat_mffd_err <error_rel> - Sets error_rel
 
    Level: advanced
 
@@ -989,8 +938,6 @@ PetscErrorCode  MatMFFDSetPeriod(Mat mat,PetscInt period)
        = error_rel*umin*sign(u'a)*||a||_{1}/||a||^2   else
 .ve
 
-.keywords: SNES, matrix-free, parameters
-
 .seealso: MatCreateSNESMF(),MatMFFDGetH(), MatCreateMFFD(), MATMFFD
           MatMFFDSetHHistory(), MatMFFDResetHHistory()
 @*/
@@ -1023,27 +970,19 @@ PetscErrorCode  MatMFFDSetFunctionError(Mat mat,PetscReal error)
    Use MatMFFDResetHHistory() to reset the history counter and collect
    a new batch of differencing parameters, h.
 
-.keywords: SNES, matrix-free, h history, differencing history
-
 .seealso: MatMFFDGetH(), MatCreateSNESMF(),
           MatMFFDResetHHistory(), MatMFFDSetFunctionError()
 
 @*/
 PetscErrorCode  MatMFFDSetHHistory(Mat J,PetscScalar history[],PetscInt nhistory)
 {
-  MatMFFD        ctx = (MatMFFD)J->data;
   PetscErrorCode ierr;
-  PetscBool      match;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(J,MAT_CLASSID,1);
   if (history) PetscValidPointer(history,2);
   PetscValidLogicalCollectiveInt(J,nhistory,3);
-  ierr = PetscObjectTypeCompare((PetscObject)J,MATMFFD,&match);CHKERRQ(ierr);
-  if (!match) SETERRQ(PetscObjectComm((PetscObject)J),PETSC_ERR_ARG_WRONG,"Not a MFFD matrix");
-  ctx->historyh    = history;
-  ctx->maxcurrenth = nhistory;
-  ctx->currenth    = 0.;
+  ierr = PetscUseMethod(J,"MatMFFDSetHHistory_C",(Mat,PetscScalar[],PetscInt),(J,history,nhistory));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1061,8 +1000,6 @@ PetscErrorCode  MatMFFDSetHHistory(Mat J,PetscScalar history[],PetscInt nhistory
    Notes:
    Use MatMFFDSetHHistory() to create the original history counter.
 
-.keywords: SNES, matrix-free, h history, differencing history
-
 .seealso: MatMFFDGetH(), MatCreateSNESMF(),
           MatMFFDSetHHistory(), MatMFFDSetFunctionError()
 
@@ -1088,7 +1025,8 @@ PetscErrorCode  MatMFFDResetHHistory(Mat J)
 .   U - the vector
 -   F - (optional) vector that contains F(u) if it has been already computed
 
-    Notes: This is rarely used directly
+    Notes:
+    This is rarely used directly
 
     If F is provided then it is not recomputed. Otherwise the function is evaluated at the base
     point during the first MatMult() after each call to MatMFFDSetBase().
@@ -1124,7 +1062,8 @@ PetscErrorCode  MatMFFDSetBase(Mat J,Vec U,Vec F)
 
     Level: advanced
 
-    Notes: For example, MatMFFDCheckPositivity() insures that all entries
+    Notes:
+    For example, MatMFFDCheckPositivity() insures that all entries
        of U + h*a are non-negative
 
      The function you provide is called after the default h has been computed and allows you to
@@ -1159,7 +1098,8 @@ PetscErrorCode  MatMFFDSetCheckh(Mat J,PetscErrorCode (*fun)(void*,Vec,Vec,Petsc
 
     Level: advanced
 
-    Notes: This is rarely used directly, rather it is passed as an argument to
+    Notes:
+    This is rarely used directly, rather it is passed as an argument to
            MatMFFDSetCheckh()
 
 .seealso:  MatMFFDSetCheckh()
@@ -1180,7 +1120,7 @@ PetscErrorCode  MatMFFDCheckPositivity(void *dummy,Vec U,Vec a,PetscScalar *h)
   ierr   = VecGetArray(U,&u_vec);CHKERRQ(ierr);
   ierr   = VecGetArray(a,&a_vec);CHKERRQ(ierr);
   ierr   = VecGetLocalSize(U,&n);CHKERRQ(ierr);
-  minval = PetscAbsScalar(*h*1.01);
+  minval = PetscAbsScalar(*h)*PetscRealConstant(1.01);
   for (i=0; i<n; i++) {
     if (PetscRealPart(u_vec[i] + *h*a_vec[i]) <= 0.0) {
       val = PetscAbsScalar(u_vec[i]/a_vec[i]);
