@@ -1,42 +1,48 @@
 #include <../src/mat/impls/adj/mpi/mpiadj.h>
-#include <private/fortranimpl.h>
+#include <petsc/private/fortranimpl.h>
 #include <petscmat.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define matpartitioningsetvertexweights_ MATPARTITIONINGSETVERTEXWEIGHTS
 #define matpartitioningview_             MATPARTITIONINGVIEW
 #define matpartitioningsettype_          MATPARTITIONINGSETTYPE
+#define matpartitioningviewfromoptions_  MATPARTITIONINGVIEWFROMOPTIONS
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
 #define matpartitioningsetvertexweights_ matpartitioningsetvertexweights
 #define matpartitioningview_             matpartitioningview
 #define matpartitioningsettype_          matpartitioningsettype
+#define matpartitioningviewfromoptions_  matpartitioningviewfromoptions
 #endif
 
-EXTERN_C_BEGIN
-
-void PETSC_STDCALL matpartitioningsetvertexweights_(MatPartitioning *part,const PetscInt weights[],PetscErrorCode *ierr)
+PETSC_EXTERN void matpartitioningsetvertexweights_(MatPartitioning *part,const PetscInt weights[],PetscErrorCode *ierr)
 {
   PetscInt len;
   PetscInt *array;
   *ierr = MatGetLocalSize((*part)->adj,&len,0); if (*ierr) return;
-  *ierr = PetscMalloc(len*sizeof(PetscInt),&array); if (*ierr) return;
-  *ierr = PetscMemcpy(array,weights,len*sizeof(PetscInt));if (*ierr) return;
+  *ierr = PetscMalloc1(len,&array); if (*ierr) return;
+  *ierr = PetscArraycpy(array,weights,len);if (*ierr) return;
   *ierr = MatPartitioningSetVertexWeights(*part,array);
 }
-void PETSC_STDCALL matpartitioningview_(MatPartitioning  *part,PetscViewer *viewer, PetscErrorCode *ierr)
+PETSC_EXTERN void matpartitioningview_(MatPartitioning *part,PetscViewer *viewer, PetscErrorCode *ierr)
 {
   PetscViewer v;
   PetscPatchDefaultViewers_Fortran(viewer,v);
   *ierr = MatPartitioningView(*part,v);
 }
 
-void PETSC_STDCALL matpartitioningsettype_(MatPartitioning *part,CHAR type PETSC_MIXED_LEN(len),
-                                           PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void matpartitioningsettype_(MatPartitioning *part,char* type,PetscErrorCode *ierr,PETSC_FORTRAN_CHARLEN_T len)
 {
   char *t;
   FIXCHAR(type,len,t);
-  *ierr = MatPartitioningSetType(*part,t);
+  *ierr = MatPartitioningSetType(*part,t);if (*ierr) return;
+  FREECHAR(type,t);
+}
+PETSC_EXTERN void matpartitioningviewfromoptions_(MatPartitioning *ao,PetscObject obj,char* type,PetscErrorCode *ierr,PETSC_FORTRAN_CHARLEN_T len)
+{
+  char *t;
+
+  FIXCHAR(type,len,t);
+  *ierr = MatPartitioningViewFromOptions(*ao,obj,t);if (*ierr) return;
   FREECHAR(type,t);
 }
 
-EXTERN_C_END
