@@ -5,8 +5,6 @@
 const char *const SNESNGMRESRestartTypes[] = {"NONE","PERIODIC","DIFFERENCE","SNESNGMRESRestartType","SNES_NGMRES_RESTART_",0};
 const char *const SNESNGMRESSelectTypes[] = {"NONE","DIFFERENCE","LINESEARCH","SNESNGMRESSelectType","SNES_NGMRES_SELECT_",0};
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_NGMRES"
 PetscErrorCode SNESReset_NGMRES(SNES snes)
 {
   SNES_NGMRES    *ngmres = (SNES_NGMRES*) snes->data;
@@ -19,8 +17,6 @@ PetscErrorCode SNESReset_NGMRES(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESDestroy_NGMRES"
 PetscErrorCode SNESDestroy_NGMRES(SNES snes)
 {
   PetscErrorCode ierr;
@@ -28,10 +24,9 @@ PetscErrorCode SNESDestroy_NGMRES(SNES snes)
 
   PetscFunctionBegin;
   ierr = SNESReset_NGMRES(snes);CHKERRQ(ierr);
-  ierr = PetscFree5(ngmres->h,ngmres->beta,ngmres->xi,ngmres->fnorms,ngmres->q);CHKERRQ(ierr);
-  ierr = PetscFree(ngmres->s);CHKERRQ(ierr);
-  ierr = PetscFree(ngmres->xnorms);CHKERRQ(ierr);
-#if PETSC_USE_COMPLEX
+  ierr = PetscFree4(ngmres->h,ngmres->beta,ngmres->xi,ngmres->q);CHKERRQ(ierr);
+  ierr = PetscFree3(ngmres->xnorms,ngmres->fnorms,ngmres->s);CHKERRQ(ierr);
+#if defined(PETSC_USE_COMPLEX)
   ierr = PetscFree(ngmres->rwork);CHKERRQ(ierr);
 #endif
   ierr = PetscFree(ngmres->work);CHKERRQ(ierr);
@@ -39,8 +34,6 @@ PetscErrorCode SNESDestroy_NGMRES(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_NGMRES"
 PetscErrorCode SNESSetUp_NGMRES(SNES snes)
 {
   SNES_NGMRES    *ngmres = (SNES_NGMRES*) snes->data;
@@ -50,10 +43,10 @@ PetscErrorCode SNESSetUp_NGMRES(SNES snes)
   DM             dm;
 
   PetscFunctionBegin;
-  if (snes->pc && snes->pcside == PC_LEFT && snes->functype == SNES_FUNCTION_UNPRECONDITIONED) {
+  if (snes->npc && snes->npcside== PC_LEFT && snes->functype == SNES_FUNCTION_UNPRECONDITIONED) {
     SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE,"SNESNGMRES does not support left preconditioning with unpreconditioned function");
   }
-  if (snes->pcside == PC_LEFT && snes->functype == SNES_FUNCTION_DEFAULT) snes->functype = SNES_FUNCTION_PRECONDITIONED;
+  if (snes->npcside== PC_LEFT && snes->functype == SNES_FUNCTION_DEFAULT) snes->functype = SNES_FUNCTION_PRECONDITIONED;
   ierr = SNESSetWorkVecs(snes,5);CHKERRQ(ierr);
 
   if (!snes->vec_sol) {
@@ -68,18 +61,13 @@ PetscErrorCode SNESSetUp_NGMRES(SNES snes)
     hsize = msize * msize;
 
     /* explicit least squares minimization solve */
-    ierr = PetscMalloc5(hsize,&ngmres->h, msize,&ngmres->beta, msize,&ngmres->xi, msize,&ngmres->fnorms, hsize,&ngmres->q);CHKERRQ(ierr);
-    ierr = PetscMalloc1(msize,&ngmres->xnorms);CHKERRQ(ierr);
+    ierr = PetscCalloc4(hsize,&ngmres->h, msize,&ngmres->beta, msize,&ngmres->xi, hsize,&ngmres->q);CHKERRQ(ierr);
+    ierr = PetscMalloc3(msize,&ngmres->xnorms,msize,&ngmres->fnorms,msize,&ngmres->s);CHKERRQ(ierr);
     ngmres->nrhs  = 1;
     ngmres->lda   = msize;
     ngmres->ldb   = msize;
-    ierr          = PetscMalloc1(msize,&ngmres->s);CHKERRQ(ierr);
-    ierr          = PetscMemzero(ngmres->h,   hsize*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr          = PetscMemzero(ngmres->q,   hsize*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr          = PetscMemzero(ngmres->xi,  msize*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr          = PetscMemzero(ngmres->beta,msize*sizeof(PetscScalar));CHKERRQ(ierr);
     ngmres->lwork = 12*msize;
-#if PETSC_USE_COMPLEX
+#if defined(PETSC_USE_COMPLEX)
     ierr = PetscMalloc1(ngmres->lwork,&ngmres->rwork);CHKERRQ(ierr);
 #endif
     ierr = PetscMalloc1(ngmres->lwork,&ngmres->work);CHKERRQ(ierr);
@@ -101,14 +89,11 @@ PetscErrorCode SNESSetUp_NGMRES(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetFromOptions_NGMRES"
 PetscErrorCode SNESSetFromOptions_NGMRES(PetscOptionItems *PetscOptionsObject,SNES snes)
 {
   SNES_NGMRES    *ngmres = (SNES_NGMRES*) snes->data;
   PetscErrorCode ierr;
   PetscBool      debug = PETSC_FALSE;
-  SNESLineSearch linesearch;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"SNES NGMRES options");CHKERRQ(ierr);
@@ -133,17 +118,9 @@ PetscErrorCode SNESSetFromOptions_NGMRES(PetscOptionItems *PetscOptionsObject,SN
   ierr = PetscOptionsBool("-snes_ngmres_restart_fm_rise", "Restart on F_M residual rise",  "SNESNGMRESSetRestartFmRise",ngmres->restart_fm_rise,&ngmres->restart_fm_rise,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   if ((ngmres->gammaA > ngmres->gammaC) && (ngmres->gammaC > 2.)) ngmres->gammaC = ngmres->gammaA;
-
-  /* set the default type of the line search if the user hasn't already. */
-  if (!snes->linesearch) {
-    ierr = SNESGetLineSearch(snes,&linesearch);CHKERRQ(ierr);
-    ierr = SNESLineSearchSetType(linesearch,SNESLINESEARCHBASIC);CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESView_NGMRES"
 PetscErrorCode SNESView_NGMRES(SNES snes,PetscViewer viewer)
 {
   SNES_NGMRES    *ngmres = (SNES_NGMRES*) snes->data;
@@ -161,8 +138,6 @@ PetscErrorCode SNESView_NGMRES(SNES snes,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_NGMRES"
 PetscErrorCode SNESSolve_NGMRES(SNES snes)
 {
   SNES_NGMRES          *ngmres = (SNES_NGMRES*) snes->data;
@@ -217,9 +192,9 @@ PetscErrorCode SNESSolve_NGMRES(SNES snes)
 
   /* initialization */
 
-  if (snes->pc && snes->pcside == PC_LEFT) {
+  if (snes->npc && snes->npcside== PC_LEFT) {
     ierr = SNESApplyNPC(snes,X,NULL,F);CHKERRQ(ierr);
-    ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+    ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
     if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
       snes->reason = SNES_DIVERGED_INNER;
       PetscFunctionReturn(0);
@@ -249,15 +224,15 @@ PetscErrorCode SNESSolve_NGMRES(SNES snes)
   ivec      = 0;
   for (k=1; k < snes->max_its+1; k++) {
     /* Computation of x^M */
-    if (snes->pc && snes->pcside == PC_RIGHT) {
+    if (snes->npc && snes->npcside== PC_RIGHT) {
       ierr = VecCopy(X,XM);CHKERRQ(ierr);
-      ierr = SNESSetInitialFunction(snes->pc,F);CHKERRQ(ierr);
+      ierr = SNESSetInitialFunction(snes->npc,F);CHKERRQ(ierr);
 
-      ierr = PetscLogEventBegin(SNES_NPCSolve,snes->pc,XM,B,0);CHKERRQ(ierr);
-      ierr = SNESSolve(snes->pc,B,XM);CHKERRQ(ierr);
-      ierr = PetscLogEventEnd(SNES_NPCSolve,snes->pc,XM,B,0);CHKERRQ(ierr);
+      ierr = PetscLogEventBegin(SNES_NPCSolve,snes->npc,XM,B,0);CHKERRQ(ierr);
+      ierr = SNESSolve(snes->npc,B,XM);CHKERRQ(ierr);
+      ierr = PetscLogEventEnd(SNES_NPCSolve,snes->npc,XM,B,0);CHKERRQ(ierr);
 
-      ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+      ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
       if (reason < 0 && reason != SNES_DIVERGED_MAX_IT) {
         snes->reason = SNES_DIVERGED_INNER;
         PetscFunctionReturn(0);
@@ -350,8 +325,6 @@ PetscErrorCode SNESSolve_NGMRES(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESSetRestartFmRise"
 /*@
  SNESNGMRESSetRestartFmRise - Increase the restart count if the step x_M increases the residual F_M
 
@@ -384,8 +357,6 @@ PetscErrorCode SNESNGMRESSetRestartFmRise(SNES snes,PetscBool flg)
     PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESSetRestartFmRise_NGMRES"
 PetscErrorCode SNESNGMRESSetRestartFmRise_NGMRES(SNES snes,PetscBool flg)
 {
   SNES_NGMRES *ngmres = (SNES_NGMRES*)snes->data;
@@ -395,8 +366,6 @@ PetscErrorCode SNESNGMRESSetRestartFmRise_NGMRES(SNES snes,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESGetRestartFmRise"
 PetscErrorCode SNESNGMRESGetRestartFmRise(SNES snes,PetscBool *flg)
 {
     PetscErrorCode (*f)(SNES,PetscBool*);
@@ -408,8 +377,6 @@ PetscErrorCode SNESNGMRESGetRestartFmRise(SNES snes,PetscBool *flg)
     PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESGetRestartFmRise_NGMRES"
 PetscErrorCode SNESNGMRESGetRestartFmRise_NGMRES(SNES snes,PetscBool *flg)
 {
   SNES_NGMRES *ngmres = (SNES_NGMRES*)snes->data;
@@ -420,8 +387,6 @@ PetscErrorCode SNESNGMRESGetRestartFmRise_NGMRES(SNES snes,PetscBool *flg)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESSetRestartType"
 /*@
     SNESNGMRESSetRestartType - Sets the restart type for SNESNGMRES.
 
@@ -445,7 +410,6 @@ PetscErrorCode SNESNGMRESGetRestartFmRise_NGMRES(SNES snes,PetscBool *flg)
     Notes:
     The default line search used is the L2 line search and it requires two additional function evaluations.
 
-.keywords: SNES, SNESNGMRES, restart, type, set SNESLineSearch
 @*/
 PetscErrorCode SNESNGMRESSetRestartType(SNES snes,SNESNGMRESRestartType rtype)
 {
@@ -457,8 +421,6 @@ PetscErrorCode SNESNGMRESSetRestartType(SNES snes,SNESNGMRESRestartType rtype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESSetSelectType"
 /*@
     SNESNGMRESSetSelectType - Sets the selection type for SNESNGMRES.  This determines how the candidate solution and
     combined solution are used to create the next iterate.
@@ -482,7 +444,6 @@ PetscErrorCode SNESNGMRESSetRestartType(SNES snes,SNESNGMRESRestartType rtype)
     Notes:
     The default line search used is the L2 line search and it requires two additional function evaluations.
 
-.keywords: SNES, SNESNGMRES, selection, type, set SNESLineSearch
 @*/
 PetscErrorCode SNESNGMRESSetSelectType(SNES snes,SNESNGMRESSelectType stype)
 {
@@ -494,8 +455,6 @@ PetscErrorCode SNESNGMRESSetSelectType(SNES snes,SNESNGMRESSelectType stype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESSetSelectType_NGMRES"
 PetscErrorCode SNESNGMRESSetSelectType_NGMRES(SNES snes,SNESNGMRESSelectType stype)
 {
   SNES_NGMRES *ngmres = (SNES_NGMRES*)snes->data;
@@ -505,8 +464,6 @@ PetscErrorCode SNESNGMRESSetSelectType_NGMRES(SNES snes,SNESNGMRESSelectType sty
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGMRESSetRestartType_NGMRES"
 PetscErrorCode SNESNGMRESSetRestartType_NGMRES(SNES snes,SNESNGMRESRestartType rtype)
 {
   SNES_NGMRES *ngmres = (SNES_NGMRES*)snes->data;
@@ -553,12 +510,11 @@ PetscErrorCode SNESNGMRESSetRestartType_NGMRES(SNES snes,SNESNGMRESRestartType r
 .seealso: SNESCreate(), SNES, SNESSetType(), SNESType (for list of available types)
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_NGMRES"
 PETSC_EXTERN PetscErrorCode SNESCreate_NGMRES(SNES snes)
 {
   SNES_NGMRES    *ngmres;
   PetscErrorCode ierr;
+  SNESLineSearch linesearch;
 
   PetscFunctionBegin;
   snes->ops->destroy        = SNESDestroy_NGMRES;
@@ -568,9 +524,11 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NGMRES(SNES snes)
   snes->ops->solve          = SNESSolve_NGMRES;
   snes->ops->reset          = SNESReset_NGMRES;
 
-  snes->usespc   = PETSC_TRUE;
+  snes->usesnpc  = PETSC_TRUE;
   snes->usesksp  = PETSC_FALSE;
-  snes->pcside   = PC_RIGHT;
+  snes->npcside  = PC_RIGHT;
+
+  snes->alwayscomputesfinalresidual = PETSC_TRUE;
 
   ierr          = PetscNewLog(snes,&ngmres);CHKERRQ(ierr);
   snes->data    = (void*) ngmres;
@@ -583,6 +541,9 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NGMRES(SNES snes)
 
   ngmres->candidate = PETSC_FALSE;
 
+  ierr = SNESGetLineSearch(snes,&linesearch);CHKERRQ(ierr);
+  ierr = SNESLineSearchSetType(linesearch,SNESLINESEARCHBASIC);CHKERRQ(ierr);
+
   ngmres->additive_linesearch = NULL;
   ngmres->approxfunc          = PETSC_FALSE;
   ngmres->restart_it          = 2;
