@@ -1,7 +1,5 @@
-#include <private/snesimpl.h>          /*I "petscsnes.h" I*/
+#include <petsc/private/snesimpl.h>          /*I "petscsnes.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESPythonSetType"
 /*@C
    SNESPythonSetType - Initalize a SNES object implemented in Python.
 
@@ -16,13 +14,12 @@
 
    Level: intermediate
 
-.keywords: SNES, Python
-
 .seealso: SNESCreate(), SNESSetType(), SNESPYTHON, PetscPythonInitialize()
 @*/
 PetscErrorCode  SNESPythonSetType(SNES snes,const char pyname[])
 {
   PetscErrorCode ierr;
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(snes,SNES_CLASSID,1);
   PetscValidCharPointer(pyname,2);
