@@ -22,14 +22,12 @@ PetscLogEvent SNESLINESEARCH_Apply;
    Notes:
    There is no way to clear one specific monitor from a SNESLineSearch object.
 
-   This does not clear the monitor set with SNESLineSearchSetDefaultMonitor() use SNESLineSearchSetDefaultMonitor(ls,NULL) to cancel 
+   This does not clear the monitor set with SNESLineSearchSetDefaultMonitor() use SNESLineSearchSetDefaultMonitor(ls,NULL) to cancel
    that one.
 
    Level: intermediate
 
-.keywords: SNESLineSearch, nonlinear, set, monitor
-
-.seealso: SNESLineSearchMonitorDefault(), SNESLineSearchMonitorSet()
+.seealso: SNESGetLineSearch(), SNESLineSearchMonitorDefault(), SNESLineSearchMonitorSet()
 @*/
 PetscErrorCode  SNESLineSearchMonitorCancel(SNESLineSearch ls)
 {
@@ -61,7 +59,7 @@ PetscErrorCode  SNESLineSearchMonitorCancel(SNESLineSearch ls)
 
    Level: developer
 
-.seealso: SNESLineSearchMonitorSet()
+.seealso: SNESGetLineSearch(), SNESLineSearchMonitorSet()
 @*/
 PetscErrorCode  SNESLineSearchMonitor(SNESLineSearch ls)
 {
@@ -95,20 +93,19 @@ PetscErrorCode  SNESLineSearchMonitor(SNESLineSearch ls)
    SNESLineSearchMonitorSet() multiple times; all will be called in the
    order in which they were set.
 
-   Fortran notes: Only a single monitor function can be set for each SNESLineSearch object
+   Fortran Notes:
+    Only a single monitor function can be set for each SNESLineSearch object
 
    Level: intermediate
 
-.keywords: SNESLineSearch, nonlinear, set, monitor
-
-.seealso: SNESLineSearchMonitorDefault(), SNESLineSearchMonitorCancel()
+.seealso: SNESGetLineSearch(), SNESLineSearchMonitorDefault(), SNESLineSearchMonitorCancel()
 @*/
 PetscErrorCode  SNESLineSearchMonitorSet(SNESLineSearch ls,PetscErrorCode (*f)(SNESLineSearch,void*),void *mctx,PetscErrorCode (*monitordestroy)(void**))
 {
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBool      identical;
-  
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ls,SNESLINESEARCH_CLASSID,1);
   for (i=0; i<ls->numbermonitors;i++) {
@@ -133,9 +130,7 @@ PetscErrorCode  SNESLineSearchMonitorSet(SNESLineSearch ls,PetscErrorCode (*f)(S
 
    Level: intermediate
 
-.keywords: SNES, nonlinear, default, monitor, norm
-
-.seealso: SNESMonitorSet(), SNESMonitorSolution()
+.seealso: SNESGetLineSearch(), SNESMonitorSet(), SNESMonitorSolution()
 @*/
 PetscErrorCode  SNESLineSearchMonitorSolutionUpdate(SNESLineSearch ls,PetscViewerAndFormat *vf)
 {
@@ -173,8 +168,6 @@ PetscErrorCode  SNESLineSearchMonitorSolutionUpdate(SNESLineSearch ls,PetscViewe
    The preferred calling sequence for users is to use SNESGetLineSearch() to acquire the SNESLineSearch instance
    already associated with the SNES.  This function is for developer use.
 
-.keywords: LineSearch, create, context
-
 .seealso: LineSearchDestroy(), SNESGetLineSearch()
 @*/
 
@@ -235,9 +228,7 @@ PetscErrorCode SNESLineSearchCreate(MPI_Comm comm, SNESLineSearch *outlinesearch
 
    Level: advanced
 
-.keywords: SNESLineSearch, SetUp
-
-.seealso: SNESLineSearchReset()
+.seealso: SNESGetLineSearch(), SNESLineSearchReset()
 @*/
 
 PetscErrorCode SNESLineSearchSetUp(SNESLineSearch linesearch)
@@ -274,13 +265,12 @@ PetscErrorCode SNESLineSearchSetUp(SNESLineSearch linesearch)
    Input Parameters:
 .  linesearch - The LineSearch instance.
 
-   Notes: Usually only called by SNESReset()
+   Notes:
+    Usually only called by SNESReset()
 
    Level: developer
 
-.keywords: SNESLineSearch, Reset
-
-.seealso: SNESLineSearchSetUp()
+.seealso: SNESGetLineSearch(), SNESLineSearchSetUp()
 @*/
 
 PetscErrorCode SNESLineSearchReset(SNESLineSearch linesearch)
@@ -309,11 +299,10 @@ PetscErrorCode SNESLineSearchReset(SNESLineSearch linesearch)
 
    Level: developer
 
-   Notes: This is used internally by PETSc and not called by users
+   Notes:
+    This is used internally by PETSc and not called by users
 
-.keywords: get, linesearch, pre-check
-
-.seealso: SNESSetFunction()
+.seealso: SNESGetLineSearch(), SNESSetFunction()
 @*/
 PetscErrorCode  SNESLineSearchSetFunction(SNESLineSearch linesearch, PetscErrorCode (*func)(SNES,Vec,Vec))
 {
@@ -323,29 +312,8 @@ PetscErrorCode  SNESLineSearchSetFunction(SNESLineSearch linesearch, PetscErrorC
   PetscFunctionReturn(0);
 }
 
-
-/*MC
-    SNESLineSearchPreCheckFunction - form of function passed to check the search direction before line search is called
-
-     Synopsis:
-     #include <petscsnes.h>
-     SNESLineSearchPreCheckFunction(SNESLineSearch snes,Vec x,Vec y, PetscBool *changed);
-
-       Input Parameters:
-+      x - solution vector
-.      y - search direction vector
--      changed - flag to indicate the precheck changed x or y.
-
-     Note: This is NOTE a PETSc function, rather it documents the calling sequence of functions passed to SNESLineSearchSetPreCheck()
-           and SNESLineSearchGetPreCheck()
-
-   Level: advanced
-
-.seealso:   SNESLineSearchSetPreCheck(), SNESLineSearchGetPreCheck(), SNESLineSearchSetPostCheck(), SNESLineSearchGetPostCheck() 
-M*/
-
 /*@C
-   SNESLineSearchSetPreCheck - Sets a user function that is called after the initial search direction has been computed but 
+   SNESLineSearchSetPreCheck - Sets a user function that is called after the initial search direction has been computed but
          before the line search routine has been applied. Allows the user to adjust the result of (usually a linear solve) that
          determined the search direction.
 
@@ -353,14 +321,12 @@ M*/
 
    Input Parameters:
 +  linesearch - the SNESLineSearch context
-.  func - [optional] function evaluation routine, see SNESLineSearchPreCheckFunction for the calling sequence
+.  func - [optional] function evaluation routine, see SNESLineSearchPreCheck() for the calling sequence
 -  ctx        - [optional] user-defined context for private data for the function evaluation routine (may be NULL)
 
    Level: intermediate
 
-.keywords: set, linesearch, pre-check
-
-.seealso: SNESLineSearchSetPostCheck(), SNESLineSearchGetPostCheck(), SNESLineSearchGetPreCheck()
+.seealso: SNESGetLineSearch(), SNESLineSearchPreCheck(), SNESLineSearchSetPostCheck(), SNESLineSearchGetPostCheck(), SNESLineSearchGetPreCheck()
 @*/
 PetscErrorCode  SNESLineSearchSetPreCheck(SNESLineSearch linesearch, PetscErrorCode (*func)(SNESLineSearch,Vec,Vec,PetscBool*,void*),void *ctx)
 {
@@ -378,14 +344,12 @@ PetscErrorCode  SNESLineSearchSetPreCheck(SNESLineSearch linesearch, PetscErrorC
 .  linesearch - the SNESLineSearch context
 
    Output Parameters:
-+  func       - [optional] function evaluation routine, see SNESLineSearchPreCheckFunction for calling sequence
++  func       - [optional] function evaluation routine, see SNESLineSearchPreCheck() for calling sequence
 -  ctx        - [optional] user-defined context for private data for the function evaluation routine (may be NULL)
 
    Level: intermediate
 
-.keywords: get, linesearch, pre-check
-
-.seealso: SNESLineSearchGetPostCheck(), SNESLineSearchSetPreCheck()
+.seealso: SNESGetLineSearch(), SNESGetLineSearch(), SNESLineSearchPreCheck(), SNESLineSearchGetPostCheck(), SNESLineSearchSetPreCheck(), SNESLineSearchSetPostCheck()
 @*/
 PetscErrorCode  SNESLineSearchGetPreCheck(SNESLineSearch linesearch, PetscErrorCode (**func)(SNESLineSearch,Vec,Vec,PetscBool*,void*),void **ctx)
 {
@@ -396,28 +360,6 @@ PetscErrorCode  SNESLineSearchGetPreCheck(SNESLineSearch linesearch, PetscErrorC
   PetscFunctionReturn(0);
 }
 
-/*MC
-    SNESLineSearchPostCheckFunction - form of function that is called after line search is complete
-
-     Synopsis:
-     #include <petscsnes.h>
-     SNESLineSearchPostheckFunction(SNESLineSearch linesearch,Vec x,Vec y,  Vec w, *changed_y, PetscBool *changed_w);
-
-     Input Parameters:
-+      x - old solution vector
-.      y - search direction vector
-.      w - new solution vector
-.      changed_y - indicates that the line search changed y
--      changed_w - indicates that the line search changed w
-
-     Note: This is NOTE a PETSc function, rather it documents the calling sequence of functions passed to SNESLineSearchSetPostCheck()
-           and SNESLineSearchGetPostCheck()
-
-   Level: advanced
-
-.seealso:   SNESLineSearchSetPreCheck(), SNESLineSearchSetPostCheck(), SNESLineSearchGetPreCheck(), SNESLineSearchGetPostCheck()
-M*/
-
 /*@C
    SNESLineSearchSetPostCheck - Sets a user function that is called after the line search has been applied to determine the step
        direction and length. Allows the user a chance to change or override the decision of the line search routine
@@ -426,14 +368,12 @@ M*/
 
    Input Parameters:
 +  linesearch - the SNESLineSearch context
-.  func - [optional] function evaluation routine, see SNESLineSearchPostCheckFunction for the calling sequence
+.  func - [optional] function evaluation routine, see SNESLineSearchPostCheck()  for the calling sequence
 -  ctx        - [optional] user-defined context for private data for the function evaluation routine (may be NULL)
 
    Level: intermediate
 
-.keywords: set, linesearch, post-check
-
-.seealso: SNESLineSearchSetPreCheck()
+.seealso: SNESGetLineSearch(), SNESLineSearchPostCheck(), SNESLineSearchSetPreCheck(), SNESLineSearchGetPreCheck(), SNESLineSearchGetPostCheck()
 @*/
 PetscErrorCode  SNESLineSearchSetPostCheck(SNESLineSearch linesearch, PetscErrorCode (*func)(SNESLineSearch,Vec,Vec,Vec,PetscBool*,PetscBool*,void*),void *ctx)
 {
@@ -451,14 +391,12 @@ PetscErrorCode  SNESLineSearchSetPostCheck(SNESLineSearch linesearch, PetscError
 .  linesearch - the SNESLineSearch context
 
    Output Parameters:
-+  func - [optional] function evaluation routine, see for the calling sequence SNESLineSearchPostCheckFunction
++  func - [optional] function evaluation routine, see for the calling sequence SNESLineSearchPostCheck()
 -  ctx        - [optional] user-defined context for private data for the function evaluation routine (may be NULL)
 
    Level: intermediate
 
-.keywords: get, linesearch, post-check
-
-.seealso: SNESLineSearchGetPreCheck(), SNESLineSearchSetPostCheck()
+.seealso: SNESGetLineSearch(), SNESLineSearchGetPreCheck(), SNESLineSearchSetPostCheck(), SNESLineSearchPostCheck(), SNESLineSearchSetPreCheck()
 @*/
 PetscErrorCode  SNESLineSearchGetPostCheck(SNESLineSearch linesearch, PetscErrorCode (**func)(SNESLineSearch,Vec,Vec,Vec,PetscBool*,PetscBool*,void*),void **ctx)
 {
@@ -484,9 +422,7 @@ PetscErrorCode  SNESLineSearchGetPostCheck(SNESLineSearch linesearch, PetscError
 
    Level: developer
 
-.keywords: SNESLineSearch, Create
-
-.seealso: SNESLineSearchPostCheck()
+.seealso: SNESGetLineSearch(), SNESLineSearchPostCheck(), SNESLineSearchSetPreCheck(), SNESLineSearchGetPreCheck(), SNESLineSearchSetPostCheck(), SNESLineSearchGetPostCheck()
 @*/
 PetscErrorCode SNESLineSearchPreCheck(SNESLineSearch linesearch,Vec X,Vec Y,PetscBool *changed)
 {
@@ -518,9 +454,7 @@ PetscErrorCode SNESLineSearchPreCheck(SNESLineSearch linesearch,Vec X,Vec Y,Pets
 
    Level: developer
 
-.keywords: SNESLineSearch, Create
-
-.seealso: SNESLineSearchPreCheck()
+.seealso: SNESGetLineSearch(), SNESLineSearchPreCheck(), SNESLineSearchSetPostCheck(), SNESLineSearchGetPostCheck(), SNESLineSearchSetPrecheck(), SNESLineSearchGetPrecheck()
 @*/
 PetscErrorCode SNESLineSearchPostCheck(SNESLineSearch linesearch,Vec X,Vec Y,Vec W,PetscBool *changed_Y,PetscBool *changed_W)
 {
@@ -568,7 +502,7 @@ PetscErrorCode SNESLineSearchPostCheck(SNESLineSearch linesearch,Vec X,Vec Y,Vec
    Reference:
    Hindmarsh and Payne (1996) Time step limits for stable solutions of the ice sheet equation, Annals of Glaciology.
 
-.seealso: SNESLineSearchSetPreCheck()
+.seealso: SNESGetLineSearch(), SNESLineSearchSetPreCheck()
 @*/
 PetscErrorCode SNESLineSearchPreCheckPicard(SNESLineSearch linesearch,Vec X,Vec Y,PetscBool *changed,void *ctx)
 {
@@ -606,10 +540,11 @@ PetscErrorCode SNESLineSearchPreCheckPicard(SNESLineSearch linesearch,Vec X,Vec
     PetscReal alpha,ydiffnorm;
     ierr  = VecAXPY(Ylast,-1.0,Y);CHKERRQ(ierr);
     ierr  = VecNorm(Ylast,NORM_2,&ydiffnorm);CHKERRQ(ierr);
-    alpha = ylastnorm / ydiffnorm;
+    alpha = (ydiffnorm > .001*ylastnorm) ? ylastnorm / ydiffnorm : 1000.0;
     ierr  = VecCopy(Y,Ylast);CHKERRQ(ierr);
     ierr  = VecScale(Y,alpha);CHKERRQ(ierr);
     ierr  = PetscInfo3(snes,"Angle %14.12e degrees less than threshold %14.12e, corrected step by alpha=%14.12e\n",(double)(theta*180./PETSC_PI),(double)angle,(double)alpha);CHKERRQ(ierr);
+    *changed = PETSC_TRUE;
   } else {
     ierr     = PetscInfo2(snes,"Angle %14.12e degrees exceeds threshold %14.12e, no correction applied\n",(double)(theta*180./PETSC_PI),(double)angle);CHKERRQ(ierr);
     ierr     = VecCopy(Y,Ylast);CHKERRQ(ierr);
@@ -648,14 +583,12 @@ PetscErrorCode SNESLineSearchPreCheckPicard(SNESLineSearch linesearch,Vec X,Vec
    help with convergence of the nonlinear method.  Various SNES types use line searches
    in different ways, but the overarching theme is that a line search is used to determine
    an optimal damping parameter of a step at each iteration of the method.  Each
-   application of the line search may invoke SNESComputeFunction several times, and
+   application of the line search may invoke SNESComputeFunction() several times, and
    therefore may be fairly expensive.
 
    Level: Intermediate
 
-.keywords: SNESLineSearch, Create
-
-.seealso: SNESLineSearchCreate(), SNESLineSearchPreCheck(), SNESLineSearchPostCheck(), SNESSolve(), SNESComputeFunction(), SNESLineSearchSetComputeNorms(),
+.seealso: SNESGetLineSearch(), SNESLineSearchCreate(), SNESLineSearchPreCheck(), SNESLineSearchPostCheck(), SNESSolve(), SNESComputeFunction(), SNESLineSearchSetComputeNorms(),
           SNESLineSearchType, SNESLineSearchSetType()
 @*/
 PetscErrorCode SNESLineSearchApply(SNESLineSearch linesearch, Vec X, Vec F, PetscReal * fnorm, Vec Y)
@@ -701,11 +634,9 @@ PetscErrorCode SNESLineSearchApply(SNESLineSearch linesearch, Vec X, Vec F, Pets
    Input Parameters:
 .  linesearch - The linesearch context
 
-   Level: Intermediate
+   Level: developer
 
-.keywords: SNESLineSearch, Destroy
-
-.seealso: SNESLineSearchCreate(), SNESLineSearchReset(), SNESDestroy()
+.seealso: SNESGetLineSearch(), SNESLineSearchCreate(), SNESLineSearchReset(), SNESDestroy()
 @*/
 PetscErrorCode SNESLineSearchDestroy(SNESLineSearch * linesearch)
 {
@@ -738,11 +669,11 @@ PetscErrorCode SNESLineSearchDestroy(SNESLineSearch * linesearch)
 
    Level: intermediate
 
-   Developer Note: This monitor is implemented differently than the other SNESLineSearchMonitors that are set with 
-     SNESLineSearchMonitorSet() since it is called in many locations of the line search routines to display aspects of the 
+   Developer Note: This monitor is implemented differently than the other SNESLineSearchMonitors that are set with
+     SNESLineSearchMonitorSet() since it is called in many locations of the line search routines to display aspects of the
      line search that are not visible to the other monitors.
 
-.seealso: SNESLineSearchGetDefaultMonitor(), PetscViewer, SNESLineSearchSetMonitor()
+.seealso: SNESGetLineSearch(), SNESLineSearchGetDefaultMonitor(), PetscViewer, SNESLineSearchSetMonitor()
 @*/
 PetscErrorCode  SNESLineSearchSetDefaultMonitor(SNESLineSearch linesearch, PetscViewer viewer)
 {
@@ -771,7 +702,7 @@ PetscErrorCode  SNESLineSearchSetDefaultMonitor(SNESLineSearch linesearch, Petsc
 
    Level: intermediate
 
-.seealso: SNESLineSearchSetDefaultMonitor(), PetscViewer
+.seealso: SNESGetLineSearch(), SNESLineSearchSetDefaultMonitor(), PetscViewer
 @*/
 PetscErrorCode  SNESLineSearchGetDefaultMonitor(SNESLineSearch linesearch, PetscViewer *monitor)
 {
@@ -815,7 +746,7 @@ PetscErrorCode  SNESLineSearchMonitorSetFromOptions(SNESLineSearch ls,const char
   PetscBool         flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)ls),((PetscObject)ls)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)ls),((PetscObject) ls)->options,((PetscObject)ls)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     PetscViewerAndFormat *vf;
     ierr = PetscViewerAndFormatCreate(viewer,format,&vf);CHKERRQ(ierr);
@@ -849,7 +780,7 @@ PetscErrorCode  SNESLineSearchMonitorSetFromOptions(SNESLineSearch ls,const char
 . -snes_linesearch_damping - The linesearch damping parameter
 . -snes_linesearch_keeplambda - Keep the previous search length as the initial guess.
 . -snes_linesearch_precheck_picard - Use precheck that speeds up convergence of picard method
-- -snes_linesearch_precheck_picard_angle - Angle used in picard precheck method
+- -snes_linesearch_precheck_picard_angle - Angle used in Picard precheck method
 
    Logically Collective on SNESLineSearch
 
@@ -878,13 +809,13 @@ PetscErrorCode SNESLineSearchSetFromOptions(SNESLineSearch linesearch)
     ierr = SNESLineSearchSetType(linesearch,deft);CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)linesearch),((PetscObject)linesearch)->prefix,"-snes_linesearch_monitor",&viewer,NULL,&set);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)linesearch),((PetscObject) linesearch)->options,((PetscObject)linesearch)->prefix,"-snes_linesearch_monitor",&viewer,NULL,&set);CHKERRQ(ierr);
   if (set) {
     ierr = SNESLineSearchSetDefaultMonitor(linesearch,viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
   ierr = SNESLineSearchMonitorSetFromOptions(linesearch,"-snes_linesearch_monitor_solution_update","View correction at each iteration","SNESLineSearchMonitorSolutionUpdate",SNESLineSearchMonitorSolutionUpdate,NULL);CHKERRQ(ierr);
-  
+
   /* tolerances */
   ierr = PetscOptionsReal("-snes_linesearch_minlambda","Minimum step length","SNESLineSearchSetTolerances",linesearch->steptol,&linesearch->steptol,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-snes_linesearch_maxstep","Maximum step size","SNESLineSearchSetTolerances",linesearch->maxstep,&linesearch->maxstep,NULL);CHKERRQ(ierr);
@@ -1103,7 +1034,7 @@ PetscErrorCode  SNESLineSearchGetLambda(SNESLineSearch linesearch,PetscReal *lam
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(linesearch,SNESLINESEARCH_CLASSID,1);
-  PetscValidPointer(lambda, 2);
+  PetscValidRealPointer(lambda, 2);
   *lambda = linesearch->lambda;
   PetscFunctionReturn(0);
 }
@@ -1163,27 +1094,27 @@ PetscErrorCode  SNESLineSearchGetTolerances(SNESLineSearch linesearch,PetscReal
   PetscFunctionBegin;
   PetscValidHeaderSpecific(linesearch,SNESLINESEARCH_CLASSID,1);
   if (steptol) {
-    PetscValidPointer(steptol, 2);
+    PetscValidRealPointer(steptol, 2);
     *steptol = linesearch->steptol;
   }
   if (maxstep) {
-    PetscValidPointer(maxstep, 3);
+    PetscValidRealPointer(maxstep, 3);
     *maxstep = linesearch->maxstep;
   }
   if (rtol) {
-    PetscValidPointer(rtol, 4);
+    PetscValidRealPointer(rtol, 4);
     *rtol = linesearch->rtol;
   }
   if (atol) {
-    PetscValidPointer(atol, 5);
+    PetscValidRealPointer(atol, 5);
     *atol = linesearch->atol;
   }
   if (ltol) {
-    PetscValidPointer(ltol, 6);
+    PetscValidRealPointer(ltol, 6);
     *ltol = linesearch->ltol;
   }
   if (max_its) {
-    PetscValidPointer(max_its, 7);
+    PetscValidIntPointer(max_its, 7);
     *max_its = linesearch->max_its;
   }
   PetscFunctionReturn(0);
@@ -1273,13 +1204,13 @@ PetscErrorCode  SNESLineSearchGetDamping(SNESLineSearch linesearch,PetscReal *da
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(linesearch,SNESLINESEARCH_CLASSID,1);
-  PetscValidPointer(damping, 2);
+  PetscValidRealPointer(damping, 2);
   *damping = linesearch->damping;
   PetscFunctionReturn(0);
 }
 
 /*@
-   SNESLineSearchSetDamping - Sets the line search damping paramter.
+   SNESLineSearchSetDamping - Sets the line search damping parameter.
 
    Input Parameters:
 +  linesearch - linesearch context
@@ -1329,7 +1260,7 @@ PetscErrorCode  SNESLineSearchGetOrder(SNESLineSearch linesearch,PetscInt *order
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(linesearch,SNESLINESEARCH_CLASSID,1);
-  PetscValidPointer(order, 2);
+  PetscValidIntPointer(order, 2);
   *order = linesearch->order;
   PetscFunctionReturn(0);
 }
@@ -1338,8 +1269,8 @@ PetscErrorCode  SNESLineSearchGetOrder(SNESLineSearch linesearch,PetscInt *order
    SNESLineSearchSetOrder - Sets the maximum order of the polynomial fit used in the line search
 
    Input Parameters:
-.  linesearch - linesearch context
-.  order - The damping parameter
++  linesearch - linesearch context
+-  order - The damping parameter
 
    Level: intermediate
 
@@ -1585,8 +1516,6 @@ PetscErrorCode SNESLineSearchSetVecs(SNESLineSearch linesearch,Vec X,Vec F,Vec Y
 
    Level: advanced
 
-.keywords: SNESLineSearch, append, options, prefix, database
-
 .seealso: SNESGetOptionsPrefix()
 @*/
 PetscErrorCode  SNESLineSearchAppendOptionsPrefix(SNESLineSearch linesearch,const char prefix[])
@@ -1617,8 +1546,6 @@ PetscErrorCode  SNESLineSearchAppendOptionsPrefix(SNESLineSearch linesearch,cons
 
    Level: advanced
 
-.keywords: SNESLineSearch, get, options, prefix, database
-
 .seealso: SNESAppendOptionsPrefix()
 @*/
 PetscErrorCode  SNESLineSearchGetOptionsPrefix(SNESLineSearch linesearch,const char *prefix[])
@@ -1640,10 +1567,6 @@ PetscErrorCode  SNESLineSearchGetOptionsPrefix(SNESLineSearch linesearch,const c
 
    Level: developer
 
-   Developers Note: This is PETSC_EXTERN because it may be used by user written plugin SNES implementations
-
-.keywords: SNESLineSearch, work, vector
-
 .seealso: SNESSetWorkVecs()
 @*/
 PetscErrorCode  SNESLineSearchSetWorkVecs(SNESLineSearch linesearch, PetscInt nwork)
@@ -1749,11 +1672,9 @@ PetscErrorCode  SNESLineSearchSetReason(SNESLineSearch linesearch, SNESLineSearc
 
     Level: developer
 
-.keywords: SNES, line search, VI, nonlinear, set, line search
-
 .seealso: SNESLineSearchGetVIFunctions(), SNESLineSearchSetPostCheck(), SNESLineSearchSetPreCheck()
 @*/
-extern PetscErrorCode SNESLineSearchSetVIFunctions(SNESLineSearch linesearch, SNESLineSearchVIProjectFunc projectfunc, SNESLineSearchVINormFunc normfunc)
+PetscErrorCode SNESLineSearchSetVIFunctions(SNESLineSearch linesearch, SNESLineSearchVIProjectFunc projectfunc, SNESLineSearchVINormFunc normfunc)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(linesearch,SNESLINESEARCH_CLASSID,1);
@@ -1776,11 +1697,9 @@ extern PetscErrorCode SNESLineSearchSetVIFunctions(SNESLineSearch linesearch, SN
 
     Level: developer
 
-.keywords: SNES, line search, VI, nonlinear, get, line search
-
 .seealso: SNESLineSearchSetVIFunctions(), SNESLineSearchGetPostCheck(), SNESLineSearchGetPreCheck()
 @*/
-extern PetscErrorCode SNESLineSearchGetVIFunctions(SNESLineSearch linesearch, SNESLineSearchVIProjectFunc *projectfunc, SNESLineSearchVINormFunc *normfunc)
+PetscErrorCode SNESLineSearchGetVIFunctions(SNESLineSearch linesearch, SNESLineSearchVIProjectFunc *projectfunc, SNESLineSearchVINormFunc *normfunc)
 {
   PetscFunctionBegin;
   if (projectfunc) *projectfunc = linesearch->ops->viproject;
@@ -1798,6 +1717,7 @@ PetscErrorCode  SNESLineSearchRegister(const char sname[],PetscErrorCode (*funct
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = SNESInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&SNESLineSearchList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
