@@ -1,11 +1,11 @@
 
 #include <petsc/private/viewerimpl.h>  /*I "petscviewer.h" I*/
-#include <petsc/private/hash.h>
+#include <petsc/private/hashtable.h>
 #if defined(PETSC_HAVE_SAWS)
 #include <petscviewersaws.h>
 #endif
 
-PetscFunctionList PetscViewerList = 0;
+PetscFunctionList PetscViewerList = NULL;
 
 
 PetscOptionsHelpPrinted PetscOptionsHelpPrintedSingleton = NULL;
@@ -71,7 +71,7 @@ PetscErrorCode PetscOptionsHelpPrintedCheck(PetscOptionsHelpPrinted hp,const cha
   size_t          l1,l2;
 #if !defined(PETSC_HAVE_THREADSAFETY)
   char            *both;
-  khint_t         newitem;
+  int             newitem;
 #endif
   PetscErrorCode  ierr;
 
@@ -111,7 +111,8 @@ static PetscInt  inoviewers = 0;
 
   Level: developer
 
-  Notes: Calling XXXViewFromOptions in an inner loop can be very expensive.  This can appear, for example, when using
+  Notes:
+    Calling XXXViewFromOptions in an inner loop can be very expensive.  This can appear, for example, when using
    many small subsolves.  Call this function to control viewer creation in PetscOptionsGetViewer, thus removing the expensive XXXViewFromOptions calls.
 
 .seealso: PetscOptionsGetViewer(), PetscOptionsPopGetViewerOff()
@@ -133,7 +134,8 @@ PetscErrorCode  PetscOptionsPushGetViewerOff(PetscBool flg)
 
   Level: developer
 
-  Notes: Calling XXXViewFromOptions in an inner loop can be very expensive.  This can appear, for example, when using
+  Notes:
+    Calling XXXViewFromOptions in an inner loop can be very expensive.  This can appear, for example, when using
    many small subsolves.  Call this function to control viewer creation in PetscOptionsGetViewer, thus removing the expensive XXXViewFromOptions calls.
 
 .seealso: PetscOptionsGetViewer(), PetscOptionsPushGetViewerOff()
@@ -156,7 +158,8 @@ PetscErrorCode  PetscOptionsPopGetViewerOff(void)
 
   Level: developer
 
-  Notes: Calling XXXViewFromOptions in an inner loop can be very expensive.  This can appear, for example, when using
+  Notes:
+    Calling XXXViewFromOptions in an inner loop can be very expensive.  This can appear, for example, when using
    many small subsolves.
 
 .seealso: PetscOptionsGetViewer(), PetscOptionsPushGetViewerOff(), PetscOptionsPopGetViewerOff()
@@ -164,7 +167,7 @@ PetscErrorCode  PetscOptionsPopGetViewerOff(void)
 PetscErrorCode  PetscOptionsGetViewerOff(PetscBool *flg)
 {
   PetscFunctionBegin;
-  PetscValidPointer(flg,0);
+  PetscValidBoolPointer(flg,1);
   *flg = noviewer;
   PetscFunctionReturn(0);
 }
@@ -172,7 +175,7 @@ PetscErrorCode  PetscOptionsGetViewerOff(PetscBool *flg)
 /*@C
    PetscOptionsGetViewer - Gets a viewer appropriate for the type indicated by the user
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameters:
 +  comm - the communicator to own the viewer
@@ -186,7 +189,8 @@ PetscErrorCode  PetscOptionsGetViewerOff(PetscBool *flg)
 
    Level: intermediate
 
-   Notes: If no value is provided ascii:stdout is used
+   Notes:
+    If no value is provided ascii:stdout is used
 $       ascii[:[filename][:[format][:append]]]    defaults to stdout - format can be one of ascii_info, ascii_info_detail, or ascii_matlab,
                                                   for example ascii::ascii_info prints just the information about the object not all details
                                                   unless :append is given filename opens in write mode, overwriting what was already there
@@ -212,23 +216,22 @@ $       saws[:communicatorname]                    publishes object to the Scien
           PetscOptionsFList(), PetscOptionsEList(), PetscOptionsPushGetViewerOff(), PetscOptionsPopGetViewerOff(),
           PetscOptionsGetViewerOff()
 @*/
-PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,const char pre[],const char name[],PetscViewer *viewer,PetscViewerFormat *format,PetscBool  *set)
+PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,PetscOptions options,const char pre[],const char name[],PetscViewer *viewer,PetscViewerFormat *format,PetscBool  *set)
 {
-  char                           *value;
+  const char                     *value;
   PetscErrorCode                 ierr;
   PetscBool                      flag,hashelp;
 
   PetscFunctionBegin;
   PetscValidCharPointer(name,3);
 
-  if (set) *set = PETSC_FALSE;
-#if defined(PETSC_SKIP_VIEWFROMOPTIONS)
-  PetscFunctionReturn(0);
-#endif
+  if (viewer) *viewer = NULL;
+  if (format) *format = PETSC_VIEWER_DEFAULT;
+  if (set)    *set    = PETSC_FALSE;
   ierr = PetscOptionsGetViewerOff(&flag);CHKERRQ(ierr);
   if (flag) PetscFunctionReturn(0);
 
-  ierr = PetscOptionsHasName(NULL,NULL,"-help",&hashelp);CHKERRQ(ierr);
+  ierr = PetscOptionsHasHelp(NULL,&hashelp);CHKERRQ(ierr);
   if (hashelp) {
     PetscBool found;
 
@@ -236,21 +239,18 @@ PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,const char pre[],const char
       ierr = PetscOptionsHelpPrintedCreate(&PetscOptionsHelpPrintedSingleton);CHKERRQ(ierr);
     }
     ierr = PetscOptionsHelpPrintedCheck(PetscOptionsHelpPrintedSingleton,pre,name,&found);CHKERRQ(ierr);
-    if (!found) {
-      if (viewer) {
-        ierr = (*PetscHelpPrintf)(comm,"\n  -%s%s ascii[:[filename][:[format][:append]]]: %s (%s)\n",pre ? pre : "",name+1,"Prints object to stdout or ASCII file","PetscOptionsGetViewer");CHKERRQ(ierr);
-        ierr = (*PetscHelpPrintf)(comm,"    -%s%s binary[:[filename][:[format][:append]]]: %s (%s)\n",pre ? pre : "",name+1,"Saves object to a binary file","PetscOptionsGetViewer");CHKERRQ(ierr);
-        ierr = (*PetscHelpPrintf)(comm,"    -%s%s draw[:[drawtype][:filename|format]] %s (%s)\n",pre ? pre : "",name+1,"Draws object","PetscOptionsGetViewer");CHKERRQ(ierr);
-        ierr = (*PetscHelpPrintf)(comm,"    -%s%s socket[:port]: %s (%s)\n",pre ? pre : "",name+1,"Pushes object to a Unix socket","PetscOptionsGetViewer");CHKERRQ(ierr);
-        ierr = (*PetscHelpPrintf)(comm,"    -%s%s saws[:communicatorname]: %s (%s)\n\n",pre ? pre : "",name+1,"Publishes object to SAWs","PetscOptionsGetViewer");CHKERRQ(ierr);
-      } else {
-        ierr = (*PetscHelpPrintf)(comm,"    -%s%s\n",pre ? pre : "",name+1);CHKERRQ(ierr);
-      }
+    if (!found && viewer) {
+      ierr = (*PetscHelpPrintf)(comm,"-------------------------------------------------\nViewer (-%s%s) options:\n",pre ? pre : "",name+1);CHKERRQ(ierr);
+      ierr = (*PetscHelpPrintf)(comm,"  -%s%s ascii[:[filename][:[format][:append]]]: %s (%s)\n",pre ? pre : "",name+1,"Prints object to stdout or ASCII file","PetscOptionsGetViewer");CHKERRQ(ierr);
+      ierr = (*PetscHelpPrintf)(comm,"  -%s%s binary[:[filename][:[format][:append]]]: %s (%s)\n",pre ? pre : "",name+1,"Saves object to a binary file","PetscOptionsGetViewer");CHKERRQ(ierr);
+      ierr = (*PetscHelpPrintf)(comm,"  -%s%s draw[:[drawtype][:filename|format]] %s (%s)\n",pre ? pre : "",name+1,"Draws object","PetscOptionsGetViewer");CHKERRQ(ierr);
+      ierr = (*PetscHelpPrintf)(comm,"  -%s%s socket[:port]: %s (%s)\n",pre ? pre : "",name+1,"Pushes object to a Unix socket","PetscOptionsGetViewer");CHKERRQ(ierr);
+      ierr = (*PetscHelpPrintf)(comm,"  -%s%s saws[:communicatorname]: %s (%s)\n",pre ? pre : "",name+1,"Publishes object to SAWs","PetscOptionsGetViewer");CHKERRQ(ierr);
     }
   }
 
   if (format) *format = PETSC_VIEWER_DEFAULT;
-  ierr = PetscOptionsFindPair_Private(NULL,pre,name,&value,&flag);CHKERRQ(ierr);
+  ierr = PetscOptionsFindPair(options,pre,name,&value,&flag);CHKERRQ(ierr);
   if (flag) {
     if (set) *set = PETSC_TRUE;
     if (!value) {
@@ -261,7 +261,7 @@ PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,const char pre[],const char
     } else {
       char       *loc0_vtype,*loc1_fname,*loc2_fmt = NULL,*loc3_fmode = NULL;
       PetscInt   cnt;
-      const char *viewers[] = {PETSCVIEWERASCII,PETSCVIEWERBINARY,PETSCVIEWERDRAW,PETSCVIEWERSOCKET,PETSCVIEWERMATLAB,PETSCVIEWERSAWS,PETSCVIEWERVTK,PETSCVIEWERHDF5,PETSCVIEWERGLVIS,0};
+      const char *viewers[] = {PETSCVIEWERASCII,PETSCVIEWERBINARY,PETSCVIEWERDRAW,PETSCVIEWERSOCKET,PETSCVIEWERMATLAB,PETSCVIEWERSAWS,PETSCVIEWERVTK,PETSCVIEWERHDF5,PETSCVIEWERGLVIS,PETSCVIEWEREXODUSII,NULL};
 
       ierr = PetscStrallocpy(value,&loc0_vtype);CHKERRQ(ierr);
       ierr = PetscStrchr(loc0_vtype,':',&loc1_fname);CHKERRQ(ierr);
@@ -311,6 +311,11 @@ PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,const char pre[],const char
           case 8:
             if (!(*viewer = PETSC_VIEWER_GLVIS_(comm))) CHKERRQ(PETSC_ERR_PLIB);
             break;
+#if defined(PETSC_HAVE_EXODUSII)
+          case 9:
+            if (!(*viewer = PETSC_VIEWER_EXODUSII_(comm))) CHKERRQ(PETSC_ERR_PLIB);
+            break;
+#endif
           default: SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported viewer %s",loc0_vtype);
           }
           ierr = PetscObjectReference((PetscObject)*viewer);CHKERRQ(ierr);
@@ -341,6 +346,7 @@ PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,const char pre[],const char
             if (*loc1_fname) {
               ierr = PetscViewerDrawSetDrawType(*viewer,loc1_fname);CHKERRQ(ierr);
             }
+            ierr = PetscViewerSetFromOptions(*viewer);CHKERRQ(ierr);
           }
         }
       }
@@ -365,7 +371,7 @@ PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,const char pre[],const char
 /*@
    PetscViewerCreate - Creates a viewing context
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -375,10 +381,6 @@ PetscErrorCode  PetscOptionsGetViewer(MPI_Comm comm,const char pre[],const char
 
    Level: advanced
 
-   Concepts: graphics^creating PetscViewer
-   Concepts: file input/output^creating PetscViewer
-   Concepts: sockets^creating PetscViewer
-
 .seealso: PetscViewerDestroy(), PetscViewerSetType(), PetscViewerType
 
 @*/
@@ -388,11 +390,11 @@ PetscErrorCode  PetscViewerCreate(MPI_Comm comm,PetscViewer *inviewer)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  *inviewer = 0;
+  *inviewer = NULL;
   ierr = PetscViewerInitializePackage();CHKERRQ(ierr);
-  ierr         = PetscHeaderCreate(viewer,PETSC_VIEWER_CLASSID,"PetscViewer","PetscViewer","Viewer",comm,PetscViewerDestroy,NULL);CHKERRQ(ierr);
+  ierr         = PetscHeaderCreate(viewer,PETSC_VIEWER_CLASSID,"PetscViewer","PetscViewer","Viewer",comm,PetscViewerDestroy,PetscViewerView);CHKERRQ(ierr);
   *inviewer    = viewer;
-  viewer->data = 0;
+  viewer->data = NULL;
   PetscFunctionReturn(0);
 }
 
@@ -406,7 +408,7 @@ PetscErrorCode  PetscViewerCreate(MPI_Comm comm,PetscViewer *inviewer)
 -  type        - for example, PETSCVIEWERASCII
 
    Options Database Command:
-.  -draw_type  <type> - Sets the type; use -help for a list
+.  -viewer_type  <type> - Sets the type; use -help for a list
     of available methods (for instance, ascii)
 
    Level: advanced
@@ -433,7 +435,7 @@ PetscErrorCode  PetscViewerSetType(PetscViewer viewer,PetscViewerType type)
     ierr         = (*viewer->ops->destroy)(viewer);CHKERRQ(ierr);
 
     viewer->ops->destroy = NULL;
-    viewer->data         = 0;
+    viewer->data         = NULL;
   }
   ierr = PetscMemzero(viewer->ops,sizeof(struct _PetscViewerOps));CHKERRQ(ierr);
 
@@ -468,8 +470,6 @@ $     PetscViewerSetType(viewer,"my_viewer_type")
    or at runtime via the option
 $     -viewer_type my_viewer_type
 
-  Concepts: registering^Viewers
-
 .seealso: PetscViewerRegisterAll(), PetscViewerRegisterDestroy()
  @*/
 PetscErrorCode  PetscViewerRegister(const char *sname,PetscErrorCode (*function)(PetscViewer))
@@ -477,6 +477,7 @@ PetscErrorCode  PetscViewerRegister(const char *sname,PetscErrorCode (*function)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = PetscViewerInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&PetscViewerList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -495,8 +496,6 @@ PetscErrorCode  PetscViewerRegister(const char *sname,PetscErrorCode (*function)
    Notes:
     Must be called after PetscViewerCreate() before the PetscViewer is used.
 
-  Concepts: PetscViewer^setting options
-
 .seealso: PetscViewerCreate(), PetscViewerSetType(), PetscViewerType
 
 @*/
