@@ -13,12 +13,6 @@
 
 #include <petsc/private/fortranimpl.h>
 
-#if defined(PETSC_HAVE_CUDA)
-#include <cublas_v2.h>
-
-extern cublasHandle_t cublasv2handle;
-#endif
-
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscinitialize_              PETSCINITIALIZE
 #define petscinitializenoarguments_   PETSCINITIALIZENOARGUMENTS
@@ -27,7 +21,7 @@ extern cublasHandle_t cublasv2handle;
 #define iargc_                        IARGC
 #define getarg_                       GETARG
 #define mpi_init_                     MPI_INIT
-#define petscgetcommoncomm_           PETSCGETCOMMONCOMM
+#define petscgetcomm_                 PETSCGETCOMM
 #define petsccommandargumentcount_    PETSCCOMMANDARGUMENTCOUNT
 #define petscgetcommandargument_      PETSCGETCOMMANDARGUMENT
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
@@ -38,7 +32,7 @@ extern cublasHandle_t cublasv2handle;
 #define mpi_init_                     mpi_init
 #define iargc_                        iargc
 #define getarg_                       getarg
-#define petscgetcommoncomm_           petscgetcommoncomm
+#define petscgetcomm_                 petscgetcomm
 #define petsccommandargumentcount_    petsccommandargumentcount
 #define petscgetcommandargument_      petscgetcommandargument
 #endif
@@ -72,7 +66,7 @@ extern cublasHandle_t cublasv2handle;
 #undef getarg_
 #define iargc_ petsccommandargumentcount_
 #define getarg_ petscgetcommandargument_
-#elif defined(PETSC_HAVE_GFORTRAN_IARGC) /* gfortran from gcc4 */
+#elif defined(PETSC_HAVE__GFORTRAN_IARGC) /* gfortran from gcc4 */
 #undef iargc_
 #undef getarg_
 #define iargc_  _gfortran_iargc
@@ -108,8 +102,8 @@ extern cublasHandle_t cublasv2handle;
 #endif   /* mpi_init_ */
 #endif   /* PETSC_HAVE_MPIUNI */
 
-PETSC_EXTERN void PETSC_STDCALL mpi_init_(int*);
-PETSC_EXTERN void PETSC_STDCALL petscgetcommoncomm_(PetscMPIInt*);
+PETSC_EXTERN void mpi_init_(int*);
+PETSC_EXTERN void petscgetcomm_(PetscMPIInt*);
 
 /*
      Different Fortran compilers handle command lines in different ways
@@ -121,10 +115,6 @@ PETSC_EXTERN void getarg_(int*,char*,int);
 PETSC_EXTERN short __stdcall NARGS();
 PETSC_EXTERN void __stdcall GETARG(short*,char*,int,short *);
 
-#elif defined(PETSC_HAVE_FORTRAN_STDCALL)
-PETSC_EXTERN int PETSC_STDCALL IARGC();
-PETSC_EXTERN void PETSC_STDCALL GETARG(int *,char *,int);
-
 #elif defined(PETSC_HAVE_PXFGETARG_NEW)
 PETSC_EXTERN int iargc_();
 PETSC_EXTERN void getarg_(int*,char*,int*,int*,int);
@@ -141,7 +131,7 @@ PETSC_EXTERN void PXFGETARG(int*,_fcd,int*,int*);
 #endif
 
 #if (defined(PETSC_HAVE_COMPLEX) && !defined(PETSC_HAVE_MPI_C_DOUBLE_COMPLEX)) || defined(PETSC_USE_REAL___FLOAT128) || defined(PETSC_USE_REAL___FP16)
-extern MPI_Op MPIU_SUM;
+PETSC_EXTERN MPI_Op MPIU_SUM;
 
 PETSC_EXTERN void MPIAPI PetscSum_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 
@@ -154,20 +144,19 @@ PETSC_EXTERN void MPIAPI PetscMin_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 #endif
 
 PETSC_INTERN void MPIAPI MPIU_MaxSum_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
-PETSC_EXTERN PetscMPIInt MPIAPI Petsc_DelCounter(MPI_Comm,PetscMPIInt,void*,void*);
-PETSC_EXTERN PetscMPIInt MPIAPI Petsc_DelComm_Inner(MPI_Comm,PetscMPIInt,void*,void*);
-PETSC_EXTERN PetscMPIInt MPIAPI Petsc_DelComm_Outer(MPI_Comm,PetscMPIInt,void*,void*);
+PETSC_EXTERN PetscMPIInt MPIAPI Petsc_Counter_Attr_Delete_Fn(MPI_Comm,PetscMPIInt,void*,void*);
+PETSC_EXTERN PetscMPIInt MPIAPI Petsc_InnerComm_Attr_Delete_Fn(MPI_Comm,PetscMPIInt,void*,void*);
+PETSC_EXTERN PetscMPIInt MPIAPI Petsc_OuterComm_Attr_Delete_Fn(MPI_Comm,PetscMPIInt,void*,void*);
 
-extern PetscErrorCode  PetscOptionsCheckInitial_Private(void);
-extern PetscErrorCode  PetscOptionsCheckInitial_Components(void);
-extern PetscErrorCode  PetscInitialize_DynamicLibraries(void);
+PETSC_INTERN PetscErrorCode PetscOptionsCheckInitial_Private(void);
+PETSC_INTERN PetscErrorCode PetscInitialize_DynamicLibraries(void);
 #if defined(PETSC_USE_LOG)
-extern PetscErrorCode  PetscLogInitialize(void);
+PETSC_INTERN PetscErrorCode PetscLogInitialize(void);
 #endif
-extern PetscErrorCode  PetscMallocAlign(size_t,int,const char[],const char[],void**);
-extern PetscErrorCode  PetscFreeAlign(void*,int,const char[],const char[]);
-extern int  PetscGlobalArgc;
-extern char **PetscGlobalArgs;
+PETSC_EXTERN PetscErrorCode PetscMallocAlign(size_t,PetscBool,int,const char[],const char[],void**);
+PETSC_EXTERN PetscErrorCode PetscFreeAlign(void*,int,const char[],const char[]);
+PETSC_INTERN int  PetscGlobalArgc;
+PETSC_INTERN char **PetscGlobalArgs;
 
 /*
     Reads in Fortran command line argments and sends them to
@@ -198,7 +187,7 @@ PetscErrorCode PETScParseFortranArgs_Private(int *argc,char ***argv)
   ierr = MPI_Bcast(argc,1,MPI_INT,0,PETSC_COMM_WORLD);CHKERRQ(ierr);
 
   /* PetscTrMalloc() not yet set, so don't use PetscMalloc() */
-  ierr = PetscMallocAlign((*argc+1)*(warg*sizeof(char)+sizeof(char*)),0,0,0,(void**)argv);CHKERRQ(ierr);
+  ierr = PetscMallocAlign((*argc+1)*(warg*sizeof(char)+sizeof(char*)),PETSC_FALSE,0,0,0,(void**)argv);CHKERRQ(ierr);
   (*argv)[0] = (char*)(*argv + *argc + 1);
 
   if (!rank) {
@@ -240,23 +229,23 @@ PetscErrorCode PETScParseFortranArgs_Private(int *argc,char ***argv)
 }
 
 #if defined(PETSC_SERIALIZE_FUNCTIONS)
-extern PetscFPT PetscFPTData;
+PETSC_INTERN PetscFPT PetscFPTData;
 #endif
 
-#if defined(PETSC_HAVE_THREADSAFETY)
-extern PetscSpinlock PetscViewerASCIISpinLockOpen;
-extern PetscSpinlock PetscViewerASCIISpinLockStdout;
-extern PetscSpinlock PetscViewerASCIISpinLockStderr;
-extern PetscSpinlock PetscCommSpinLock;
+#if defined(PETSC_HAVE_ADIOS)
+#include <adios.h>
+#include <adios_read.h>
 #endif
-
 /* -----------------------------------------------------------------------------------------------*/
 
 #if defined(PETSC_HAVE_SAWS)
 #include <petscviewersaws.h>
-extern PetscErrorCode  PetscInitializeSAWs(const char[]);
+PETSC_INTERN PetscErrorCode  PetscInitializeSAWs(const char[]);
 #endif
 
+PETSC_EXTERN PetscMPIInt MPIAPI Petsc_ShmComm_Attr_Delete_Fn(MPI_Comm,PetscMPIInt,void *,void *);
+PETSC_INTERN PetscErrorCode PetscPreMPIInit_Private();
+
 /*
     petscinitialize - Version called from Fortran.
 
@@ -269,10 +258,6 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
   int            j,i;
 #if defined (PETSC_USE_NARGS)
   short          flg;
-#endif
-#if defined(PETSC_HAVE_CUDA)
-  PetscBool      flg2;
-  cublasStatus_t cberr;
 #endif
   int            flag;
   PetscMPIInt    size;
@@ -319,12 +304,14 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Calling PetscSetProgramName()\n");return;}
 
   /* check if PETSC_COMM_WORLD is initialized by the user in fortran */
-  petscgetcommoncomm_(&f_petsc_comm_world);
+  petscgetcomm_(&f_petsc_comm_world);
   MPI_Initialized(&flag);
   if (!flag) {
     PetscMPIInt mierr;
 
     if (f_petsc_comm_world) {(*PetscErrorPrintf)("You cannot set PETSC_COMM_WORLD if you have not initialized MPI first\n");return;}
+
+    *ierr = PetscPreMPIInit_Private(); if(*ierr) return;
     mpi_init_(&mierr);
     if (mierr) {
       *ierr = mierr;
@@ -335,6 +322,8 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
   }
   if (f_petsc_comm_world) PETSC_COMM_WORLD = MPI_Comm_f2c(*(MPI_Fint*)&f_petsc_comm_world); /* User called MPI_INITIALIZE() and changed PETSC_COMM_WORLD */
   else PETSC_COMM_WORLD = MPI_COMM_WORLD;
+  *ierr = MPI_Comm_set_errhandler(PETSC_COMM_WORLD,MPI_ERRORS_RETURN);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Setting MPI error handler\n");return;}
   PetscInitializeCalled = PETSC_TRUE;
 
   *ierr = PetscSpinlockCreate(&PetscViewerASCIISpinLockOpen);
@@ -355,6 +344,13 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
 
   MPIU_BOOL = MPI_INT;
   MPIU_ENUM = MPI_INT;
+  MPIU_FORTRANADDR = (sizeof(void*) == sizeof(int)) ? MPI_INT : MPIU_INT64;
+  if (sizeof(size_t) == sizeof(unsigned)) MPIU_SIZE_T = MPI_UNSIGNED;
+  else if (sizeof(size_t) == sizeof(unsigned long)) MPIU_SIZE_T = MPI_UNSIGNED_LONG;
+#if defined(PETSC_SIZEOF_LONG_LONG)
+  else if (sizeof(size_t) == sizeof(unsigned long long)) MPIU_SIZE_T = MPI_UNSIGNED_LONG_LONG;
+#endif
+  else {(*PetscErrorPrintf)("PetscInitialize: Could not find MPI type for size_t\n"); return;}
 
 #if defined(PETSC_HAVE_COMPLEX)
   /*
@@ -427,17 +423,19 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI types\n");return;}
   *ierr = MPI_Type_commit(&MPIU_2SCALAR);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI types\n");return;}
-#if defined(PETSC_USE_64BIT_INDICES) || !defined(MPI_2INT)
+#if defined(PETSC_USE_64BIT_INDICES)
   *ierr = MPI_Type_contiguous(2,MPIU_INT,&MPIU_2INT);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI types\n");return;}
   *ierr = MPI_Type_commit(&MPIU_2INT);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI types\n");return;}
 #endif
-  *ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,Petsc_DelCounter,&Petsc_Counter_keyval,(void*)0);
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_Counter_Attr_Delete_Fn,&Petsc_Counter_keyval,(void*)0);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
-  *ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,Petsc_DelComm_Outer,&Petsc_InnerComm_keyval,(void*)0);
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_InnerComm_Attr_Delete_Fn,&Petsc_InnerComm_keyval,(void*)0);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
-  *ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,Petsc_DelComm_Inner,&Petsc_OuterComm_keyval,(void*)0);
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_OuterComm_Attr_Delete_Fn,&Petsc_OuterComm_keyval,(void*)0);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_ShmComm_Attr_Delete_Fn,&Petsc_ShmComm_keyval,(void*)0);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
 
   /*
@@ -451,11 +449,15 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
     PETScParseFortranArgs_Private(&PetscGlobalArgc,&PetscGlobalArgs);
     FIXCHAR(filename,len,t1);
     *ierr = PetscOptionsInsert(NULL,&PetscGlobalArgc,&PetscGlobalArgs,t1);
-    FREECHAR(filename,t1);
     if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating options database\n");return;}
+    FREECHAR(filename,t1);
+    if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Freeing string in creating options database\n");return;}
   }
   *ierr = PetscOptionsCheckInitial_Private();
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Checking initial options\n");return;}
+  /* call a second time to check options database */
+  *ierr = PetscErrorPrintfInitialize();
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Calling PetscErrorPrintfInitialize()\n");return;}
   *ierr = PetscCitationsInitialize();
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:PetscCitationsInitialize()\n");return;}
 #if defined(PETSC_HAVE_SAWS)
@@ -480,8 +482,6 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
   if (*ierr) { (*PetscErrorPrintf)("PetscInitialize:Getting hostname\n");return;}
   *ierr = PetscInfo1(0,"Running on machine: %s\n",hostname);
   if (*ierr) { (*PetscErrorPrintf)("PetscInitialize:Calling PetscInfo()\n");return;}
-  *ierr = PetscOptionsCheckInitial_Components();
-  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Checking initial options\n");return;}
 
 #if defined(PETSC_USE_DEBUG) && !defined(PETSC_HAVE_THREADSAFETY)
   *ierr = PetscStackCreate();
@@ -492,29 +492,30 @@ static void petscinitialize_internal(char* filename, PetscInt len, PetscBool rea
   *ierr = PetscFPTCreate(10000);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:PetscFPTCreate()\n");return;}
 #endif
-
-#if defined(PETSC_HAVE_CUDA)
-  flg2  = PETSC_TRUE;
-  *ierr = PetscOptionsGetBool(NULL,NULL,"-cublas",&flg2,NULL);
-  if (flg2) {
-    cberr = cublasCreate(&cublasv2handle);
-    if (((int)cberr) != (int)CUBLAS_STATUS_SUCCESS) {(*PetscErrorPrintf)("PetscInitialize:CUBLAS error %d\n",cberr);return;}
-  }
+#if defined(PETSC_HAVE_ADIOS)
+  *ierr = adios_init_noxml(PETSC_COMM_WORLD);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_init_noxml()\n");return;}
+  *ierr = adios_declare_group(&Petsc_adios_group,"PETSc","",adios_stat_default);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_declare_group()\n");return;}
+  *ierr = adios_select_method(Petsc_adios_group,"MPI","","");
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_select_method()\n");return;}
+  *ierr = adios_read_init_method(ADIOS_READ_METHOD_BP,PETSC_COMM_WORLD,"");
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_read_init_method()\n");return;}
 #endif
 }
 
-PETSC_EXTERN void PETSC_STDCALL petscinitialize_(char* filename PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void petscinitialize_(char* filename,PetscErrorCode *ierr,PETSC_FORTRAN_CHARLEN_T len)
 {
   petscinitialize_internal(filename, len, PETSC_TRUE, ierr);
 }
 
-PETSC_EXTERN void PETSC_STDCALL petscinitializenoarguments_(PetscErrorCode *ierr)
+PETSC_EXTERN void petscinitializenoarguments_(PetscErrorCode *ierr)
 {
   petscinitialize_internal(NULL, (PetscInt) 0, PETSC_FALSE, ierr);
 }
 
 
-PETSC_EXTERN void PETSC_STDCALL petscfinalize_(PetscErrorCode *ierr)
+PETSC_EXTERN void petscfinalize_(PetscErrorCode *ierr)
 {
 #if defined(PETSC_HAVE_SUNMATHPRO)
   extern void standard_arithmetic();
@@ -526,7 +527,7 @@ PETSC_EXTERN void PETSC_STDCALL petscfinalize_(PetscErrorCode *ierr)
   *ierr = PetscFinalize();
 }
 
-PETSC_EXTERN void PETSC_STDCALL petscend_(PetscErrorCode *ierr)
+PETSC_EXTERN void petscend_(PetscErrorCode *ierr)
 {
 #if defined(PETSC_HAVE_SUNMATHPRO)
   extern void standard_arithmetic();
