@@ -81,7 +81,7 @@ PetscErrorCode  PetscStrToArray(const char s[],char sp,int *argc,char ***args)
     }
   }
   free(lens);
-  (*args)[*argc] = 0;
+  (*args)[*argc] = NULL;
 
   *argc = 0;
   for (i=0; i<n; i++) {
@@ -168,13 +168,18 @@ PetscErrorCode  PetscStrlen(const char s[],size_t *len)
       Null string returns a new null string
 
       Not for use in Fortran
+      
+      Warning: If t has previously been allocated then that memory is lost, you may need to PetscFree() 
+      the array before calling this routine.
+
+.seealso: PetscStrArrayallocpy(), PetscStrcpy(), PetscStrNArrayallocpy()
 
 @*/
 PetscErrorCode  PetscStrallocpy(const char s[],char *t[])
 {
   PetscErrorCode ierr;
   size_t         len;
-  char           *tmp = 0;
+  char           *tmp = NULL;
 
   PetscFunctionBegin;
   if (s) {
@@ -202,7 +207,10 @@ PetscErrorCode  PetscStrallocpy(const char s[],char *t[])
    Note:
       Not for use in Fortran
 
-.seealso: PetscStrallocpy() PetscStrArrayDestroy()
+      Warning: If t has previously been allocated then that memory is lost, you may need to PetscStrArrayDestroy() 
+      the array before calling this routine.
+      
+.seealso: PetscStrallocpy(), PetscStrArrayDestroy(), PetscStrNArrayallocpy()
 
 @*/
 PetscErrorCode  PetscStrArrayallocpy(const char *const *list,char ***t)
@@ -268,7 +276,7 @@ PetscErrorCode PetscStrArrayDestroy(char ***list)
    Note:
       Not for use in Fortran
 
-.seealso: PetscStrallocpy() PetscStrArrayDestroy()
+.seealso: PetscStrallocpy(), PetscStrArrayallocpy(), PetscStrNArrayDestroy()
 
 @*/
 PetscErrorCode  PetscStrNArrayallocpy(PetscInt n,const char *const *list,char ***t)
@@ -332,6 +340,8 @@ PetscErrorCode PetscStrNArrayDestroy(PetscInt n,char ***list)
      Null string returns a string starting with zero
 
      Not for use in Fortran
+     
+     It is recommended you use PetscStrncpy() instead of this routine
 
 .seealso: PetscStrncpy(), PetscStrcat(), PetscStrlcat()
 
@@ -401,6 +411,8 @@ PetscErrorCode  PetscStrncpy(char s[],const char t[],size_t n)
 
    Notes:
     Not for use in Fortran
+    
+    It is recommended you use PetscStrlcat() instead of this routine
 
 .seealso: PetscStrcpy(), PetscStrncpy(), PetscStrlcat()
 
@@ -846,7 +858,7 @@ PetscErrorCode  PetscStrendswithwhich(const char a[],const char *const *bs,Petsc
 @*/
 PetscErrorCode  PetscStrrstr(const char a[],const char b[],char *tmp[])
 {
-  const char *stmp = a, *ltmp = 0;
+  const char *stmp = a, *ltmp = NULL;
 
   PetscFunctionBegin;
   while (stmp) {
@@ -921,7 +933,7 @@ PetscErrorCode  PetscTokenFind(PetscToken a,char *result[])
 
   PetscFunctionBegin;
   *result = a->current;
-  if (ptr && !*ptr) {*result = 0;PetscFunctionReturn(0);}
+  if (ptr && !*ptr) {*result = NULL; PetscFunctionReturn(0);}
   token = a->token;
   if (ptr && (*ptr == '"')) {token = '"';(*result)++;ptr++;}
   while (ptr) {
@@ -932,7 +944,7 @@ PetscErrorCode  PetscTokenFind(PetscToken a,char *result[])
       break;
     }
     if (!*ptr) {
-      a->current = 0;
+      a->current = NULL;
       break;
     }
     ptr++;
@@ -1092,8 +1104,8 @@ PetscErrorCode  PetscStrreplace(MPI_Comm comm,const char aa[],char b[],size_t le
   int            i = 0;
   size_t         l,l1,l2,l3;
   char           *work,*par,*epar,env[1024],*tfree,*a = (char*)aa;
-  const char     *s[] = {"${PETSC_ARCH}","${PETSC_DIR}","${PETSC_LIB_DIR}","${DISPLAY}","${HOMEDIRECTORY}","${WORKINGDIRECTORY}","${USERNAME}","${HOSTNAME}",0};
-  char           *r[] = {0,0,0,0,0,0,0,0,0};
+  const char     *s[] = {"${PETSC_ARCH}","${PETSC_DIR}","${PETSC_LIB_DIR}","${DISPLAY}","${HOMEDIRECTORY}","${WORKINGDIRECTORY}","${USERNAME}","${HOSTNAME}",NULL};
+  char           *r[] = {NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
   PetscBool      flag;
 
   PetscFunctionBegin;
