@@ -6,9 +6,16 @@ PetscBool TaoRegisterAllCalled = PETSC_FALSE;
 PetscFunctionList TaoList = NULL;
 
 PetscClassId TAO_CLASSID;
-PetscLogEvent Tao_Solve, Tao_ObjectiveEval, Tao_GradientEval, Tao_ObjGradientEval, Tao_HessianEval, Tao_ConstraintsEval, Tao_JacobianEval;
 
-const char *TaoSubSetTypes[] = {  "subvec","mask","matrixfree","TaoSubSetType","TAO_SUBSET_",0};
+PetscLogEvent TAO_Solve;
+PetscLogEvent TAO_ObjectiveEval;
+PetscLogEvent TAO_GradientEval;
+PetscLogEvent TAO_ObjGradEval;
+PetscLogEvent TAO_HessianEval;
+PetscLogEvent TAO_JacobianEval;
+PetscLogEvent TAO_ConstraintsEval;
+
+const char *TaoSubSetTypes[] = {"subvec","mask","matrixfree","TaoSubSetType","TAO_SUBSET_",0};
 
 struct _n_TaoMonitorDrawCtx {
   PetscViewer viewer;
@@ -18,7 +25,7 @@ struct _n_TaoMonitorDrawCtx {
 /*@
   TaoCreate - Creates a TAO solver
 
-  Collective on MPI_Comm
+  Collective
 
   Input Parameter:
 . comm - MPI communicator
@@ -63,7 +70,8 @@ PetscErrorCode TaoCreate(MPI_Comm comm, Tao *newtao)
   tao->ops->computeobjectiveandgradient=0;
   tao->ops->computegradient=0;
   tao->ops->computehessian=0;
-  tao->ops->computeseparableobjective=0;
+  tao->ops->computeresidual=0;
+  tao->ops->computeresidualjacobian=0;
   tao->ops->computeconstraints=0;
   tao->ops->computejacobian=0;
   tao->ops->computejacobianequality=0;
@@ -81,12 +89,13 @@ PetscErrorCode TaoCreate(MPI_Comm comm, Tao *newtao)
 
   tao->solution=NULL;
   tao->gradient=NULL;
-  tao->sep_objective = NULL;
+  tao->ls_res = NULL;
+  tao->ls_jac = NULL;
   tao->constraints=NULL;
   tao->constraints_equality=NULL;
   tao->constraints_inequality=NULL;
-  tao->sep_weights_v=NULL;
-  tao->sep_weights_w=NULL;
+  tao->res_weights_v=NULL;
+  tao->res_weights_w=NULL;
   tao->stepdirection=NULL;
   tao->niter=0;
   tao->ntotalits=0;
@@ -145,6 +154,10 @@ PetscErrorCode TaoCreate(MPI_Comm comm, Tao *newtao)
   tao->viewjacobian=PETSC_FALSE;
   tao->viewconstraints = PETSC_FALSE;
 
+  tao->bounded = PETSC_FALSE;
+
+  tao->header_printed = PETSC_FALSE;
+
   /* These flags prevents algorithms from overriding user options */
   tao->max_it_changed   =PETSC_FALSE;
   tao->max_funcs_changed=PETSC_FALSE;
@@ -194,17 +207,17 @@ PetscErrorCode TaoSolve(Tao tao)
                                 "Institution = {Argonne National Laboratory},\n"
                                 "Year   = 2014,\n"
                                 "Number = {ANL/MCS-TM-322 - Revision 3.5},\n"
-                                "url    = {http://www.mcs.anl.gov/tao}\n}\n",&set);CHKERRQ(ierr);
-
+                                "url    = {https://www.mcs.anl.gov/research/projects/tao/}\n}\n",&set);CHKERRQ(ierr);
+  tao->header_printed = PETSC_FALSE;
   ierr = TaoSetUp(tao);CHKERRQ(ierr);
   ierr = TaoResetStatistics(tao);CHKERRQ(ierr);
   if (tao->linesearch) {
     ierr = TaoLineSearchReset(tao->linesearch);CHKERRQ(ierr);
   }
 
-  ierr = PetscLogEventBegin(Tao_Solve,tao,0,0,0);CHKERRQ(ierr);
+  ierr = PetscLogEventBegin(TAO_Solve,tao,0,0,0);CHKERRQ(ierr);
   if (tao->ops->solve){ ierr = (*tao->ops->solve)(tao);CHKERRQ(ierr); }
-  ierr = PetscLogEventEnd(Tao_Solve,tao,0,0,0);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(TAO_Solve,tao,0,0,0);CHKERRQ(ierr);
 
   ierr = VecViewFromOptions(tao->solution,(PetscObject)tao,"-tao_view_solution");CHKERRQ(ierr);
 
@@ -248,7 +261,7 @@ PetscErrorCode TaoSetUp(Tao tao)
   PetscValidHeaderSpecific(tao, TAO_CLASSID,1);
   if (tao->setupcalled) PetscFunctionReturn(0);
 
-  if (!tao->solution) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetInitialVector");
+  if (!tao->solution) SETERRQ(PetscObjectComm((PetscObject)tao),PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetInitialVector");
   if (tao->ops->setup) {
     ierr = (*tao->ops->setup)(tao);CHKERRQ(ierr);
   }
@@ -292,6 +305,7 @@ PetscErrorCode TaoDestroy(Tao *tao)
   }
   ierr = VecDestroy(&(*tao)->solution);CHKERRQ(ierr);
   ierr = VecDestroy(&(*tao)->gradient);CHKERRQ(ierr);
+  ierr = VecDestroy(&(*tao)->ls_res);CHKERRQ(ierr);
 
   if ((*tao)->gradient_norm) {
     ierr = PetscObjectDereference((PetscObject)(*tao)->gradient_norm);CHKERRQ(ierr);
@@ -309,6 +323,8 @@ PetscErrorCode TaoDestroy(Tao *tao)
   ierr = VecDestroy(&(*tao)->stepdirection);CHKERRQ(ierr);
   ierr = MatDestroy(&(*tao)->hessian_pre);CHKERRQ(ierr);
   ierr = MatDestroy(&(*tao)->hessian);CHKERRQ(ierr);
+  ierr = MatDestroy(&(*tao)->ls_jac);CHKERRQ(ierr);
+  ierr = MatDestroy(&(*tao)->ls_jac_pre);CHKERRQ(ierr);
   ierr = MatDestroy(&(*tao)->jacobian_pre);CHKERRQ(ierr);
   ierr = MatDestroy(&(*tao)->jacobian);CHKERRQ(ierr);
   ierr = MatDestroy(&(*tao)->jacobian_state_pre);CHKERRQ(ierr);
@@ -321,18 +337,15 @@ PetscErrorCode TaoDestroy(Tao *tao)
   ierr = MatDestroy(&(*tao)->jacobian_inequality_pre);CHKERRQ(ierr);
   ierr = ISDestroy(&(*tao)->state_is);CHKERRQ(ierr);
   ierr = ISDestroy(&(*tao)->design_is);CHKERRQ(ierr);
-  ierr = VecDestroy(&(*tao)->sep_weights_v);CHKERRQ(ierr);
+  ierr = VecDestroy(&(*tao)->res_weights_v);CHKERRQ(ierr);
   ierr = TaoCancelMonitors(*tao);CHKERRQ(ierr);
   if ((*tao)->hist_malloc) {
-    ierr = PetscFree((*tao)->hist_obj);CHKERRQ(ierr);
-    ierr = PetscFree((*tao)->hist_resid);CHKERRQ(ierr);
-    ierr = PetscFree((*tao)->hist_cnorm);CHKERRQ(ierr);
-    ierr = PetscFree((*tao)->hist_lits);CHKERRQ(ierr);
+    ierr = PetscFree4((*tao)->hist_obj,(*tao)->hist_resid,(*tao)->hist_cnorm,(*tao)->hist_lits);CHKERRQ(ierr);
   }
-  if ((*tao)->sep_weights_n) {
-    ierr = PetscFree((*tao)->sep_weights_rows);CHKERRQ(ierr);
-    ierr = PetscFree((*tao)->sep_weights_cols);CHKERRQ(ierr);
-    ierr = PetscFree((*tao)->sep_weights_w);CHKERRQ(ierr);
+  if ((*tao)->res_weights_n) {
+    ierr = PetscFree((*tao)->res_weights_rows);CHKERRQ(ierr);
+    ierr = PetscFree((*tao)->res_weights_cols);CHKERRQ(ierr);
+    ierr = PetscFree((*tao)->res_weights_w);CHKERRQ(ierr);
   }
   ierr = PetscHeaderDestroy(tao);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -361,7 +374,7 @@ PetscErrorCode TaoDestroy(Tao *tao)
 . -tao_smonitor - same as tao_monitor, but truncates very small values
 . -tao_cmonitor - prints function value, residual, and constraint norm at each iteration
 . -tao_view_solution - prints solution vector at each iteration
-. -tao_view_separableobjective - prints separable objective vector at each iteration
+. -tao_view_ls_residual - prints least-squares residual vector at each iteration
 . -tao_view_step - prints step direction vector at each iteration
 . -tao_view_gradient - prints gradient vector at each iteration
 . -tao_draw_solution - graphically view solution vector at each iteration
@@ -383,7 +396,7 @@ PetscErrorCode TaoDestroy(Tao *tao)
 PetscErrorCode TaoSetFromOptions(Tao tao)
 {
   PetscErrorCode ierr;
-  const TaoType  default_type = TAOLMVM;
+  TaoType        default_type = TAOLMVM;
   char           type[256], monfilename[PETSC_MAX_PATH_LEN];
   PetscViewer    monviewer;
   PetscBool      flg;
@@ -449,10 +462,10 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
       ierr = TaoSetMonitor(tao,TaoStepDirectionMonitor,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsString("-tao_view_separableobjective","view separable objective vector after each evaluation","TaoSetMonitor","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsString("-tao_view_residual","view least-squares residual vector after each evaluation","TaoSetMonitor","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = PetscViewerASCIIOpen(comm,monfilename,&monviewer);CHKERRQ(ierr);
-      ierr = TaoSetMonitor(tao,TaoSeparableObjectiveMonitor,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
+      ierr = TaoSetMonitor(tao,TaoResidualMonitor,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
     }
 
     ierr = PetscOptionsString("-tao_monitor","Use the default convergence monitor","TaoSetMonitor","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
@@ -461,6 +474,12 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
       ierr = TaoSetMonitor(tao,TaoMonitorDefault,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
     }
 
+    ierr = PetscOptionsString("-tao_gmonitor","Use the convergence monitor with extra globalization info","TaoSetMonitor","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+    if (flg) {
+      ierr = PetscViewerASCIIOpen(comm,monfilename,&monviewer);CHKERRQ(ierr);
+      ierr = TaoSetMonitor(tao,TaoDefaultGMonitor,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
+    }
+
     ierr = PetscOptionsString("-tao_smonitor","Use the short convergence monitor","TaoSetMonitor","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = PetscViewerASCIIOpen(comm,monfilename,&monviewer);CHKERRQ(ierr);
@@ -535,6 +554,29 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
   PetscFunctionReturn(0);
 }
 
+/*@C
+   TaoViewFromOptions - View from Options
+
+   Collective on Tao
+
+   Input Parameters:
++  A - the  Tao context
+.  obj - Optional object
+-  name - command line option
+
+   Level: intermediate
+.seealso:  Tao, TaoView, PetscObjectViewFromOptions(), TaoCreate()
+@*/
+PetscErrorCode  TaoViewFromOptions(Tao A,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(A,TAO_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)A,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
   TaoView - Prints information about the Tao
 
@@ -563,7 +605,7 @@ PetscErrorCode TaoView(Tao tao, PetscViewer viewer)
 {
   PetscErrorCode      ierr;
   PetscBool           isascii,isstring;
-  const TaoType type;
+  TaoType             type;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao,TAO_CLASSID,1);
@@ -1168,8 +1210,6 @@ PetscErrorCode TaoGetKSP(Tao tao, KSP *ksp)
 
    Level: intermediate
 
-.keywords: TAO
-
 .seealso:  TaoGetKSP()
 @*/
 PetscErrorCode  TaoGetLinearSolveIterations(Tao tao,PetscInt *lits)
@@ -1316,6 +1356,36 @@ PetscErrorCode TaoResetStatistics(Tao tao)
   PetscFunctionReturn(0);
 }
 
+/*@C
+  TaoSetUpdate - Sets the general-purpose update function called
+  at the beginning of every iteration of the nonlinear solve. Specifically
+  it is called at the top of every iteration, after the new solution and the gradient
+  is determined, but before the Hessian is computed (if applicable).
+
+  Logically Collective on Tao
+
+  Input Parameters:
++ tao - The tao solver context
+- func - The function
+
+  Calling sequence of func:
+$ func (Tao tao, PetscInt step);
+
+. step - The current step of the iteration
+
+  Level: advanced
+
+.seealso TaoSolve()
+@*/
+PetscErrorCode  TaoSetUpdate(Tao tao, PetscErrorCode (*func)(Tao, PetscInt,void*), void *ctx)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(tao, TAO_CLASSID,1);
+  tao->ops->update = func;
+  tao->user_update = ctx;
+  PetscFunctionReturn(0);
+}
+
 /*@C
   TaoSetConvergenceTest - Sets the function that is to be used to test
   for convergence o fthe iterative minimization solution.  The new convergence
@@ -1365,7 +1435,7 @@ PetscErrorCode TaoSetConvergenceTest(Tao tao, PetscErrorCode (*conv)(Tao,void*),
           monitor routine (may be NULL)
 
    Calling sequence of mymonitor:
-$     int mymonitor(Tao tao,void *mctx)
+$     PetscErrorCode mymonitor(Tao tao,void *mctx)
 
 +    tao - the Tao solver context
 -    mctx - [optional] monitoring context
@@ -1377,7 +1447,7 @@ $     int mymonitor(Tao tao,void *mctx)
 .    -tao_cmonitor       - same as smonitor plus constraint norm
 .    -tao_view_solution   - view solution at each iteration
 .    -tao_view_gradient   - view gradient at each iteration
-.    -tao_view_separableobjective - view separable objective function at each iteration
+.    -tao_view_ls_residual - view least-squares residual vector at each iteration
 -    -tao_cancelmonitors - cancels all monitors that have been hardwired into a code by calls to TaoSetMonitor(), but does not cancel those set via the options database.
 
 
@@ -1386,7 +1456,8 @@ $     int mymonitor(Tao tao,void *mctx)
    TaoSetMonitor() multiple times; all will be called in the
    order in which they were set.
 
-   Fortran Notes: Only one monitor function may be set
+   Fortran Notes:
+    Only one monitor function may be set
 
    Level: intermediate
 
@@ -1396,21 +1467,18 @@ PetscErrorCode TaoSetMonitor(Tao tao, PetscErrorCode (*func)(Tao, void*), void *
 {
   PetscErrorCode ierr;
   PetscInt       i;
+  PetscBool      identical;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao,TAO_CLASSID,1);
-  if (tao->numbermonitors >= MAXTAOMONITORS) SETERRQ1(PETSC_COMM_SELF,1,"Cannot attach another monitor -- max=",MAXTAOMONITORS);
+  if (tao->numbermonitors >= MAXTAOMONITORS) SETERRQ1(PetscObjectComm((PetscObject)tao),1,"Cannot attach another monitor -- max=",MAXTAOMONITORS);
 
   for (i=0; i<tao->numbermonitors;i++) {
-    if (func == tao->monitor[i] && dest == tao->monitordestroy[i] && ctx == tao->monitorcontext[i]) {
-      if (dest) {
-        ierr = (*dest)(&ctx);CHKERRQ(ierr);
-      }
-      PetscFunctionReturn(0);
-    }
+    ierr = PetscMonitorCompare((PetscErrorCode (*)(void))func,ctx,dest,(PetscErrorCode (*)(void))tao->monitor[i],tao->monitorcontext[i],tao->monitordestroy[i],&identical);CHKERRQ(ierr);
+    if (identical) PetscFunctionReturn(0);
   }
   tao->monitor[tao->numbermonitors] = func;
-  tao->monitorcontext[tao->numbermonitors] = ctx;
+  tao->monitorcontext[tao->numbermonitors] = (void*)ctx;
   tao->monitordestroy[tao->numbermonitors] = dest;
   ++tao->numbermonitors;
   PetscFunctionReturn(0);
@@ -1485,8 +1553,9 @@ PetscErrorCode TaoMonitorDefault(Tao tao, void *ctx)
   gnorm=tao->residual;
   ierr = PetscViewerASCIIGetTab(viewer, &tabs);CHKERRQ(ierr);
   ierr = PetscViewerASCIISetTab(viewer, ((PetscObject)tao)->tablevel);CHKERRQ(ierr);
-  if (its == 0 && ((PetscObject)tao)->prefix) {
+  if (its == 0 && ((PetscObject)tao)->prefix && !tao->header_printed) {
      ierr = PetscViewerASCIIPrintf(viewer,"  Iteration information for %s solve.\n",((PetscObject)tao)->prefix);CHKERRQ(ierr);
+     tao->header_printed = PETSC_TRUE;
    }
   ierr=PetscViewerASCIIPrintf(viewer,"%3D TAO,",its);CHKERRQ(ierr);
   ierr=PetscViewerASCIIPrintf(viewer,"  Function value: %g,",(double)fct);CHKERRQ(ierr);
@@ -1499,6 +1568,60 @@ PetscErrorCode TaoMonitorDefault(Tao tao, void *ctx)
   PetscFunctionReturn(0);
 }
 
+/*@
+   TaoDefaultGMonitor - Default routine for monitoring progress of the
+   Tao solvers (default) with extra detail on the globalization method.
+   This monitor prints the function value and gradient norm at each
+   iteration, as well as the step size and trust radius. Note that the
+   step size and trust radius may be the same for some algorithms.
+   It can be turned on from the command line using the
+   -tao_gmonitor option
+
+   Collective on Tao
+
+   Input Parameters:
++  tao - the Tao context
+-  ctx - PetscViewer context or NULL
+
+   Options Database Keys:
+.  -tao_monitor
+
+   Level: advanced
+
+.seealso: TaoDefaultSMonitor(), TaoSetMonitor()
+@*/
+PetscErrorCode TaoDefaultGMonitor(Tao tao, void *ctx)
+{
+  PetscErrorCode ierr;
+  PetscInt       its, tabs;
+  PetscReal      fct,gnorm,stp,tr;
+  PetscViewer    viewer = (PetscViewer)ctx;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
+  its=tao->niter;
+  fct=tao->fc;
+  gnorm=tao->residual;
+  stp=tao->step;
+  tr=tao->trust;
+  ierr = PetscViewerASCIIGetTab(viewer, &tabs);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISetTab(viewer, ((PetscObject)tao)->tablevel);CHKERRQ(ierr);
+  if (its == 0 && ((PetscObject)tao)->prefix && !tao->header_printed) {
+     ierr = PetscViewerASCIIPrintf(viewer,"  Iteration information for %s solve.\n",((PetscObject)tao)->prefix);CHKERRQ(ierr);
+     tao->header_printed = PETSC_TRUE;
+   }
+  ierr=PetscViewerASCIIPrintf(viewer,"%3D TAO,",its);CHKERRQ(ierr);
+  ierr=PetscViewerASCIIPrintf(viewer,"  Function value: %g,",(double)fct);CHKERRQ(ierr);
+  if (gnorm >= PETSC_INFINITY) {
+    ierr=PetscViewerASCIIPrintf(viewer,"  Residual: Inf,");CHKERRQ(ierr);
+  } else {
+    ierr=PetscViewerASCIIPrintf(viewer,"  Residual: %g,",(double)gnorm);CHKERRQ(ierr);
+  }
+  ierr = PetscViewerASCIIPrintf(viewer,"  Step: %g,  Trust: %g\n",(double)stp,(double)tr);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISetTab(viewer, tabs);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    TaoDefaultSMonitor - Default routine for monitoring progress of the
    solver. Same as TaoMonitorDefault() except
@@ -1524,7 +1647,7 @@ PetscErrorCode TaoMonitorDefault(Tao tao, void *ctx)
 PetscErrorCode TaoDefaultSMonitor(Tao tao, void *ctx)
 {
   PetscErrorCode ierr;
-  PetscInt       its;
+  PetscInt       its, tabs;
   PetscReal      fct,gnorm;
   PetscViewer    viewer = (PetscViewer)ctx;
 
@@ -1533,6 +1656,8 @@ PetscErrorCode TaoDefaultSMonitor(Tao tao, void *ctx)
   its=tao->niter;
   fct=tao->fc;
   gnorm=tao->residual;
+  ierr = PetscViewerASCIIGetTab(viewer, &tabs);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISetTab(viewer, ((PetscObject)tao)->tablevel);CHKERRQ(ierr);
   ierr=PetscViewerASCIIPrintf(viewer,"iter = %3D,",its);CHKERRQ(ierr);
   ierr=PetscViewerASCIIPrintf(viewer," Function value %g,",(double)fct);CHKERRQ(ierr);
   if (gnorm >= PETSC_INFINITY) {
@@ -1544,6 +1669,7 @@ PetscErrorCode TaoDefaultSMonitor(Tao tao, void *ctx)
   } else {
     ierr=PetscViewerASCIIPrintf(viewer," Residual: < 1.0e-11 \n");CHKERRQ(ierr);
   }
+  ierr = PetscViewerASCIISetTab(viewer, tabs);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1568,7 +1694,7 @@ PetscErrorCode TaoDefaultSMonitor(Tao tao, void *ctx)
 PetscErrorCode TaoDefaultCMonitor(Tao tao, void *ctx)
 {
   PetscErrorCode ierr;
-  PetscInt       its;
+  PetscInt       its, tabs;
   PetscReal      fct,gnorm;
   PetscViewer    viewer = (PetscViewer)ctx;
 
@@ -1577,10 +1703,13 @@ PetscErrorCode TaoDefaultCMonitor(Tao tao, void *ctx)
   its=tao->niter;
   fct=tao->fc;
   gnorm=tao->residual;
+  ierr = PetscViewerASCIIGetTab(viewer, &tabs);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISetTab(viewer, ((PetscObject)tao)->tablevel);CHKERRQ(ierr);
   ierr=PetscViewerASCIIPrintf(viewer,"iter = %D,",its);CHKERRQ(ierr);
   ierr=PetscViewerASCIIPrintf(viewer," Function value: %g,",(double)fct);CHKERRQ(ierr);
   ierr=PetscViewerASCIIPrintf(viewer,"  Residual: %g ",(double)gnorm);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"  Constraint: %g \n",(double)tao->cnorm);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISetTab(viewer, tabs);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1605,7 +1734,7 @@ PetscErrorCode TaoDefaultCMonitor(Tao tao, void *ctx)
 PetscErrorCode TaoSolutionMonitor(Tao tao, void *ctx)
 {
   PetscErrorCode ierr;
-  PetscViewer    viewer  = (PetscViewer)ctx;;
+  PetscViewer    viewer  = (PetscViewer)ctx;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
@@ -1758,9 +1887,9 @@ PetscErrorCode TaoDrawStepMonitor(Tao tao, void *ctx)
 }
 
 /*@C
-   TaoSeparableObjectiveMonitor - Views the separable objective function at each iteration
+   TaoResidualMonitor - Views the least-squares residual at each iteration
    It can be turned on from the command line using the
-   -tao_view_separableobjective option
+   -tao_view_ls_residual option
 
    Collective on Tao
 
@@ -1769,20 +1898,20 @@ PetscErrorCode TaoDrawStepMonitor(Tao tao, void *ctx)
 -  ctx - PetscViewer context or NULL
 
    Options Database Keys:
-.  -tao_view_separableobjective
+.  -tao_view_ls_residual
 
    Level: advanced
 
 .seealso: TaoDefaultSMonitor(), TaoSetMonitor()
 @*/
-PetscErrorCode TaoSeparableObjectiveMonitor(Tao tao, void *ctx)
+PetscErrorCode TaoResidualMonitor(Tao tao, void *ctx)
 {
   PetscErrorCode ierr;
   PetscViewer    viewer  = (PetscViewer)ctx;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
-  ierr = VecView(tao->sep_objective,viewer);CHKERRQ(ierr);
+  ierr = VecView(tao->ls_res,viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1852,7 +1981,7 @@ PetscErrorCode TaoDefaultConvergenceTest(Tao tao,void *dummy)
   } else if (trradius < steptol && niter > 0){
     ierr = PetscInfo2(tao,"Trust region/step size too small: %g < %g\n", (double)trradius,(double)steptol);CHKERRQ(ierr);
     reason = TAO_CONVERGED_STEPTOL;
-  } else if (niter > tao->max_it) {
+  } else if (niter >= tao->max_it) {
     ierr = PetscInfo2(tao,"Exceeded maximum number of iterations: %D > %D\n",niter,tao->max_it);CHKERRQ(ierr);
     reason = TAO_DIVERGED_MAXITS;
   } else {
@@ -1958,7 +2087,8 @@ PetscErrorCode TaoAppendOptionsPrefix(Tao tao, const char p[])
   Output Parameters:
 . prefix - pointer to the prefix string used is returned
 
-  Notes: On the fortran side, the user should pass in a string 'prefix' of
+  Notes:
+    On the fortran side, the user should pass in a string 'prefix' of
   sufficient length to hold the prefix.
 
   Level: advanced
@@ -2000,7 +2130,7 @@ PetscErrorCode TaoGetOptionsPrefix(Tao tao, const char *p[])
 .seealso: TaoCreate(), TaoGetType(), TaoType
 
 @*/
-PetscErrorCode TaoSetType(Tao tao, const TaoType type)
+PetscErrorCode TaoSetType(Tao tao, TaoType type)
 {
   PetscErrorCode ierr;
   PetscErrorCode (*create_xxx)(Tao);
@@ -2013,7 +2143,7 @@ PetscErrorCode TaoSetType(Tao tao, const TaoType type)
   if (issame) PetscFunctionReturn(0);
 
   ierr = PetscFunctionListFind(TaoList, type, (void(**)(void))&create_xxx);CHKERRQ(ierr);
-  if (!create_xxx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested Tao type %s",type);
+  if (!create_xxx) SETERRQ1(PetscObjectComm((PetscObject)tao),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested Tao type %s",type);
 
   /* Destroy the existing solver information */
   if (tao->ops->destroy) {
@@ -2041,7 +2171,7 @@ PetscErrorCode TaoSetType(Tao tao, const TaoType type)
    TaoRegister - Adds a method to the TAO package for unconstrained minimization.
 
    Synopsis:
-   TaoRegister(char *name_solver,char *path,char *name_Create,int (*routine_Create)(Tao))
+   TaoRegister(char *name_solver,char *path,char *name_Create,PetscErrorCode (*routine_Create)(Tao))
 
    Not collective
 
@@ -2071,6 +2201,7 @@ PetscErrorCode TaoRegister(const char sname[], PetscErrorCode (*func)(Tao))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = TaoInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&TaoList,sname, (void (*)(void))func);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -2112,8 +2243,6 @@ PetscErrorCode TaoRegisterDestroy(void)
 
    Level: intermediate
 
-.keywords: Tao, nonlinear, get, iteration, number,
-
 .seealso:   TaoGetLinearSolveIterations(), TaoGetResidualNorm(), TaoGetObjective()
 @*/
 PetscErrorCode  TaoGetIterationNumber(Tao tao,PetscInt *iter)
@@ -2139,8 +2268,6 @@ PetscErrorCode  TaoGetIterationNumber(Tao tao,PetscInt *iter)
 
    Level: intermediate
 
-.keywords: Tao, nonlinear, get, iteration, number,
-
 .seealso:   TaoGetLinearSolveIterations(), TaoGetIterationNumber(), TaoGetResidualNorm()
 @*/
 PetscErrorCode  TaoGetObjective(Tao tao,PetscReal *value)
@@ -2169,8 +2296,6 @@ PetscErrorCode  TaoGetObjective(Tao tao,PetscReal *value)
    Developer Note: This is the 2-norm of the residual, we cannot use TaoGetGradientNorm() because that has
                    a different meaning. For some reason Tao sometimes calls the gradient the residual.
 
-.keywords: Tao, nonlinear, get, iteration, number,
-
 .seealso:   TaoGetLinearSolveIterations(), TaoGetIterationNumber(), TaoGetObjective()
 @*/
 PetscErrorCode  TaoGetResidualNorm(Tao tao,PetscReal *value)
@@ -2188,13 +2313,11 @@ PetscErrorCode  TaoGetResidualNorm(Tao tao,PetscReal *value)
    Not Collective
 
    Input Parameter:
-.  tao - Tao context
-.  iter - iteration number
++  tao - Tao context
+-  iter - iteration number
 
    Level: developer
 
-.keywords: Tao, nonlinear, set, iteration, number,
-
 .seealso:   TaoGetLinearSolveIterations()
 @*/
 PetscErrorCode  TaoSetIterationNumber(Tao tao,PetscInt iter)
@@ -2228,8 +2351,6 @@ PetscErrorCode  TaoSetIterationNumber(Tao tao,PetscInt iter)
 
    Level: intermediate
 
-.keywords: Tao, nonlinear, get, iteration, number,
-
 .seealso:   TaoGetLinearSolveIterations()
 @*/
 PetscErrorCode  TaoGetTotalIterationNumber(Tao tao,PetscInt *iter)
@@ -2247,13 +2368,11 @@ PetscErrorCode  TaoGetTotalIterationNumber(Tao tao,PetscInt *iter)
    Not Collective
 
    Input Parameter:
-.  tao - Tao context
-.  iter - iteration number
++  tao - Tao context
+-  iter - iteration number
 
    Level: developer
 
-.keywords: Tao, nonlinear, set, iteration, number,
-
 .seealso:   TaoGetLinearSolveIterations()
 @*/
 PetscErrorCode  TaoSetTotalIterationNumber(Tao tao,PetscInt iter)
@@ -2402,7 +2521,7 @@ PetscErrorCode TaoGetSolutionStatus(Tao tao, PetscInt *its, PetscReal *f, PetscR
    Level: intermediate
 
 @*/
-PetscErrorCode TaoGetType(Tao tao, const TaoType *type)
+PetscErrorCode TaoGetType(Tao tao,TaoType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao,TAO_CLASSID,1);
@@ -2450,7 +2569,7 @@ PetscErrorCode TaoMonitor(Tao tao, PetscInt its, PetscReal f, PetscReal res, Pet
   if (!its) {
     tao->cnorm0 = cnorm; tao->gnorm0 = res;
   }
-  if (PetscIsInfOrNanReal(f) || PetscIsInfOrNanReal(res)) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
+  if (PetscIsInfOrNanReal(f) || PetscIsInfOrNanReal(res)) SETERRQ(PetscObjectComm((PetscObject)tao),PETSC_ERR_USER, "User provided compute function generated Inf or NaN");
   for (i=0;i<tao->numbermonitors;i++) {
     ierr = (*tao->monitor[i])(tao,tao->monitorcontext[i]);CHKERRQ(ierr);
   }
@@ -2504,11 +2623,8 @@ PetscErrorCode TaoSetConvergenceHistory(Tao tao, PetscReal obj[], PetscReal resi
 
   if (na == PETSC_DECIDE || na == PETSC_DEFAULT) na = 1000;
   if (!obj && !resid && !cnorm && !lits) {
-    ierr = PetscCalloc1(na,&obj);CHKERRQ(ierr);
-    ierr = PetscCalloc1(na,&resid);CHKERRQ(ierr);
-    ierr = PetscCalloc1(na,&cnorm);CHKERRQ(ierr);
-    ierr = PetscCalloc1(na,&lits);CHKERRQ(ierr);
-    tao->hist_malloc=PETSC_TRUE;
+    ierr = PetscCalloc4(na,&obj,na,&resid,na,&cnorm,na,&lits);CHKERRQ(ierr);
+    tao->hist_malloc = PETSC_TRUE;
   }
 
   tao->hist_obj = obj;
@@ -2534,7 +2650,7 @@ PetscErrorCode TaoSetConvergenceHistory(Tao tao, PetscReal obj[], PetscReal resi
 .  resid - array used to hold residual history
 .  cnorm - array used to hold constraint violation history
 .  lits  - integer array used to hold linear solver iteration count
--  nhist  - size of obj, resid, cnorm, and lits (will be less than or equal to na given in TaoSetHistory)
+-  nhist  - size of obj, resid, cnorm, and lits
 
    Notes:
     This routine must be preceded by calls to TaoSetConvergenceHistory()
@@ -2559,7 +2675,7 @@ PetscErrorCode TaoGetConvergenceHistory(Tao tao, PetscReal **obj, PetscReal **re
   if (obj)   *obj   = tao->hist_obj;
   if (cnorm) *cnorm = tao->hist_cnorm;
   if (resid) *resid = tao->hist_resid;
-  if (nhist) *nhist   = tao->hist_len;
+  if (nhist) *nhist = tao->hist_len;
   PetscFunctionReturn(0);
 }
 
@@ -2713,8 +2829,6 @@ PetscErrorCode  TaoGradientNorm(Tao tao, Vec gradient, NormType type, PetscReal
 
    Level: intermediate
 
-.keywords: Tao,  vector, monitor, view
-
 .seealso: TaoMonitorSet(), TaoMonitorDefault(), VecView(), TaoMonitorDrawCtx()
 @*/
 PetscErrorCode  TaoMonitorDrawCtxCreate(MPI_Comm comm,const char host[],const char label[],int x,int y,int m,int n,PetscInt howoften,TaoMonitorDrawCtx *ctx)
@@ -2739,8 +2853,6 @@ PetscErrorCode  TaoMonitorDrawCtxCreate(MPI_Comm comm,const char host[],const ch
 
    Level: intermediate
 
-.keywords: Tao,  vector, monitor, view
-
 .seealso: TaoMonitorSet(), TaoMonitorDefault(), VecView(), TaoMonitorDrawSolution()
 @*/
 PetscErrorCode  TaoMonitorDrawCtxDestroy(TaoMonitorDrawCtx *ictx)
