@@ -5,26 +5,62 @@
 
 const char *const ISColoringTypes[] = {"global","ghosted","ISColoringType","IS_COLORING_",0};
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringReference"
 PetscErrorCode ISColoringReference(ISColoring coloring)
 {
   PetscFunctionBegin;
-  (coloring)->refct++;
+  coloring->refct++;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringSetType"
+/*@C
+
+    ISColoringSetType - indicates if the coloring is for the local representation (including ghost points) or the global representation
+
+   Collective on coloring
+
+   Input Parameters:
++    coloring - the coloring object
+-    type - either IS_COLORING_LOCAL or IS_COLORING_GLOBAL
+
+   Notes:
+     With IS_COLORING_LOCAL the coloring is in the numbering of the local vector, for IS_COLORING_GLOBAL it is in the number of the global vector
+
+   Level: intermediate
+
+.seealso: MatFDColoringCreate(), ISColoring, ISColoringCreate(), IS_COLORING_LOCAL, IS_COLORING_GLOBAL, ISColoringGetType()
+
+@*/
 PetscErrorCode ISColoringSetType(ISColoring coloring,ISColoringType type)
 {
   PetscFunctionBegin;
-  (coloring)->ctype = type;
+  coloring->ctype = type;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+
+    ISColoringGetType - gets if the coloring is for the local representation (including ghost points) or the global representation
+
+   Collective on coloring
+
+   Input Parameter:
+.   coloring - the coloring object
+
+   Output Parameter:
+.    type - either IS_COLORING_LOCAL or IS_COLORING_GLOBAL
+
+   Level: intermediate
+
+.seealso: MatFDColoringCreate(), ISColoring, ISColoringCreate(), IS_COLORING_LOCAL, IS_COLORING_GLOBAL, ISColoringSetType()
+
+@*/
+PetscErrorCode ISColoringGetType(ISColoring coloring,ISColoringType *type)
+{
+  PetscFunctionBegin;
+  *type = coloring->ctype;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringDestroy"
 /*@
    ISColoringDestroy - Destroys a coloring context.
 
@@ -59,10 +95,8 @@ PetscErrorCode  ISColoringDestroy(ISColoring *iscoloring)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringViewFromOptions"
 /*
-  ISColoringViewFromOptions - Processes command line options to determine if/how an ISColoring object is to be viewed. 
+  ISColoringViewFromOptions - Processes command line options to determine if/how an ISColoring object is to be viewed.
 
   Collective on ISColoring
 
@@ -86,7 +120,7 @@ PetscErrorCode ISColoringViewFromOptions(ISColoring obj,PetscObject bobj,const c
 
   PetscFunctionBegin;
   prefix = bobj ? bobj->prefix : NULL;
-  ierr   = PetscOptionsGetViewer(obj->comm,prefix,optionname,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr   = PetscOptionsGetViewer(obj->comm,NULL,prefix,optionname,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = ISColoringView(obj,viewer);CHKERRQ(ierr);
@@ -96,8 +130,6 @@ PetscErrorCode ISColoringViewFromOptions(ISColoring obj,PetscObject bobj,const c
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringView"
 /*@C
    ISColoringView - Views a coloring context.
 
@@ -134,29 +166,57 @@ PetscErrorCode  ISColoringView(ISColoring iscoloring,PetscViewer viewer)
     ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
     ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"ISColoring Object: %d MPI processes\n",size);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"ISColoringType: %s\n",ISColoringTypes[iscoloring->ctype]);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Number of colors %d\n",rank,iscoloring->n);CHKERRQ(ierr);
     ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
   }
 
-  ierr = ISColoringGetIS(iscoloring,PETSC_IGNORE,&is);CHKERRQ(ierr);
+  ierr = ISColoringGetIS(iscoloring,PETSC_USE_POINTER,PETSC_IGNORE,&is);CHKERRQ(ierr);
   for (i=0; i<iscoloring->n; i++) {
     ierr = ISView(iscoloring->is[i],viewer);CHKERRQ(ierr);
   }
-  ierr = ISColoringRestoreIS(iscoloring,&is);CHKERRQ(ierr);
+  ierr = ISColoringRestoreIS(iscoloring,PETSC_USE_POINTER,&is);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringGetIS"
 /*@C
-   ISColoringGetIS - Extracts index sets from the coloring context
+   ISColoringGetColors - Returns an array with the color for each node
+
+   Not Collective
+
+   Input Parameter:
+.  iscoloring - the coloring context
+
+   Output Parameters:
++  n - number of nodes
+.  nc - number of colors
+-  colors - color for each node
+
+   Level: advanced
+
+.seealso: ISColoringRestoreIS(), ISColoringView(), ISColoringGetIS()
+@*/
+PetscErrorCode  ISColoringGetColors(ISColoring iscoloring,PetscInt *n,PetscInt *nc,const ISColoringValue **colors)
+{
+  PetscFunctionBegin;
+  PetscValidPointer(iscoloring,1);
+
+  if (n) *n = iscoloring->N;
+  if (nc) *nc = iscoloring->n;
+  if (colors) *colors = iscoloring->colors;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   ISColoringGetIS - Extracts index sets from the coloring context. Each is contains the nodes of one color
 
    Collective on ISColoring
 
    Input Parameter:
-.  iscoloring - the coloring context
++  iscoloring - the coloring context
+-  mode - if this value is PETSC_OWN_POINTER then the caller owns the pointer and must free the array of IS and each IS in the array
 
    Output Parameters:
 +  nn - number of index sets in the coloring context
@@ -164,9 +224,9 @@ PetscErrorCode  ISColoringView(ISColoring iscoloring,PetscViewer viewer)
 
    Level: advanced
 
-.seealso: ISColoringRestoreIS(), ISColoringView()
+.seealso: ISColoringRestoreIS(), ISColoringView(), ISColoringGetColoring()
 @*/
-PetscErrorCode  ISColoringGetIS(ISColoring iscoloring,PetscInt *nn,IS *isis[])
+PetscErrorCode  ISColoringGetIS(ISColoring iscoloring,PetscCopyMode mode, PetscInt *nn,IS *isis[])
 {
   PetscErrorCode ierr;
 
@@ -193,13 +253,13 @@ PetscErrorCode  ISColoringGetIS(ISColoring iscoloring,PetscInt *nn,IS *isis[])
       ierr = PetscMalloc1(nc,&ii);CHKERRQ(ierr);
       ierr = PetscMalloc1(n,&ii[0]);CHKERRQ(ierr);
       for (i=1; i<nc; i++) ii[i] = ii[i-1] + mcolors[i-1];
-      ierr = PetscMemzero(mcolors,nc*sizeof(PetscInt));CHKERRQ(ierr);
+      ierr = PetscArrayzero(mcolors,nc);CHKERRQ(ierr);
 
       if (iscoloring->ctype == IS_COLORING_GLOBAL) {
         ierr = MPI_Scan(&iscoloring->N,&base,1,MPIU_INT,MPI_SUM,iscoloring->comm);CHKERRQ(ierr);
         base -= iscoloring->N;
         for (i=0; i<n; i++) ii[colors[i]][mcolors[colors[i]]++] = i + base; /* global idx */
-      } else if (iscoloring->ctype == IS_COLORING_GHOSTED) {
+      } else if (iscoloring->ctype == IS_COLORING_LOCAL) {
         for (i=0; i<n; i++) ii[colors[i]][mcolors[colors[i]]++] = i;   /* local idx */
       } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not provided for this ISColoringType type");
 
@@ -208,18 +268,18 @@ PetscErrorCode  ISColoringGetIS(ISColoring iscoloring,PetscInt *nn,IS *isis[])
         ierr = ISCreateGeneral(iscoloring->comm,mcolors[i],ii[i],PETSC_COPY_VALUES,is+i);CHKERRQ(ierr);
       }
 
-      iscoloring->is = is;
+      if (mode != PETSC_OWN_POINTER) iscoloring->is = is;
+      *isis = is;
       ierr = PetscFree(ii[0]);CHKERRQ(ierr);
       ierr = PetscFree(ii);CHKERRQ(ierr);
       ierr = PetscFree(mcolors);CHKERRQ(ierr);
+    } else {
+      *isis = iscoloring->is;
     }
-    *isis = iscoloring->is;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringRestoreIS"
 /*@C
    ISColoringRestoreIS - Restores the index sets extracted from the coloring context
 
@@ -227,13 +287,14 @@ PetscErrorCode  ISColoringGetIS(ISColoring iscoloring,PetscInt *nn,IS *isis[])
 
    Input Parameter:
 +  iscoloring - the coloring context
+.  mode - who retains ownership of the is
 -  is - array of index sets
 
    Level: advanced
 
 .seealso: ISColoringGetIS(), ISColoringView()
 @*/
-PetscErrorCode  ISColoringRestoreIS(ISColoring iscoloring,IS *is[])
+PetscErrorCode  ISColoringRestoreIS(ISColoring iscoloring,PetscCopyMode mode,IS *is[])
 {
   PetscFunctionBegin;
   PetscValidPointer(iscoloring,1);
@@ -243,13 +304,11 @@ PetscErrorCode  ISColoringRestoreIS(ISColoring iscoloring,IS *is[])
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringCreate"
 /*@
     ISColoringCreate - Generates an ISColoring context from lists (provided
     by each processor) of colors for each node.
 
-    Collective on MPI_Comm
+    Collective
 
     Input Parameters:
 +   comm - communicator for the processors creating the coloring
@@ -266,7 +325,8 @@ PetscErrorCode  ISColoringRestoreIS(ISColoring iscoloring,IS *is[])
 
    Level: advanced
 
-    Notes: By default sets coloring type to  IS_COLORING_GLOBAL
+    Notes:
+    By default sets coloring type to  IS_COLORING_GLOBAL
 
 .seealso: MatColoringCreate(), ISColoringView(), ISColoringDestroy(), ISColoringSetType()
 
@@ -320,7 +380,7 @@ PetscErrorCode  ISColoringCreate(MPI_Comm comm,PetscInt ncolors,PetscInt n,const
   if (mode == PETSC_COPY_VALUES) {
     ierr = PetscMalloc1(n,&(*iscoloring)->colors);CHKERRQ(ierr);
     ierr = PetscLogObjectMemory((PetscObject)(*iscoloring),n*sizeof(ISColoringValue));CHKERRQ(ierr);
-    ierr = PetscMemcpy((*iscoloring)->colors,colors,n*sizeof(ISColoringValue));CHKERRQ(ierr);
+    ierr = PetscArraycpy((*iscoloring)->colors,colors,n);CHKERRQ(ierr);
     (*iscoloring)->allocated = PETSC_TRUE;
   } else if (mode == PETSC_OWN_POINTER) {
     (*iscoloring)->colors    = (ISColoringValue*)colors;
@@ -334,17 +394,15 @@ PetscErrorCode  ISColoringCreate(MPI_Comm comm,PetscInt ncolors,PetscInt n,const
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBuildTwoSided"
 /*@
     ISBuildTwoSided - Takes an IS that describes where we will go. Generates an IS that contains new numbers from remote or local
     on the IS.
 
     Collective on IS
 
-    Input Parameters
-.   to - an IS describes where we will go. Negative target rank will be ignored
-.   toindx - an IS describes what indices should send. NULL means sending natural numbering
+    Input Parameters:
++   ito - an IS describes where we will go. Negative target rank will be ignored
+-   toindx - an IS describes what indices should send. NULL means sending natural numbering
 
     Output Parameter:
 .   rows - contains new numbers from remote or local
@@ -356,15 +414,15 @@ PetscErrorCode  ISColoringCreate(MPI_Comm comm,PetscInt ncolors,PetscInt n,const
 @*/
 PetscErrorCode  ISBuildTwoSided(IS ito,IS toindx, IS *rows)
 {
-   const PetscInt       *ito_indices,*toindx_indices;
-   PetscInt             *send_indices,rstart,*recv_indices,nrecvs,nsends;
-   PetscInt             *tosizes,*fromsizes,i,j,*tosizes_tmp,*tooffsets_tmp,ito_ln;
-   PetscMPIInt          *toranks,*fromranks,size,target_rank,*fromperm_newtoold,nto,nfrom;
-   PetscLayout           isrmap;
-   MPI_Comm              comm;
-   PetscSF               sf;
-   PetscSFNode          *iremote;
-   PetscErrorCode        ierr;
+   const PetscInt *ito_indices,*toindx_indices;
+   PetscInt       *send_indices,rstart,*recv_indices,nrecvs,nsends;
+   PetscInt       *tosizes,*fromsizes,i,j,*tosizes_tmp,*tooffsets_tmp,ito_ln;
+   PetscMPIInt    *toranks,*fromranks,size,target_rank,*fromperm_newtoold,nto,nfrom;
+   PetscLayout     isrmap;
+   MPI_Comm        comm;
+   PetscSF         sf;
+   PetscSFNode    *iremote;
+   PetscErrorCode  ierr;
 
    PetscFunctionBegin;
    ierr = PetscObjectGetComm((PetscObject)ito,&comm);CHKERRQ(ierr);
@@ -372,69 +430,65 @@ PetscErrorCode  ISBuildTwoSided(IS ito,IS toindx, IS *rows)
    ierr = ISGetLocalSize(ito,&ito_ln);CHKERRQ(ierr);
    /* why we do not have ISGetLayout? */
    isrmap = ito->map;
-   ierr = PetscLayoutGetRange(isrmap,&rstart,PETSC_NULL);CHKERRQ(ierr);
+   ierr = PetscLayoutGetRange(isrmap,&rstart,NULL);CHKERRQ(ierr);
    ierr = ISGetIndices(ito,&ito_indices);CHKERRQ(ierr);
    ierr = PetscCalloc2(size,&tosizes_tmp,size+1,&tooffsets_tmp);CHKERRQ(ierr);
-   for(i=0; i<ito_ln; i++){
-     if(ito_indices[i]<0) continue;
+   for (i=0; i<ito_ln; i++) {
+     if (ito_indices[i]<0) continue;
 #if defined(PETSC_USE_DEBUG)
-     if(ito_indices[i]>=size) SETERRQ2(comm,PETSC_ERR_ARG_OUTOFRANGE,"target rank %d is larger than communicator size %d ",ito_indices[i],size);
+     if (ito_indices[i]>=size) SETERRQ2(comm,PETSC_ERR_ARG_OUTOFRANGE,"target rank %d is larger than communicator size %d ",ito_indices[i],size);
 #endif
      tosizes_tmp[ito_indices[i]]++;
    }
    nto = 0;
-   for(i=0; i<size; i++){
-	 tooffsets_tmp[i+1] = tooffsets_tmp[i]+tosizes_tmp[i];
-     if(tosizes_tmp[i]>0) nto++;
-    }
+   for (i=0; i<size; i++) {
+     tooffsets_tmp[i+1] = tooffsets_tmp[i]+tosizes_tmp[i];
+     if (tosizes_tmp[i]>0) nto++;
+   }
    ierr = PetscCalloc2(nto,&toranks,2*nto,&tosizes);CHKERRQ(ierr);
-   nto = 0;
-   for(i=0; i<size; i++){
-     if(tosizes_tmp[i]>0){
-        toranks[nto]      = i;
-        tosizes[2*nto]    = tosizes_tmp[i];/* size */
-        tosizes[2*nto+1]  = tooffsets_tmp[i];/* offset */
-        nto++;
+   nto  = 0;
+   for (i=0; i<size; i++) {
+     if (tosizes_tmp[i]>0) {
+       toranks[nto]     = i;
+       tosizes[2*nto]   = tosizes_tmp[i];/* size */
+       tosizes[2*nto+1] = tooffsets_tmp[i];/* offset */
+       nto++;
      }
    }
    nsends = tooffsets_tmp[size];
-   ierr = PetscCalloc1(nsends,&send_indices);CHKERRQ(ierr);
-   if(toindx){
-	 ierr = ISGetIndices(toindx,&toindx_indices);CHKERRQ(ierr);
+   ierr   = PetscCalloc1(nsends,&send_indices);CHKERRQ(ierr);
+   if (toindx) {
+     ierr = ISGetIndices(toindx,&toindx_indices);CHKERRQ(ierr);
    }
-   for(i=0; i<ito_ln; i++){
-	 if(ito_indices[i]<0) continue;
-	 target_rank = ito_indices[i];
-	 send_indices[tooffsets_tmp[target_rank]] = toindx? toindx_indices[i]:(i+rstart);
-	 tooffsets_tmp[target_rank]++;
+   for (i=0; i<ito_ln; i++) {
+     if (ito_indices[i]<0) continue;
+     target_rank = ito_indices[i];
+     send_indices[tooffsets_tmp[target_rank]] = toindx? toindx_indices[i]:(i+rstart);
+     tooffsets_tmp[target_rank]++;
    }
-   if(toindx){
-   	 ierr = ISRestoreIndices(toindx,&toindx_indices);CHKERRQ(ierr);
+   if (toindx) {
+     ierr = ISRestoreIndices(toindx,&toindx_indices);CHKERRQ(ierr);
    }
    ierr = ISRestoreIndices(ito,&ito_indices);CHKERRQ(ierr);
    ierr = PetscFree2(tosizes_tmp,tooffsets_tmp);CHKERRQ(ierr);
    ierr = PetscCommBuildTwoSided(comm,2,MPIU_INT,nto,toranks,tosizes,&nfrom,&fromranks,&fromsizes);CHKERRQ(ierr);
    ierr = PetscFree2(toranks,tosizes);CHKERRQ(ierr);
-   ierr = PetscCalloc1(nfrom,&fromperm_newtoold);CHKERRQ(ierr);
-   for(i=0; i<nfrom; i++){
-	 fromperm_newtoold[i] = i;
-   }
-   ierr = PetscSortMPIIntWithArray(nfrom,fromranks,fromperm_newtoold);CHKERRQ(ierr);
-   nrecvs   = 0;
-   for(i=0; i<nfrom; i++){
-	 nrecvs += fromsizes[i*2];
-   }
-   ierr = PetscCalloc1(nrecvs,&recv_indices);CHKERRQ(ierr);
-   ierr = PetscCalloc1(nrecvs,&iremote);CHKERRQ(ierr);
+   ierr = PetscMalloc1(nfrom,&fromperm_newtoold);CHKERRQ(ierr);
+   for (i=0; i<nfrom; i++) fromperm_newtoold[i] = i;
+   ierr   = PetscSortMPIIntWithArray(nfrom,fromranks,fromperm_newtoold);CHKERRQ(ierr);
    nrecvs = 0;
-   for(i=0; i<nfrom; i++){
-     for(j=0; j<fromsizes[2*fromperm_newtoold[i]]; j++){
+   for (i=0; i<nfrom; i++) nrecvs += fromsizes[i*2];
+   ierr   = PetscCalloc1(nrecvs,&recv_indices);CHKERRQ(ierr);
+   ierr   = PetscMalloc1(nrecvs,&iremote);CHKERRQ(ierr);
+   nrecvs = 0;
+   for (i=0; i<nfrom; i++) {
+     for (j=0; j<fromsizes[2*fromperm_newtoold[i]]; j++) {
        iremote[nrecvs].rank    = fromranks[i];
        iremote[nrecvs++].index = fromsizes[2*fromperm_newtoold[i]+1]+j;
      }
    }
    ierr = PetscSFCreate(comm,&sf);CHKERRQ(ierr);
-   ierr = PetscSFSetGraph(sf,nsends,nrecvs,PETSC_NULL,PETSC_OWN_POINTER,iremote,PETSC_OWN_POINTER);CHKERRQ(ierr);
+   ierr = PetscSFSetGraph(sf,nsends,nrecvs,NULL,PETSC_OWN_POINTER,iremote,PETSC_OWN_POINTER);CHKERRQ(ierr);
    ierr = PetscSFSetType(sf,PETSCSFBASIC);CHKERRQ(ierr);
    /* how to put a prefix ? */
    ierr = PetscSFSetFromOptions(sf);CHKERRQ(ierr);
@@ -445,18 +499,16 @@ PetscErrorCode  ISBuildTwoSided(IS ito,IS toindx, IS *rows)
    ierr = PetscFree(fromsizes);CHKERRQ(ierr);
    ierr = PetscFree(fromperm_newtoold);CHKERRQ(ierr);
    ierr = PetscFree(send_indices);CHKERRQ(ierr);
-   if(rows){
-	 ierr = PetscSortInt(nrecvs,recv_indices);CHKERRQ(ierr);
-     ierr = ISCreateGeneral(comm, nrecvs,recv_indices,PETSC_OWN_POINTER,rows);CHKERRQ(ierr);
-   }else{
-	 ierr = PetscFree(recv_indices);CHKERRQ(ierr);
+   if (rows) {
+     ierr = PetscSortInt(nrecvs,recv_indices);CHKERRQ(ierr);
+     ierr = ISCreateGeneral(comm,nrecvs,recv_indices,PETSC_OWN_POINTER,rows);CHKERRQ(ierr);
+   } else {
+     ierr = PetscFree(recv_indices);CHKERRQ(ierr);
    }
    PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "ISPartitioningToNumbering"
 /*@
     ISPartitioningToNumbering - Takes an ISPartitioning and on each processor
     generates an IS that contains a new global node number for each index based
@@ -464,8 +516,9 @@ PetscErrorCode  ISBuildTwoSided(IS ito,IS toindx, IS *rows)
 
     Collective on IS
 
-    Input Parameters
+    Input Parameters:
 .   partitioning - a partitioning as generated by MatPartitioningApply()
+                   or MatPartitioningApplyND()
 
     Output Parameter:
 .   is - on each processor the index set that defines the global numbers
@@ -480,13 +533,23 @@ PetscErrorCode  ISBuildTwoSided(IS ito,IS toindx, IS *rows)
 PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
 {
   MPI_Comm       comm;
+  IS             ndorder;
   PetscInt       i,np,npt,n,*starts = NULL,*sums = NULL,*lsizes = NULL,*newi = NULL;
   const PetscInt *indices = NULL;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscObjectGetComm((PetscObject)part,&comm);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(part,IS_CLASSID,1);
+  PetscValidPointer(is,2);
+  /* see if the partitioning comes from nested dissection */
+  ierr = PetscObjectQuery((PetscObject)part,"_petsc_matpartitioning_ndorder",(PetscObject*)&ndorder);CHKERRQ(ierr);
+  if (ndorder) {
+    ierr = PetscObjectReference((PetscObject)ndorder);CHKERRQ(ierr);
+    *is  = ndorder;
+    PetscFunctionReturn(0);
+  }
 
+  ierr = PetscObjectGetComm((PetscObject)part,&comm);CHKERRQ(ierr);
   /* count the number of partitions, i.e., virtual processors */
   ierr = ISGetLocalSize(part,&n);CHKERRQ(ierr);
   ierr = ISGetIndices(part,&indices);CHKERRQ(ierr);
@@ -501,7 +564,7 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
         starts - global number of first element in each partition on this processor
   */
   ierr = PetscMalloc3(np,&lsizes,np,&starts,np,&sums);CHKERRQ(ierr);
-  ierr = PetscMemzero(lsizes,np*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscArrayzero(lsizes,np);CHKERRQ(ierr);
   for (i=0; i<n; i++) lsizes[indices[i]]++;
   ierr = MPIU_Allreduce(lsizes,sums,np,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
   ierr = MPI_Scan(lsizes,starts,np,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
@@ -524,8 +587,6 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISPartitioningCount"
 /*@
     ISPartitioningCount - Takes a ISPartitioning and determines the number of
     resulting elements on each (partition) process
@@ -533,7 +594,8 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
     Collective on IS
 
     Input Parameters:
-+   partitioning - a partitioning as generated by MatPartitioningApply()
++   partitioning - a partitioning as generated by MatPartitioningApply() or
+                   MatPartitioningApplyND()
 -   len - length of the array count, this is the total number of partitions
 
     Output Parameter:
@@ -548,10 +610,11 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
         of count) is the size of the communicator associated with IS,
         but it can be set by MatPartitioningSetNParts. The resulting array
         of lengths can for instance serve as input of PCBJacobiSetTotalBlocks.
-
+        If the partitioning has been obtained by MatPartitioningApplyND(),
+        the returned count does not include the separators.
 
 .seealso: MatPartitioningCreate(), AOCreateBasic(), ISPartitioningToNumbering(),
-        MatPartitioningSetNParts(), MatPartitioningApply()
+        MatPartitioningSetNParts(), MatPartitioningApply(), MatPartitioningApplyND()
 
 @*/
 PetscErrorCode  ISPartitioningCount(IS part,PetscInt len,PetscInt count[])
@@ -589,7 +652,9 @@ PetscErrorCode  ISPartitioningCount(IS part,PetscInt len,PetscInt count[])
         starts - global number of first element in each partition on this processor
   */
   ierr = PetscCalloc1(len,&lsizes);CHKERRQ(ierr);
-  for (i=0; i<n; i++) lsizes[indices[i]]++;
+  for (i=0; i<n; i++) {
+    if (indices[i] > -1) lsizes[indices[i]]++;
+  }
   ierr = ISRestoreIndices(part,&indices);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(len,&npp);CHKERRQ(ierr);
   ierr = MPIU_Allreduce(lsizes,count,npp,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
@@ -597,8 +662,6 @@ PetscErrorCode  ISPartitioningCount(IS part,PetscInt len,PetscInt count[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISAllGather"
 /*@
     ISAllGather - Given an index set (IS) on each processor, generates a large
     index set (same on each processor) by concatenating together each
@@ -625,10 +688,6 @@ PetscErrorCode  ISPartitioningCount(IS part,PetscInt len,PetscInt count[])
 
     Level: intermediate
 
-    Concepts: gather^index sets
-    Concepts: index sets^gathering to all processors
-    Concepts: IS^gathering to all processors
-
 .seealso: ISCreateGeneral(), ISCreateStride(), ISCreateBlock()
 @*/
 PetscErrorCode  ISAllGather(IS is,IS *isout)
@@ -674,13 +733,11 @@ PetscErrorCode  ISAllGather(IS is,IS *isout)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISAllGatherColors"
 /*@C
     ISAllGatherColors - Given a a set of colors on each processor, generates a large
     set (same on each processor) by concatenating together each processors colors
 
-    Collective on MPI_Comm
+    Collective
 
     Input Parameter:
 +   comm - communicator to share the indices
@@ -697,10 +754,6 @@ PetscErrorCode  ISAllGather(IS is,IS *isout)
 
     Level: intermediate
 
-    Concepts: gather^index sets
-    Concepts: index sets^gathering to all processors
-    Concepts: IS^gathering to all processors
-
 .seealso: ISCreateGeneral(), ISCreateStride(), ISCreateBlock(), ISAllGather()
 @*/
 PetscErrorCode  ISAllGatherColors(MPI_Comm comm,PetscInt n,ISColoringValue *lindices,PetscInt *outN,ISColoringValue *outindices[])
@@ -728,8 +781,6 @@ PetscErrorCode  ISAllGatherColors(MPI_Comm comm,PetscInt n,ISColoringValue *lind
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISComplement"
 /*@
     ISComplement - Given an index set (IS) generates the complement index set. That is all
        all indices that are NOT in the given set.
@@ -744,7 +795,8 @@ PetscErrorCode  ISAllGatherColors(MPI_Comm comm,PetscInt n,ISColoringValue *lind
     Output Parameter:
 .   isout - the complement
 
-    Notes:  The communicator for this new IS is the same as for the input IS
+    Notes:
+    The communicator for this new IS is the same as for the input IS
 
       For a parallel IS, this will generate the local part of the complement on each process
 
@@ -753,10 +805,6 @@ PetscErrorCode  ISAllGatherColors(MPI_Comm comm,PetscInt n,ISColoringValue *lind
 
     Level: intermediate
 
-    Concepts: gather^index sets
-    Concepts: index sets^gathering to all processors
-    Concepts: IS^gathering to all processors
-
 .seealso: ISCreateGeneral(), ISCreateStride(), ISCreateBlock(), ISAllGather()
 @*/
 PetscErrorCode  ISComplement(IS is,PetscInt nmin,PetscInt nmax,IS *isout)
