@@ -218,7 +218,8 @@ PetscErrorCode VecStashExpand_Private(VecStash *stash,PetscInt incr)
   owners - an array of size 'no-of-procs' which gives the ownership range
            for each node.
 
-  Notes: The 'owners' array in the cased of the blocked-stash has the
+  Notes:
+    The 'owners' array in the cased of the blocked-stash has the
   ranges specified blocked global indices, and for the regular stash in
   the proper global indices.
 */
@@ -413,7 +414,7 @@ PetscErrorCode VecStashSortCompress_Private(VecStash *stash)
     ierr = PetscSortIntWithArray(stash->n,stash->idx,perm);CHKERRQ(ierr);
 
     /* Out-of-place copy of arr */
-    ierr = PetscMemcpy(arr,stash->array,bs*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscMemcpy(arr,stash->array+perm[0]*bs,bs*sizeof(PetscScalar));CHKERRQ(ierr);
     for (i=1,j=0; i<stash->n; i++) {
       PetscInt k;
       if (stash->idx[i] == stash->idx[j]) {
@@ -441,9 +442,10 @@ PetscErrorCode VecStashSortCompress_Private(VecStash *stash)
 
 PetscErrorCode VecStashGetOwnerList_Private(VecStash *stash,PetscLayout map,PetscMPIInt *nowners,PetscMPIInt **owners)
 {
-  PetscErrorCode ierr;
-  PetscInt i,bs = stash->bs,r;
+  PetscInt       i,bs = stash->bs;
+  PetscMPIInt    r;
   PetscSegBuffer seg;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (bs != 1 && bs != map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Stash block size %D does not match layout block size %D",bs,map->bs);
